<?php
/*  Copyright (C) 2018      Open-DSI             <support@open-dsi.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *	\file       requestmanager/class/html.formrequestmanager.class.php
 *  \ingroup    requestmanager
 *	\brief      File of class with all html predefined components for request manager
 */

require_once DOL_DOCUMENT_ROOT . '/core/class/html.form.class.php';
dol_include_once('/requestmanager/class/requestmanager.class.php');
dol_include_once('/advancedictionaries/class/html.formdictionary.class.php');

/**
 *	Class to manage generation of HTML components
 *	Only common components for request manager must be here.
 *
 */
class FormRequestManager
{
    public $db;
    public $error;
    public $num;

    /**
     * @var Form  Instance of the form
     */
    public $form;

    /**
     * @var FormDictionary  Instance of the form form dictionaries
     */
    public $formdictionary;



    /**
     * Constructor
     *
     * @param   DoliDB $db Database handler
     */
    public function __construct($db)
    {
        $this->db = $db;
        $this->form = new Form($this->db);
        $this->formdictionary = new FormDictionary($this->db);
    }

    /**
     *  Output html form to select a request type for user groups
     *
     * @param   string      $selected               Preselected type
     * @param   string      $htmlname               Name of field in form
     * @param   string      $showempty              Add an empty field (Can be '1' or text key to use on empty line like 'SelectThirdParty')
     * @param   int         $forcecombo             Force to use combo box
     * @param   array       $events                 Ajax event options to run on change. Example: array(array('method'=>'getContacts', 'url'=>dol_buildpath('/core/ajax/contacts.php',1), 'htmlname'=>'contactid', 'params'=>array('add-customer-contact'=>'disabled')))
     * @param  	int		    $usesearchtoselect	    Minimum length of input string to start autocomplete
     * @param   int         $limit                  Maximum number of elements
     * @param   string      $morecss                Add more css styles to the SELECT component
     * @param   string      $moreparam              Add more parameters onto the select tag. For example 'style="width: 95%"' to avoid select2 component to go over parent container
     * @param   string      $selected_input_value   Value of preselected input text (for use with ajax)
     * @param   int         $hidelabel              Hide label (0=no, 1=yes, 2=show search icon (before) and placeholder, 3 search icon after)
     * @param   string      $selectlabel            Text of the label (can be translated)
     * @param   int         $autofocus              Autofocus the field in form (1 auto focus, 0 not)
     * @param   array       $ajaxoptions            Options for ajax_autocompleter
     * @param   bool        $options_only           Return options only (for ajax treatment)
     * @return  string                              HTML string with select box for request type.
     */
	public function select_type($usergroups, $selected = '', $htmlname = 'type', $showempty = '', $forcecombo = 0, $events = array(), $usesearchtoselect=0, $limit = 0, $morecss = 'minwidth100', $moreparam = '', $selected_input_value = '', $hidelabel = 1, $selectlabel = '', $autofocus=0, $ajaxoptions = array(), $options_only=false)
    {
        global $langs;

        $out = $this->formdictionary->select_dictionary('requestmanager', 'requestmanagerrequesttype', $selected, $htmlname, $showempty, 'rowid', '{{label}}', is_array($usergroups) ? array('user_group'=>$usergroups) : array(), array('label'=>'ASC'), $forcecombo, $events, $usesearchtoselect, $limit, $morecss, $moreparam, $selected_input_value, $hidelabel, $selectlabel, $autofocus, $ajaxoptions, $options_only);
        if ($out < 0 && is_int($out)) {
            return $langs->trans('RequestManagerDontHaveRequestTypeForYoursGroup');
        }

        return $out;
    }

    /**
     *  Output html form to select a category for a request type
     *
     * @param   int         $request_type           Id of the request type
     * @param   string      $selected               Preselected type
     * @param   string      $htmlname               Name of field in form
     * @param   string      $showempty              Add an empty field (Can be '1' or text key to use on empty line like 'SelectThirdParty')
     * @param   int         $forcecombo             Force to use combo box
     * @param   array       $events                 Ajax event options to run on change. Example: array(array('method'=>'getContacts', 'url'=>dol_buildpath('/core/ajax/contacts.php',1), 'htmlname'=>'contactid', 'params'=>array('add-customer-contact'=>'disabled')))
     * @param  	int		    $usesearchtoselect	    Minimum length of input string to start autocomplete
     * @param   int         $limit                  Maximum number of elements
     * @param   string      $morecss                Add more css styles to the SELECT component
     * @param   string      $moreparam              Add more parameters onto the select tag. For example 'style="width: 95%"' to avoid select2 component to go over parent container
     * @param   string      $selected_input_value   Value of preselected input text (for use with ajax)
     * @param   int         $hidelabel              Hide label (0=no, 1=yes, 2=show search icon (before) and placeholder, 3 search icon after)
     * @param   string      $selectlabel            Text of the label (can be translated)
     * @param   int         $autofocus              Autofocus the field in form (1 auto focus, 0 not)
     * @param   array       $ajaxoptions            Options for ajax_autocompleter
     * @param   bool        $options_only           Return options only (for ajax treatment)
     * @return  string                              HTML string with select box for category.
     */
	public function select_category($request_type, $selected = '', $htmlname = 'category', $showempty = '', $forcecombo = 0, $events = array(), $usesearchtoselect=0, $limit = 0, $morecss = 'minwidth100', $moreparam = '', $selected_input_value = '', $hidelabel = 1, $selectlabel = '', $autofocus=0, $ajaxoptions = array(), $options_only=false)
    {
        $categories_id = array();
        if ($request_type > 0) {
            // Get request type
            dol_include_once('/advancedictionaries/class/dictionary.class.php');
            $dictionaryLine = Dictionary::getDictionaryLine($this->db, 'requestmanager', 'requestmanagerrequesttype');
            $dictionaryLine->fetch($request_type);
            $categories_id = explode(',', $dictionaryLine->fields['category']);
            unset($dictionaryLine);
        }

        return $this->formdictionary->select_dictionary('requestmanager', 'requestmanagercategory', $selected, $htmlname, $showempty, 'rowid', '{{label}}', array('rowid' => $categories_id), array('label'=>'ASC'), $forcecombo, $events, $usesearchtoselect, $limit, $morecss, $moreparam, $selected_input_value, $hidelabel, $selectlabel, $autofocus, $ajaxoptions, $options_only);
    }

    /**
     *  Output html form to select a source
     *
     * @param   string      $selected               Preselected type
     * @param   string      $htmlname               Name of field in form
     * @param   string      $showempty              Add an empty field (Can be '1' or text key to use on empty line like 'SelectThirdParty')
     * @param   int         $forcecombo             Force to use combo box
     * @param   array       $events                 Ajax event options to run on change. Example: array(array('method'=>'getContacts', 'url'=>dol_buildpath('/core/ajax/contacts.php',1), 'htmlname'=>'contactid', 'params'=>array('add-customer-contact'=>'disabled')))
     * @param  	int		    $usesearchtoselect	    Minimum length of input string to start autocomplete
     * @param   int         $limit                  Maximum number of elements
     * @param   string      $morecss                Add more css styles to the SELECT component
     * @param   string      $moreparam              Add more parameters onto the select tag. For example 'style="width: 95%"' to avoid select2 component to go over parent container
     * @param   string      $selected_input_value   Value of preselected input text (for use with ajax)
     * @param   int         $hidelabel              Hide label (0=no, 1=yes, 2=show search icon (before) and placeholder, 3 search icon after)
     * @param   string      $selectlabel            Text of the label (can be translated)
     * @param   int         $autofocus              Autofocus the field in form (1 auto focus, 0 not)
     * @param   array       $ajaxoptions            Options for ajax_autocompleter
     * @param   bool        $options_only           Return options only (for ajax treatment)
     * @return  string                              HTML string with select box for source.
     */
	public function select_source($selected = '', $htmlname = 'source', $showempty = '', $forcecombo = 0, $events = array(), $usesearchtoselect=0, $limit = 0, $morecss = 'minwidth100', $moreparam = '', $selected_input_value = '', $hidelabel = 1, $selectlabel = '', $autofocus=0, $ajaxoptions = array(), $options_only=false)
    {
        return $this->formdictionary->select_dictionary('requestmanager', 'requestmanagersource', $selected, $htmlname, $showempty, 'rowid', '{{label}}', array(), array('label'=>'ASC'), $forcecombo, $events, $usesearchtoselect, $limit, $morecss, $moreparam, $selected_input_value, $hidelabel, $selectlabel, $autofocus, $ajaxoptions, $options_only);
    }

    /**
     *  Output html form to select a urgency
     *
     * @param   string      $selected               Preselected type
     * @param   string      $htmlname               Name of field in form
     * @param   string      $showempty              Add an empty field (Can be '1' or text key to use on empty line like 'SelectThirdParty')
     * @param   int         $forcecombo             Force to use combo box
     * @param   array       $events                 Ajax event options to run on change. Example: array(array('method'=>'getContacts', 'url'=>dol_buildpath('/core/ajax/contacts.php',1), 'htmlname'=>'contactid', 'params'=>array('add-customer-contact'=>'disabled')))
     * @param  	int		    $usesearchtoselect	    Minimum length of input string to start autocomplete
     * @param   int         $limit                  Maximum number of elements
     * @param   string      $morecss                Add more css styles to the SELECT component
     * @param   string      $moreparam              Add more parameters onto the select tag. For example 'style="width: 95%"' to avoid select2 component to go over parent container
     * @param   string      $selected_input_value   Value of preselected input text (for use with ajax)
     * @param   int         $hidelabel              Hide label (0=no, 1=yes, 2=show search icon (before) and placeholder, 3 search icon after)
     * @param   string      $selectlabel            Text of the label (can be translated)
     * @param   int         $autofocus              Autofocus the field in form (1 auto focus, 0 not)
     * @param   array       $ajaxoptions            Options for ajax_autocompleter
     * @param   bool        $options_only           Return options only (for ajax treatment)
     * @return  string                              HTML string with select box for urgency.
     */
	public function select_urgency($selected = '', $htmlname = 'urgency', $showempty = '', $forcecombo = 0, $events = array(), $usesearchtoselect=0, $limit = 0, $morecss = 'minwidth100', $moreparam = '', $selected_input_value = '', $hidelabel = 1, $selectlabel = '', $autofocus=0, $ajaxoptions = array(), $options_only=false)
    {
        return $this->formdictionary->select_dictionary('requestmanager', 'requestmanagerurgency', $selected, $htmlname, $showempty, 'rowid', '{{label}}', array(), array('label'=>'ASC'), $forcecombo, $events, $usesearchtoselect, $limit, $morecss, $moreparam, $selected_input_value, $hidelabel, $selectlabel, $autofocus, $ajaxoptions, $options_only);
    }

    /**
     *  Output html form to select a impact
     *
     * @param   string      $selected               Preselected type
     * @param   string      $htmlname               Name of field in form
     * @param   string      $showempty              Add an empty field (Can be '1' or text key to use on empty line like 'SelectThirdParty')
     * @param   int         $forcecombo             Force to use combo box
     * @param   array       $events                 Ajax event options to run on change. Example: array(array('method'=>'getContacts', 'url'=>dol_buildpath('/core/ajax/contacts.php',1), 'htmlname'=>'contactid', 'params'=>array('add-customer-contact'=>'disabled')))
     * @param  	int		    $usesearchtoselect	    Minimum length of input string to start autocomplete
     * @param   int         $limit                  Maximum number of elements
     * @param   string      $morecss                Add more css styles to the SELECT component
     * @param   string      $moreparam              Add more parameters onto the select tag. For example 'style="width: 95%"' to avoid select2 component to go over parent container
     * @param   string      $selected_input_value   Value of preselected input text (for use with ajax)
     * @param   int         $hidelabel              Hide label (0=no, 1=yes, 2=show search icon (before) and placeholder, 3 search icon after)
     * @param   string      $selectlabel            Text of the label (can be translated)
     * @param   int         $autofocus              Autofocus the field in form (1 auto focus, 0 not)
     * @param   array       $ajaxoptions            Options for ajax_autocompleter
     * @param   bool        $options_only           Return options only (for ajax treatment)
     * @return  string                              HTML string with select box for impact.
     */
	public function select_impact($selected = '', $htmlname = 'impact', $showempty = '', $forcecombo = 0, $events = array(), $usesearchtoselect=0, $limit = 0, $morecss = 'minwidth100', $moreparam = '', $selected_input_value = '', $hidelabel = 1, $selectlabel = '', $autofocus=0, $ajaxoptions = array(), $options_only=false)
    {
        return $this->formdictionary->select_dictionary('requestmanager', 'requestmanagerimpact', $selected, $htmlname, $showempty, 'rowid', '{{label}}', array(), array('label'=>'ASC'), $forcecombo, $events, $usesearchtoselect, $limit, $morecss, $moreparam, $selected_input_value, $hidelabel, $selectlabel, $autofocus, $ajaxoptions, $options_only);
    }

    /**
     *  Output html form to select a priority
     *
     * @param   string      $selected               Preselected type
     * @param   string      $htmlname               Name of field in form
     * @param   string      $showempty              Add an empty field (Can be '1' or text key to use on empty line like 'SelectThirdParty')
     * @param   int         $forcecombo             Force to use combo box
     * @param   array       $events                 Ajax event options to run on change. Example: array(array('method'=>'getContacts', 'url'=>dol_buildpath('/core/ajax/contacts.php',1), 'htmlname'=>'contactid', 'params'=>array('add-customer-contact'=>'disabled')))
     * @param  	int		    $usesearchtoselect	    Minimum length of input string to start autocomplete
     * @param   int         $limit                  Maximum number of elements
     * @param   string      $morecss                Add more css styles to the SELECT component
     * @param   string      $moreparam              Add more parameters onto the select tag. For example 'style="width: 95%"' to avoid select2 component to go over parent container
     * @param   string      $selected_input_value   Value of preselected input text (for use with ajax)
     * @param   int         $hidelabel              Hide label (0=no, 1=yes, 2=show search icon (before) and placeholder, 3 search icon after)
     * @param   string      $selectlabel            Text of the label (can be translated)
     * @param   int         $autofocus              Autofocus the field in form (1 auto focus, 0 not)
     * @param   array       $ajaxoptions            Options for ajax_autocompleter
     * @param   bool        $options_only           Return options only (for ajax treatment)
     * @return  string                              HTML string with select box for priority.
     */
	public function select_priority($selected = '', $htmlname = 'priority', $showempty = '', $forcecombo = 0, $events = array(), $usesearchtoselect=0, $limit = 0, $morecss = 'minwidth100', $moreparam = '', $selected_input_value = '', $hidelabel = 1, $selectlabel = '', $autofocus=0, $ajaxoptions = array(), $options_only=false)
    {
        return $this->formdictionary->select_dictionary('requestmanager', 'requestmanagerpriority', $selected, $htmlname, $showempty, 'rowid', '{{label}}', array(), array('label'=>'ASC'), $forcecombo, $events, $usesearchtoselect, $limit, $morecss, $moreparam, $selected_input_value, $hidelabel, $selectlabel, $autofocus, $ajaxoptions, $options_only);
    }

    /**
     *  Output html form to select a status
     *
     * @param   int         $request_type           Id of the request type
     * @param   string      $selected               Preselected type
     * @param   string      $htmlname               Name of field in form
     * @param   string      $showempty              Add an empty field (Can be '1' or text key to use on empty line like 'SelectThirdParty')
     * @param   int         $forcecombo             Force to use combo box
     * @param   array       $events                 Ajax event options to run on change. Example: array(array('method'=>'getContacts', 'url'=>dol_buildpath('/core/ajax/contacts.php',1), 'htmlname'=>'contactid', 'params'=>array('add-customer-contact'=>'disabled')))
     * @param  	int		    $usesearchtoselect	    Minimum length of input string to start autocomplete
     * @param   int         $limit                  Maximum number of elements
     * @param   string      $morecss                Add more css styles to the SELECT component
     * @param   string      $moreparam              Add more parameters onto the select tag. For example 'style="width: 95%"' to avoid select2 component to go over parent container
     * @param   string      $selected_input_value   Value of preselected input text (for use with ajax)
     * @param   int         $hidelabel              Hide label (0=no, 1=yes, 2=show search icon (before) and placeholder, 3 search icon after)
     * @param   string      $selectlabel            Text of the label (can be translated)
     * @param   int         $autofocus              Autofocus the field in form (1 auto focus, 0 not)
     * @param   array       $ajaxoptions            Options for ajax_autocompleter
     * @param   bool        $options_only           Return options only (for ajax treatment)
     * @return  string                              HTML string with select box for status.
     */
	public function select_status($request_type, $selected = '', $htmlname = 'status', $showempty = '', $forcecombo = 0, $events = array(), $usesearchtoselect=0, $limit = 0, $morecss = 'minwidth100', $moreparam = '', $selected_input_value = '', $hidelabel = 1, $selectlabel = '', $autofocus=0, $ajaxoptions = array(), $options_only=false)
    {
        return $this->formdictionary->select_dictionary('requestmanager', 'requestmanagerstatus', $selected, $htmlname, $showempty, 'rowid', '{{label}}', array('request_type'=>array($request_type), 'type'=>array(RequestManager::STATUS_TYPE_IN_PROGRESS)), array('label'=>'ASC'), $forcecombo, $events, $usesearchtoselect, $limit, $morecss, $moreparam, $selected_input_value, $hidelabel, $selectlabel, $autofocus, $ajaxoptions, $options_only);
    }

    /**
     *  Output html form to select a reason for resolution
     *
     * @param   int         $request_type           Id of the request type
     * @param   string      $selected               Preselected type
     * @param   string      $htmlname               Name of field in form
     * @param   string      $showempty              Add an empty field (Can be '1' or text key to use on empty line like 'SelectThirdParty')
     * @param   int         $forcecombo             Force to use combo box
     * @param   array       $events                 Ajax event options to run on change. Example: array(array('method'=>'getContacts', 'url'=>dol_buildpath('/core/ajax/contacts.php',1), 'htmlname'=>'contactid', 'params'=>array('add-customer-contact'=>'disabled')))
     * @param  	int		    $usesearchtoselect	    Minimum length of input string to start autocomplete
     * @param   int         $limit                  Maximum number of elements
     * @param   string      $morecss                Add more css styles to the SELECT component
     * @param   string      $moreparam              Add more parameters onto the select tag. For example 'style="width: 95%"' to avoid select2 component to go over parent container
     * @param   string      $selected_input_value   Value of preselected input text (for use with ajax)
     * @param   int         $hidelabel              Hide label (0=no, 1=yes, 2=show search icon (before) and placeholder, 3 search icon after)
     * @param   string      $selectlabel            Text of the label (can be translated)
     * @param   int         $autofocus              Autofocus the field in form (1 auto focus, 0 not)
     * @param   array       $ajaxoptions            Options for ajax_autocompleter
     * @param   bool        $options_only           Return options only (for ajax treatment)
     * @return  string                              HTML string with select box for status.
     */
	public function select_reason_resolution($request_type, $selected = '', $htmlname = 'reason_resolution', $showempty = '', $forcecombo = 0, $events = array(), $usesearchtoselect=0, $limit = 0, $morecss = 'minwidth100', $moreparam = '', $selected_input_value = '', $hidelabel = 1, $selectlabel = '', $autofocus=0, $ajaxoptions = array(), $options_only=false)
    {
        $reasons_resolutions = '';
        dol_include_once('/advancedictionaries/class/dictionary.class.php');
        $statusDictionary = Dictionary::getDictionary($this->db, 'requestmanager', 'requestmanagerstatus');
        $statusDictionary->fetch_lines(1, array('request_type'=>array($request_type), 'type'=>array(RequestManager::STATUS_TYPE_RESOLVED)));
        if (count($statusDictionary->lines) == 1) {
            $lines = array_values($statusDictionary->lines);
            $reasons_resolutions = $lines[0]->fields['reason_resolution'];
        }
        $reasons_resolutions = isset($reasons_resolutions) ? $reasons_resolutions : '';
        $reasons_resolutions = array_filter(array_map('trim', explode(',', $reasons_resolutions)), 'strlen');

        return $this->formdictionary->select_dictionary('requestmanager', 'requestmanagerreasonresolution', $selected, $htmlname, $showempty, 'rowid', '{{label}}', count($reasons_resolutions) ? array('rowid' => $reasons_resolutions) : null, array('label'=>'ASC'), $forcecombo, $events, $usesearchtoselect, $limit, $morecss, $moreparam, $selected_input_value, $hidelabel, $selectlabel, $autofocus, $ajaxoptions, $options_only);
    }

    /**
     *	Return multiselect list of product categories
     *
     *	@param	string	$htmlname		Name of select
     *	@param	array	$selected		Array with key+value preselected
     *	@param	int		$key_in_label   1 pour afficher la key dans la valeur "[key] value"
     *	@param	int		$value_as_key   1 to use value as key
     *	@param  string	$morecss        Add more css style
     *	@param  int		$translate		Translate and encode value
     *  @param	int		$width			Force width of select box. May be used only when using jquery couch. Example: 250, 95%
     *  @param	string	$moreattrib		Add more options on select component. Example: 'disabled'
     *  @param	string	$elemtype		Type of element we show ('category', ...)
     *	@return	string					HTML multiselect string
     *  @see selectarray
     */
	public function multiselect_categories($selected=array(), $htmlname='categories', $key_in_label=0, $value_as_key=0, $morecss='', $translate=0, $width=0, $moreattrib='',$elemtype='')
    {
        global $conf;

        include_once DOL_DOCUMENT_ROOT . '/categories/class/categorie.class.php';

        $cat = new Categorie($this->db);
        $cate_arbo = $cat->get_full_arbo(Categorie::TYPE_PRODUCT);

        $list = array();
        foreach ($cate_arbo as $k => $cat) {
            if (((preg_match('/^'.$conf->global->REQUESTMANAGER_ROOT_PRODUCT_CATEGORIES.'$/', $cat['fullpath']) ||
                preg_match('/_'.$conf->global->REQUESTMANAGER_ROOT_PRODUCT_CATEGORIES.'$/', $cat['fullpath'])) && $conf->global->REQUESTMANAGER_ROOT_PRODUCT_CATEGORY_INCLUDE) ||
                preg_match('/^'.$conf->global->REQUESTMANAGER_ROOT_PRODUCT_CATEGORIES.'_/', $cat['fullpath']) ||
                preg_match('/_'.$conf->global->REQUESTMANAGER_ROOT_PRODUCT_CATEGORIES.'_/', $cat['fullpath'])) {
                $list[$cat['id']] = $cat['fulllabel'];
            }
        }

        $out = '';

        $out .= $this->form->multiselectarray($htmlname, $list, $selected, $key_in_label, $value_as_key, $morecss, $translate, $width, $moreattrib, $elemtype);

        return $out;
    }

    /**
     *	Return multiselect list of all contacts (for a third party or all)
     *
     *	@param	int		$socid      	Id ot third party or 0 for all
     *	@param  array	$selected   	List of ID contact pre-selectionne
     *	@param  string	$htmlname  	    Name of HTML field ('none' for a not editable field)
     *	@param  string	$exclude        List of contacts id to exclude
     *	@param	string	$limitto		Disable answers that are not id in this array list
     *	@param	integer	$showfunction   Add function into label
     *	@param	string	$moreclass		Add more class to class style
     *	@param	bool	$options_only	Return options only (for ajax treatment)
     *	@param	integer	$showsoc	    Add company into label
     * 	@param	int		$forcecombo		Force to use combo box
     *  @param	array	$events			Event options. Example: array(array('method'=>'getContacts', 'url'=>dol_buildpath('/core/ajax/contacts.php',1), 'htmlname'=>'contactid', 'params'=>array('add-customer-contact'=>'disabled')))
     *	@return	 int					<0 if KO, Nb of contact in list if OK
     */
	public function multiselect_contacts($socid, $selected=array(), $htmlname='contactid', $exclude='', $limitto='', $showfunction=0, $moreclass='', $options_only=false, $showsoc=0, $forcecombo=0, $events=array())
    {
        global $conf;

        $out = '';

        $out .= $this->multiselect_javascript_code($selected, $htmlname);

        $save_conf = $conf->use_javascript_ajax;
        $conf->use_javascript_ajax = 0;
        $out .= $this->form->selectcontacts($socid,'',$htmlname,0,$exclude,$limitto,$showfunction, $moreclass, $options_only, $showsoc, $forcecombo, $events);
        $conf->use_javascript_ajax = $save_conf;

        return $out;
    }

	/**
	 *	Return HTML code of the SELECT of list of all contacts (for a third party or all).
	 *  This also set the number of contacts found into $this->num
	 *
	 * @since 9.0 Add afterSelectContactOptions hook
	 *
	 *	@param	int			$socid      	Id ot third party or 0 for all or -1 for empty list
	 *	@param	int			$request_id    	Id ot request
	 *	@param  array|int	$selected   	Array of ID of pre-selected contact id
	 *	@param  string		$htmlname  	    Name of HTML field ('none' for a not editable field)
	 *	@param  int			$showempty     	0=no empty value, 1=add an empty value, 2=add line 'Internal' (used by user edit), 3=add an empty value only if more than one record into list
	 *	@param  string		$exclude        List of contacts id to exclude
	 *	@param	string		$limitto		Disable answers that are not id in this array list
	 *	@param	integer		$showfunction   Add function into label
	 *	@param	string		$moreclass		Add more class to class style
	 *	@param	bool		$options_only	Return options only (for ajax treatment)
	 *	@param	integer		$showsoc	    Add company into label
	 * 	@param	int			$forcecombo		Force to use combo box (so no ajax beautify effect)
	 *  @param	array		$events			Event options. Example: array(array('method'=>'getContacts', 'url'=>dol_buildpath('/core/ajax/contacts.php',1), 'htmlname'=>'contactid', 'params'=>array('add-customer-contact'=>'disabled')))
	 *  @param	string		$moreparam		Add more parameters onto the select tag. For example 'style="width: 95%"' to avoid select2 component to go over parent container
	 *  @param	string		$htmlid			Html id to use instead of htmlname
	 *  @param	bool		$multiple		add [] in the name of element and add 'multiple' attribut
	 *	@return	 int						<0 if KO, Nb of contact in list if OK
	 */
	public function selectcontacts($socid, $request_id=0, $selected = '', $htmlname = 'contactid', $showempty = 0, $exclude = '', $limitto = '', $showfunction = 0, $moreclass = '', $options_only = false, $showsoc = 0, $forcecombo = 0, $events = array(), $moreparam = '', $htmlid = '', $multiple = false)
	{
		global $conf,$langs,$hookmanager,$action;

		$langs->load('companies');

		if (empty($htmlid)) $htmlid = $htmlname;

		if ($selected === '') $selected = array();
		elseif (!is_array($selected)) $selected = array($selected);
		$out='';

		if (! is_object($hookmanager))
		{
			include_once DOL_DOCUMENT_ROOT.'/core/class/hookmanager.class.php';
			$hookmanager=new HookManager($this->db);
		}

		// We search third parties
		$sql = "SELECT sp.rowid, sp.lastname, sp.statut, sp.firstname, sp.poste";
		if ($showsoc > 0) $sql.= " , s.nom as company";
		$sql.= " FROM ".MAIN_DB_PREFIX ."socpeople as sp";
		if ($showsoc > 0) $sql.= " LEFT OUTER JOIN  ".MAIN_DB_PREFIX ."societe as s ON s.rowid=sp.fk_soc";
		if ($request_id > 0) {
			$sql.= " LEFT OUTER JOIN  ".MAIN_DB_PREFIX ."element_contact as ec ON ec.fk_socpeople=sp.rowid";
			$sql.= " LEFT OUTER JOIN  ".MAIN_DB_PREFIX ."c_type_contact as ctc ON ctc.rowid=ec.fk_c_type_contact";
		}
		$sql.= " WHERE sp.entity IN (".getEntity('socpeople').")";
		if ($request_id > 0) $sql.= " AND ctc.element='requestmanager' AND ec.element_id=".$request_id;
		elseif ($socid > 0 || $socid == -1) $sql.= " AND sp.fk_soc=".$socid;
		if (! empty($conf->global->CONTACT_HIDE_INACTIVE_IN_COMBOBOX)) $sql.= " AND sp.statut <> 0";
		$sql.= " ORDER BY sp.lastname ASC";

		dol_syslog(get_class($this)."::select_contacts", LOG_DEBUG);
		$resql=$this->db->query($sql);
		if ($resql)
		{
			$num=$this->db->num_rows($resql);

			if ($conf->use_javascript_ajax && ! $forcecombo && ! $options_only)
			{
				include_once DOL_DOCUMENT_ROOT . '/core/lib/ajax.lib.php';
				$out .= ajax_combobox($htmlid, $events, $conf->global->CONTACT_USE_SEARCH_TO_SELECT);
			}

			if ($htmlname != 'none' && ! $options_only) $out.= '<select class="flat'.($moreclass?' '.$moreclass:'').'" id="'.$htmlid.'" name="'.$htmlname.($multiple ? '[]' : '').'" '.($multiple ? 'multiple' : '').' '.(!empty($moreparam) ? $moreparam : '').'>';
			if (($showempty == 1 || ($showempty == 3 && $num > 1)) && !$multiple) $out.= '<option value="0"'.(in_array(0, $selected)?' selected':'').'>&nbsp;</option>';
			if ($showempty == 2) $out.= '<option value="0"'.(in_array(0, $selected)?' selected':'').'>'.$langs->trans("Internal").'</option>';

			$num = $this->db->num_rows($resql);
			$i = 0;
			if ($num)
			{
				include_once DOL_DOCUMENT_ROOT.'/contact/class/contact.class.php';
				$contactstatic=new Contact($this->db);

				while ($i < $num)
				{
					$obj = $this->db->fetch_object($resql);

					$contactstatic->id=$obj->rowid;
					$contactstatic->lastname=$obj->lastname;
					$contactstatic->firstname=$obj->firstname;
					if ($obj->statut == 1) {
						if ($htmlname != 'none')
						{
							$disabled=0;
							if (is_array($exclude) && count($exclude) && in_array($obj->rowid, $exclude)) $disabled=1;
							if (is_array($limitto) && count($limitto) && ! in_array($obj->rowid, $limitto)) $disabled=1;
							if (!empty($selected) && in_array($obj->rowid, $selected))
							{
								$out.= '<option value="'.$obj->rowid.'"';
								if ($disabled) $out.= ' disabled';
								$out.= ' selected>';
								$out.= $contactstatic->getFullName($langs);
								if ($showfunction && $obj->poste) $out.= ' ('.$obj->poste.')';
								if (($showsoc > 0) && $obj->company) $out.= ' - ('.$obj->company.')';
								$out.= '</option>';
							}
							else
							{
								$out.= '<option value="'.$obj->rowid.'"';
								if ($disabled) $out.= ' disabled';
								$out.= '>';
								$out.= $contactstatic->getFullName($langs);
								if ($showfunction && $obj->poste) $out.= ' ('.$obj->poste.')';
								if (($showsoc > 0) && $obj->company) $out.= ' - ('.$obj->company.')';
								$out.= '</option>';
							}
						}
						else
						{
							if (in_array($obj->rowid, $selected))
							{
								$out.= $contactstatic->getFullName($langs);
								if ($showfunction && $obj->poste) $out.= ' ('.$obj->poste.')';
								if (($showsoc > 0) && $obj->company) $out.= ' - ('.$obj->company.')';
							}
						}
					}
					$i++;
				}
			}
			else
			{
				$out.= '<option value="-1"'.(($showempty==2 || $multiple) ? '' : ' selected').' disabled>';
				$out.= ($socid != -1) ? ($langs->trans($socid?"NoContactDefinedForThirdParty":"NoContactDefined")) : $langs->trans('SelectAThirdPartyFirst');
				$out.= '</option>';
			}

			$parameters = array(
				'socid'=>$socid,
				'htmlname'=>$htmlname,
				'resql'=>$resql,
				'out'=>&$out,
				'showfunction'=>$showfunction,
				'showsoc'=>$showsoc,
			);

			$reshook = $hookmanager->executeHooks('afterSelectContactOptions', $parameters, $this, $action);    // Note that $action and $object may have been modified by some hooks

			if ($htmlname != 'none' && ! $options_only)
			{
				$out.= '</select>';
			}

			$this->num = $num;
			return $out;
		}
		else
		{
			dol_print_error($this->db);
			return -1;
		}
	}

    /**
     *	Return multiselect list of groups
     *
     *  @param	array	$selected       List of ID group preselected
     *  @param  string	$htmlname       Field name in form
     *  @param  string	$exclude        Array list of groups id to exclude
     * 	@param	int		$disabled		If select list must be disabled
     *  @param  string	$include        Array list of groups id to include
     * 	@param	int		$enableonly		Array list of groups id to be enabled. All other must be disabled
     * 	@param	int		$force_entity	0 or Id of environment to force
     *  @return	string
     *  @see select_dolusers
     */
	public function multiselect_dolgroups($selected=array(), $htmlname='groupid', $exclude='', $disabled=0, $include='', $enableonly=0, $force_entity=0)
    {
        global $conf;

        $out = '';

        $out .= $this->multiselect_javascript_code($selected, $htmlname);

        $save_conf = $conf->use_javascript_ajax;
        $conf->use_javascript_ajax = 0;
        $out .= $this->form->select_dolgroups('', $htmlname, 0, $exclude, $disabled, $include, $enableonly, $force_entity);
        $conf->use_javascript_ajax = $save_conf;

        return $out;
    }

    /**
     *	Return multiselect list of users
     *
     *  @param	array|int	    $selected       List of user id or user object of user preselected. If -1, we use id of current user.
     *  @param  string	        $htmlname       Field name in form
     *  @param  array	        $exclude        Array list of users id to exclude
     * 	@param	int		        $disabled		If select list must be disabled
     *  @param  array|string	$include        Array list of users id to include or 'hierarchy' to have only supervised users or 'hierarchyme' to have supervised + me
     * 	@param	array	        $enableonly		Array list of users id to be enabled. All other must be disabled
     *  @param	int		        $force_entity	0 or Id of environment to force
     *  @param	int		        $maxlength		Maximum length of string into list (0=no limit)
     *  @param	int		        $showstatus		0=show user status only if status is disabled, 1=always show user status into label, -1=never show user status
     *  @param	string	        $morefilter		Add more filters into sql request
     *  @param	integer	        $show_every		0=default list, 1=add also a value "Everybody" at beginning of list
     *  @param	string	        $enableonlytext	If option $enableonlytext is set, we use this text to explain into label why record is disabled. Not used if enableonly is empty.
     *  @param	string	        $morecss		More css
     *  @param  int             $noactive       Show only active users (this will also happened whatever is this option if USER_HIDE_INACTIVE_IN_COMBOBOX is on).
     *  @param  int             $showexternal   Show external active users
     * 	@return	string					        HTML select string
     *  @see select_dolgroups
     */
	public function multiselect_dolusers($selected=array(), $htmlname='userid', $exclude=null, $disabled=0, $include='', $enableonly=array(), $force_entity=0, $maxlength=0, $showstatus=0, $morefilter='', $show_every=0, $enableonlytext='', $morecss='', $noactive=0, $showexternal=1)
    {
        global $conf, $user;

        $out = '';

        $selected_values = array();
        if (is_array($selected)) {
            foreach ($selected as $u) {
                $selected_values[] = is_object($u) ? $u->id : $u;
            }
        } elseif ($selected == -1) {
            $selected_values[] = $user->id;
        }

        $out .= $this->multiselect_javascript_code($selected_values, $htmlname);

        $save_conf = $conf->use_javascript_ajax;
        $conf->use_javascript_ajax = 0;
        $out .= $this->form->select_dolusers('', $htmlname, 0, $exclude, $disabled, $include, $enableonly, $force_entity, $maxlength, $showstatus, $morefilter, $show_every, $enableonlytext, $morecss, $noactive, $showexternal);
        $conf->use_javascript_ajax = $save_conf;

        return $out;
    }

    /**
   	 *	Return HTML code of the SELECT of list from a SQL Query.
   	 *  This also set the number of value found into $this->num
   	 *
   	 *	@param	string			    $sql      	            SQL request for get values (SELECT rowid, label FROM ...)
   	 *	@param  array|int|string	$selected   	        Array of ID of pre-selected value id
   	 *	@param  string		        $htmlname  	            Name of HTML field ('none' for a not editable field)
   	 *	@param  int			        $showempty     	        0=no empty value, 1=add an empty value
   	 *	@param  string		        $exclude                List of contacts id to exclude
   	 *	@param	string		        $limitto		        Disable answers that are not id in this array list
   	 *	@param	string		        $moreclass		        Add more class to class style
   	 *	@param	bool		        $options_only	        Return options only (for ajax treatment)
   	 * 	@param	int			        $forcecombo		        Force to use combo box
   	 *  @param	array		        $events			        Event options. See FormPeep::add_select_events for more details
   	 *  @param	string		        $moreparam		        Add more parameters onto the select tag. For example 'style="width: 95%"' to avoid select2 component to go over parent container
   	 *  @param	string		        $htmlid			        Html id to use instead of htmlname
     *  @param  int		            $outputmode             0=HTML select string, 1=Array
     *  @param  array               $extrakeyvalue          Extra key for output mode array (ex: key1 => key1 in SQL Request)
     *  @param  int		            $ismultiselect          0=Not multiselect, 1=Multiselect
     *  @param  int		            $exclude_not_included   0=Disable excluded, 1=Don't include exluded
     *  @param  int		            $has_group              0=Don't has group options, 1=Has group options (empty option group label must be '')
   	 *	@return	 int|array|string						    <0 if KO, select output if OK
   	 */
	public function select_query($sql, $selected=-1, $htmlname='', $showempty=0, $exclude='', $limitto='', $moreclass='', $options_only=false, $forcecombo=0, $events=array(), $moreparam='', $htmlid='', $outputmode=0, $extrakeyvalue=array(), $ismultiselect=0, $exclude_not_included=0, $has_group=0)
    {
        global $conf, $langs;

        if (empty($htmlid)) $htmlid = $htmlname;
        $out = '';
        $outarray = array();

        $sql .= $this->db->order(($has_group ? 'group_label,' : '') . 'label', ($has_group ? 'ASC,' : '') . 'ASC');
        $resql = $this->db->query($sql);
        if ($resql) {
            $num = $this->db->num_rows($resql);

            if (!is_array($selected)) $selected = explode(',', $selected);
            if ($conf->use_javascript_ajax && !$forcecombo && !$options_only) {
                include_once DOL_DOCUMENT_ROOT . '/core/lib/ajax.lib.php';
                if ($ismultiselect) {
                    $out .= $this->multiselect_javascript_code($selected, $htmlid, $elemtype = '');
                } else {
                    $out .= ajax_combobox($htmlid);
                }
                $out .= $this->add_select_events($htmlid, $events);
            }

            if (!$options_only) $out .= '<select class="flat' . ($moreclass ? ' ' . $moreclass : '') . '" id="' . $htmlid . '" name="' . $htmlname . '" ' . (!empty($moreparam) ? $moreparam : '') . '>';
            if (!empty($showempty)) $out .= '<option value="-1"' . ($selected == '-1' ? ' selected' : '') . '>&nbsp;</option>';

            $i = 0;
            if ($num) {
                $last_group = '';
                while ($i < $num) {
                    $obj = $this->db->fetch_object($resql);
                    $rowid = $obj->rowid;

                    $_disabled = 0;
                    $_selected = 0;
                    if (is_array($exclude) && count($exclude) && in_array($rowid, $exclude)) {
                        if ($exclude_not_included) continue; else $_disabled = 1;
                    }
                    if (is_array($limitto) && count($limitto) && !in_array($rowid, $limitto)) $_disabled = 1;
                    if (!empty($selected) && in_array($rowid, $selected)) $_selected = 1;
                    if (!empty($obj->langs)) $langs->load($obj->langs);

                    if ($has_group && $last_group != $obj->group_label) {
                        if ($i > 0) $out .= '</optgroup>';
                        $out .= '<optgroup label="' . dol_escape_htmltag($obj->group_label) . '">';
                        $last_group = $obj->group_label;
                    }
                    $out .= '<option value="' . dol_escape_htmltag($rowid) . '"';
                    if ($_disabled) $out .= ' disabled';
                    if ($_selected) $out .= ' selected';
                    $out .= '>';
                    $label = '';
                    if (!empty($obj->langs)) $label .= $langs->transnoentitiesnoconv($obj->label);
                    else $label .= $obj->label;
                    if (!empty($obj->extra_label)) {
                        if (!empty($obj->langs)) $label .= $langs->transnoentitiesnoconv($obj->extra_label);
                        else $label .= $obj->extra_label;
                    }
                    $out .= $label . '</option>';

                    if (!empty($outputmode)) {
                        $line = array('key' => $rowid, 'value' => $obj->ref, 'label' => $label);
                        if (is_array($extrakeyvalue)) {
                            foreach ($extrakeyvalue as $key) {
                                $line[$key] = isset($obj->$key) ? $obj->$key : '';
                            }
                        }
                        $outarray[] = $line;
                    }
                    $i++;
                }
                if ($has_group) $out .= '</optgroup>';
            }
            if (!$options_only) $out .= '</select>';

            $this->num = $num;
            $this->db->free($resql);

            return empty($outputmode) ? $out : $outarray;
        } else {
            $this->error = $this->db->lasterror();
            return -1;
        }
    }

    /**
   	 * Return list of labels (translated) of education
   	 *
     * @param	string	$htmlname	Name of html select field ('myid' or '.myclass')
     * @param	array	$events		Event options. Example: array(array('action'=>'getContacts', 'url'=>dol_buildpath('/core/ajax/contacts.php',1), 'htmlname'=>'contactid', 'done_action'=>array('disabled' => array('add-customer-contact'))))
     *                                  'url':          string,  Url of the Ajax script
     *                                  'action':       string,  Action name for the Ajax script
     *                                  'params':       array(), Others parameters send for Ajax script (exclude name: id, action, htmlname), if value = '{{selector}}' get value of the 'selector' input
     *                                  'htmlname':     string,  Id of the select updated with new options from Ajax script
     *                                  'done_action':  array(), List of action done when new options get successfully
     *                                      'empty_select': array(), List of html ID of select to empty
     *                                      'disabled'    : array(), List of html ID to disable if no options
   	 * @return  string
   	 */
	public function add_select_events($htmlname, $events)
    {
        global $conf;

        $out = '';
        if (!empty($conf->use_javascript_ajax)) {
            $out .= '<script type="text/javascript">
            $(document).ready(function () {
                jQuery("#'.$htmlname.'").change(function () {
                    var obj = '.json_encode($events).';
                    $.each(obj, function(key,values) {
                        if (values.action.length) {
                            runJsCodeForEvent'.$htmlname.'(values);
                        }
                    });
                });
    
                function runJsCodeForEvent'.$htmlname.'(obj) {
                    console.log("Run runJsCodeForEvent'.$htmlname.'");
                    var id = $("#'.$htmlname.'").val();
                    var action = obj.action;
                    var url = obj.url;
                    var htmlname = obj.htmlname;
                    var datas = {
                        action: action,
                        id: id,
                        htmlname: htmlname,
                    };
                    var selector_regex = new RegExp("^\\{\\{(.*)\\}\\}$", "i");
                    $.each(obj.params, function(key, value) {
                        var match = null;
                        if ($.type(value) === "string") match = value.match(selector_regex);
                        if (match) {
                            datas[key] = $(match[1]).val();
                        } else {
                            datas[key] = value;
                        }
                    });
                    var input = $("select#" + htmlname);
                    var inputautocomplete = $("#inputautocomplete"+htmlname);
                    $.getJSON(url, datas,
                        function(response) {
                            input.html(response.value);
                            if (response.num) {
                                var selecthtml_dom = $.parseHTML(response.value);
                                inputautocomplete.val(selecthtml_dom.innerHTML);
                            } else {
                                inputautocomplete.val("");
                            }
                                
                            var num = response.num;
                            $.each(obj.done_action, function(key, action) {
                                switch (key) {
                                    case "empty_select":
                                        $.each(action, function(id) {
                                            $("select#" + id).html("");
                                        });
                                        break;
                                    case "disabled":
                                        $.each(action, function(id) {
                                            if (num > 0) {
                                                $("#" + id).removeAttr("disabled");
                                            } else {
                                                $("#" + id).attr("disabled", "disabled");
                                            }
                                        });
                                        break;
                                }
                            });
                                
                            input.change();	/* Trigger event change */
                            
                            if (response.num < 0) {
                                console.error(response.error);
                            }
                        }
                    );
                }
            });
            </script>';
        }

        return $out;
    }

    /**
     *	Return multiselect javascript code
     *
     *  @param	array	$selected       Preselected values
     *  @param  string	$htmlname       Field name in form
     *  @param	string	$elemtype		Type of element we show ('category', ...)
     *  @return	string
     */
	public function multiselect_javascript_code($selected, $htmlname, $elemtype='')
    {
        global $conf;

        $out = '';

        // Add code for jquery to use multiselect
       	if (! empty($conf->global->MAIN_USE_JQUERY_MULTISELECT) || defined('REQUIRE_JQUERY_MULTISELECT'))
       	{
            $selected = array_values($selected);
       		$tmpplugin=empty($conf->global->MAIN_USE_JQUERY_MULTISELECT)?constant('REQUIRE_JQUERY_MULTISELECT'):$conf->global->MAIN_USE_JQUERY_MULTISELECT;
      			$out.='<!-- JS CODE TO ENABLE '.$tmpplugin.' for id '.$htmlname.' -->
       			<script type="text/javascript">
   	    			function formatResult(record) {'."\n";
   						if ($elemtype == 'category')
   						{
   							$out.='	//return \'<span><img src="'.DOL_URL_ROOT.'/theme/eldy/img/object_category.png'.'"> <a href="'.DOL_URL_ROOT.'/categories/viewcat.php?type=0&id=\'+record.id+\'">\'+record.text+\'</a></span>\';
   								  	return \'<span><img src="'.DOL_URL_ROOT.'/theme/eldy/img/object_category.png'.'"> \'+record.text+\'</span>\';';
   						}
   						else
   						{
   							$out.='return record.text;';
   						}
   			$out.= '	};
       				function formatSelection(record) {'."\n";
   						if ($elemtype == 'category')
   						{
   							$out.='	//return \'<span><img src="'.DOL_URL_ROOT.'/theme/eldy/img/object_category.png'.'"> <a href="'.DOL_URL_ROOT.'/categories/viewcat.php?type=0&id=\'+record.id+\'">\'+record.text+\'</a></span>\';
   								  	return \'<span><img src="'.DOL_URL_ROOT.'/theme/eldy/img/object_category.png'.'"> \'+record.text+\'</span>\';';
   						}
   						else
   						{
   							$out.='return record.text;';
   						}
   			$out.= '	};
   	    			$(document).ready(function () {
   	    			    $(\'#'.$htmlname.'\').attr("name", "'.$htmlname.'[]");
   	    			    $(\'#'.$htmlname.'\').attr("multiple", "multiple");
   	    			    //$.map('.json_encode($selected).', function(val, i) {
   	    			        $(\'#'.$htmlname.'\').val('.json_encode($selected).');
   	    			    //});
   	    			
       					$(\'#'.$htmlname.'\').'.$tmpplugin.'({
       						dir: \'ltr\',
   							// Specify format function for dropdown item
   							formatResult: formatResult,
       					 	templateResult: formatResult,		/* For 4.0 */
   							// Specify format function for selected item
   							formatSelection: formatSelection,
       					 	templateResult: formatSelection		/* For 4.0 */
       					});
       				});
       			</script>';
       	}

       	return $out;
    }

    /**
     *  Show children request block.
     *
     *  @param	RequestManager	$object		      Object we want to show children request to
     *  @param  string          $morehtmlright    More html to show on right of title
     *  @return	int							      Nb children found
     */
	public function showChildrenRequestBlock($object, $morehtmlright='')
    {
        global $conf, $langs, $hookmanager;
        global $bc;

        $object->fetch_children_request(null, 1);

        // Bypass the default method
		$hookmanager2 = clone $hookmanager; // Génère des erreurs de resultat disparaissant si appelé dans une autre hooks donc on copie la hook
		$hookmanager2->initHooks(array('requestmanagerdao'));
        $parameters = array();
        $reshook = $hookmanager2->executeHooks('showChildrenRequestBlock', $parameters, $object, $action);    // Note that $action and $object may have been modified by hook

        if (empty($reshook)) {
            $nbchildren = count($object->children_request_ids);

            print '<br><!-- showChildrenRequestBlock -->';
            print load_fiche_titre($langs->trans('RequestManagerChildrenRequest'), $morehtmlright, '');

            print '<div class="div-table-responsive-no-min">';
            print '<table class="noborder allwidth">';

            print '<tr class="liste_titre">';
            print '<td>' . $langs->trans("Ref") . '</td>';
            print '<td>' . $langs->trans("RequestManagerLabel") . '</td>';
            print '<td align="center">' . $langs->trans("RequestManagerType") . '</td>';
            print '<td align="center">' . $langs->trans("Date") . '</td>';
            print '<td align="right">' . $langs->trans("Status") . '</td>';
            print '</tr>';

            global $childrenRequestBlock;
            global $stopParentID;
            $childrenRequestBlock = $object->children_request_list;
            $stopParentID = $object->id;

            // Output template part (modules that overwrite templates must declare this into descriptor)
            $dirtpls = array_merge($conf->modules_parts['tpl'], array('/requestmanager/tpl'));
            foreach ($dirtpls as $reldir) {
                $res = @include dol_buildpath($reldir . '/childrenrequestblock.tpl.php');
                if ($res) {
                    break;
                }
            }

            if (!$nbchildren) {
                print '<tr><td class="impair opacitymedium" colspan="7">' . $langs->trans("None") . '</td></tr>';
            }

            print '</table>';
            print '</div>';

            return $nbchildren;
        }

        return $reshook;
    }

    /**
     *  Show children request linked objects block.
     *
     *  @param	RequestManager	$object		      Object we want to show children request to
     *  @param  string          $morehtmlright    More html to show on right of title
     *  @return	int							      Nb children linked objects found
     */
	public function showChildrenRequestLinkedObjectsBlock($object, $morehtmlright='')
    {
        global $conf, $langs, $hookmanager;
        global $bc;

        $object->fetchObjectLinked();
        $object->fetch_children_request(null, 1);

        $linked_objects = array();
        foreach ($object->children_request_list as $request) {
            $request->fetchObjectLinked();

            foreach ($request->linkedObjects as $elementtype => $object_list) {
                foreach ($object_list as $item) {
                    if (isset($object->linkedObjectsIds[$elementtype]) && in_array($item->id, $object->linkedObjectsIds[$elementtype])) continue;
                    $linked_objects[$elementtype . '_' . $item->id] = $item;
                }
            }
        }

        // Bypass the default method
		$hookmanager2 = clone $hookmanager; // Génère des erreurs de resultat disparaissant si appelé dans une autre hooks donc on copie la hook
		$hookmanager2->initHooks(array('requestmanagerdao'));
        $parameters = array('linked_objects' => &$linked_objects);
        $reshook = $hookmanager2->executeHooks('showChildrenRequestLinkedObjectsBlock', $parameters, $object, $action);    // Note that $action and $object may have been modified by hook

        if (empty($reshook)) {
            $nb_linked_objects = count($linked_objects);

            print '<br><!-- showChildrenRequestLinkedObjectsBlock -->';
            print load_fiche_titre($langs->trans('RequestManagerChildrenRequestLinkedObjects'), $morehtmlright, '');

            print '<div class="div-table-responsive-no-min">';
            print '<table class="noborder allwidth">';

            print '<tr class="liste_titre">';
            print '<td>' . $langs->trans("RequestManagerList") . '</td>';
            print '</tr>';

            if ($nb_linked_objects) {
                $to_print = array();
                foreach ($linked_objects as $item) {
                    $to_print[] = $item->getNomUrl(1);
                }

                print '<tr><td>' . implode(', ', $to_print) . '</td></tr>';
            } else {
                print '<tr><td class="impair opacitymedium">' . $langs->trans("None") . '</td></tr>';
            }

            print '</table>';
            print '</div>';

            return $nb_linked_objects;
        }

        return $reshook;
    }

    /**
     *  Show times spent block.
     *
     *  @param	RequestManager	$object		      Object we want to show times spent to
     *  @param  string          $morehtmlright    More html to show on right of title
     *  @return	int							      Nb children found
     */
	public function showTimesSpentBlock($object, $morehtmlright='')
    {
        global $conf, $form, $langs, $hookmanager, $user;
        global $bc;

        $langs->load('projects');
        //$object->fetch_times_spent();

        // Bypass the default method
		$hookmanager2 = clone $hookmanager; // Génère des erreurs de resultat disparaissant si appelé dans une autre hooks donc on copie la hook
		$hookmanager2->initHooks(array('requestmanagerdao'));
        $parameters = array();
        $reshook = $hookmanager2->executeHooks('showTimesSpentBlock', $parameters, $object, $action);    // Note that $action and $object may have been modified by hook

        if (empty($reshook)) {
            print '<br><!-- showTimesSpentBlock -->';
            print load_fiche_titre($langs->trans('RequestManagerTimesSpent'), $morehtmlright, '');

            if ($user->rights->requestmanager->lire &&
				($object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS || $object->statut_type == RequestManager::STATUS_TYPE_RESOLVED) &&
				($user->rights->requestmanager->timesspent->manage || in_array($user->id, $object->assigned_user_ids))
			) {
				print '<form action="' . $_SERVER["PHP_SELF"] . '?id=' . $object->id . '" method="POST">';
				print '<input type="hidden" name="token" value="' . $_SESSION['newtoken'] . '" />';
				print '<input type="hidden" name="action" value="add_time_spent" />';

				print '<div class="div-table-responsive-no-min">';
				print '<table class="noborder allwidth">';

				print '<tr class="liste_titre">';
				print '<td>' . $langs->trans("Date") . '</td>';
				print '<td>' . $langs->trans("By") . '</td>';
				print '<td>' . $langs->trans("Note") . '</td>';
				print '<td>' . $langs->trans("NewTimeSpent") . '</td>';
				print '<td></td>';
				print '</tr>';

				$date = dol_mktime(GETPOST('time_spent_datehour'), GETPOST('time_spent_datemin'), 0, GETPOST('time_spent_datemonth'), GETPOST('time_spent_dateday'), GETPOST('time_spent_dateyear'));
				$user_id = GETPOST('time_spent_user', 'int');
				$note = GETPOST('time_spent_note', 'alpha');
				$duration_hour = GETPOST('time_spent_durationhour', 'int');
				$duration_hour = $duration_hour > 0 ? $duration_hour : 0;
				$duration_minute = GETPOST('time_spent_durationmin', 'int');
				$duration_minute = $duration_minute > 0 ? $duration_minute : 0;
				$duration = $duration_hour * 3600 + $duration_minute * 60;    // We store duration in seconds

				print '<tr class="oddeven">';
				// Date
				print '<td class="maxwidthonsmartphone">';
				print $form->select_date($date, 'time_spent_date', ($conf->browser->layout == 'phone' ? 2 : 1), 1, 2, "time_spent_date", 1, 0, 1);
				print '</td>';
				// Contributor
				print '<td class="maxwidthonsmartphone">';
				if ($user->rights->requestmanager->timesspent->manage) {
					print img_object('', 'user', 'class="hideonsmartphone"');
					print $form->select_dolusers($user_id > 0 ? $user_id : $user->id, 'time_spent_user', 0, null, 0, null, null, 0, 56, '', 0, '', '', 'minwidth200imp', 1);
				} else {
					print '<input type="hidden" name="time_spent_user" value="' . $user->id . '" />';
					print $user->getNomUrl(1);
				}
				print '</td>';
				// Note
				print '<td>';
				print '<textarea name="time_spent_note" class="maxwidth100onsmartphone" rows="' . ROWS_2 . '">' . $note . '</textarea>';
				print '</td>';
				// Duration - Time spent
				print '<td>';
				print $form->select_duration('time_spent_duration', $duration, 0, 'text', 0, 1);
				print '</td>';
				print '<td align="right">';
				print '<input type="submit" class="button" name="add_time_spent" value="' . $langs->trans("Add") . '">';
				print '</td>';
				print '</tr>';
				print '</table>';

				print '</form>';
				print '</div>';
			}

			print '<div class="div-table-responsive-no-min">';
			print '<table class="noborder allwidth">';
            print '<tr class="liste_titre">';
            print '<td>' . $langs->trans("User") . '</td>';
            print '<td>' . $langs->trans("TimeSpent") . '</td>';
            print '<td>' . $langs->trans("RequestManagerEffectiveCost") . '</td>';
            print '</tr>';

            $nbtimes = 0;
            $result = $object->get_sum_time_spent(1);
            if (is_array($result)) {
                global $objectID, $timesSpentBlock;
                $objectID = $object->id;
                $timesSpentBlock = $result;
                $nbtimes = count($result);

                // Output template part (modules that overwrite templates must declare this into descriptor)
                $dirtpls = array_merge($conf->modules_parts['tpl'], array('/requestmanager/tpl'));
                foreach ($dirtpls as $reldir) {
                    $res = @include dol_buildpath($reldir . '/timesspentblock.tpl.php');
                    if ($res) {
                        break;
                    }
                }
            } else {
                setEventMessages($object->error, $object->errors, 'errors');
            }

            if (!$nbtimes) {
                print '<tr><td class="impair opacitymedium" colspan="3">' . $langs->trans("None") . '</td></tr>';
            }

            print '</table>';
            print '</div>';

            return $nbtimes;
        }

        return $reshook;
    }

    /**
     *  Output html form to select a actioncomm
     *
     * @param   int         $idActionComm           Id of the actioncomm
     * @param   array       $actionCommCodeList     [=array] List of actioncomm code
     * @param   string      $selected               Preselected actioncomm
     * @param   string      $htmlname               Name of field in form
     * @param   string      $showempty              Add an empty field (Can be '1' or text key to use on empty line like 'SelectThirdParty')
     * @param   int         $forcecombo             Force to use combo box
     * @param   array       $events                 Ajax event options to run on change. Example: array(array('method'=>'getContacts', 'url'=>dol_buildpath('/core/ajax/contacts.php',1), 'htmlname'=>'contactid', 'params'=>array('add-customer-contact'=>'disabled')))
     * @param  	int		    $usesearchtoselect	    Minimum length of input string to start autocomplete
     * @param   string      $morecss                Add more css styles to the SELECT component
     * @param   string      $moreparam              Add more parameters onto the select tag. For example 'style="width: 95%"' to avoid select2 component to go over parent container
     * @param   bool        $options_only           Return options only (for ajax treatment)
     * @return  string                              HTML string with select box for status.
     */
	public function select_actioncomm($idActionComm, $actionCommCodeList=array(), $selected='', $htmlname='actioncomm_id', $showempty=0, $forcecombo=0, $events=array(), $usesearchtoselect=0, $morecss='minwidth100', $moreparam='', $options_only=false)
    {
        global $conf, $langs, $user;

        $out = '';

        // search actioncomm
        $sql  = "SELECT";
        $sql .= " ac.id";
        $sql .= ", ac.label";
        $sql .= " FROM " . MAIN_DB_PREFIX . "actioncomm as ac";
        $sql .= " WHERE ac.entity IN (" . getEntity('agenda') . ")";
        if (count($actionCommCodeList) > 0) {
            $sql .= " AND ac.code IN (";
            $sqlCodeIn = '';
            $i = 0;
            foreach($actionCommCodeList as $actionCommCode) {
                if ($i > 0) {
                    $sqlCodeIn .= ", ";
                }
                $sqlCodeIn .= "'" . $this->db->escape($actionCommCode) . "'";

                $i++;
            }
            $sql .= $sqlCodeIn;
            $sql .= ")";
        }
        $sql .= " AND ac.elementtype IS NULL";
        if ($idActionComm > 0) {
            $sql .= " AND ac.id = " . $idActionComm;
        }
        $sql .= " ORDER BY ac.fk_user_action = " . $user->id . ", ac.datep DESC";

        dol_syslog(__METHOD__, LOG_DEBUG);
        $resql = $this->db->query($sql);
        if ($resql)
        {
            if ($conf->use_javascript_ajax && ! $forcecombo && ! $options_only)
            {
                include_once DOL_DOCUMENT_ROOT . '/core/lib/ajax.lib.php';
                $comboenhancement = ajax_combobox($htmlname, $events, $usesearchtoselect);
                $out .= $comboenhancement;
            }

            if (!$options_only) $out .= '<select id="' . $htmlname . '" class="flat' . ($morecss ? ' ' . $morecss : '') . '"' . ($moreparam ? ' ' . $moreparam : '') . ' name="' . $htmlname . '">';

            $textifempty = '';
            // Do not use textifempty = ' ' or '&nbsp;' here, or search on key will search on ' key'.
            //if (! empty($conf->use_javascript_ajax) || $forcecombo) $textifempty='';
            if (!empty($usesearchtoselect)) {
                if ($showempty && !is_numeric($showempty)) $textifempty = $langs->trans($showempty);
                else $textifempty .= $langs->trans("All");
            }
            if ($showempty) $out .= '<option value="-1">' . $textifempty . '</option>' . "\n";

            $num = $this->db->num_rows($resql);
            $i = 0;
            if ($num)
            {
                while ($i < $num)
                {
                    $obj = $this->db->fetch_object($resql);

                    $out.= '<option value="' . $obj->id . '"';
                    if ($selected && $selected == $obj->id) $out .= ' selected';
                    $out .= '>';
                    $out .= $obj->label;
                    $out .= '</option>';
                    $i++;
                }
            }
            else
            {
                $out .= '<option value="-1" disabled>' . $langs->trans("RequestManagerNoActionComm") . '</option>';
            }

            if (!$options_only)
            {
                $out .= '</select>';
            }

            $this->num = $num;
            return $out;
        }
        else
        {
            dol_print_error($this->db);
            return -1;
        }
    }

    /**
     *  Output html form to select an equipement
     *
     * @param   int         $fkSoc                  Id of company
     * @param   string      $selected               Preselected equipement
     * @param   string      $htmlname               Name of field in form
     * @param   string      $showempty              Add an empty field (Can be '1' or text key to use on empty line like 'SelectThirdParty')
     * @param   int         $forcecombo             Force to use combo box
     * @param   array       $events                 Ajax event options to run on change. Example: array(array('method'=>'getContacts', 'url'=>dol_buildpath('/core/ajax/contacts.php',1), 'htmlname'=>'contactid', 'params'=>array('add-customer-contact'=>'disabled')))
     * @param  	int		    $usesearchtoselect	    Minimum length of input string to start autocomplete
     * @param   string      $morecss                Add more css styles to the SELECT component
     * @param   string      $moreparam              Add more parameters onto the select tag. For example 'style="width: 95%"' to avoid select2 component to go over parent container
     * @param   bool        $options_only           Return options only (for ajax treatment)
     * @return  string                              HTML string with select box for status.
     */
	public function select_equipement($fkSoc, $selected='', $htmlname='equipement_id', $showempty=0, $forcecombo=0, $events=array(), $usesearchtoselect=0, $morecss='minwidth100', $moreparam='', $options_only=false)
    {
        global $conf, $langs;

        $out = '';

        dol_syslog(__METHOD__ . " fkSoc=" . $fkSoc,  LOG_DEBUG);

        // search equipement
        $requestManager = new RequestManager($this->db);
        $resql = $requestManager->findAllEquipemenByFkSoc($fkSoc);
        if ($resql)
        {
            if ($conf->use_javascript_ajax && ! $forcecombo && ! $options_only)
            {
                include_once DOL_DOCUMENT_ROOT . '/core/lib/ajax.lib.php';
                $comboenhancement = ajax_combobox($htmlname, $events, $usesearchtoselect);
                $out .= $comboenhancement;
            }

            if (!$options_only) $out .= '<select id="' . $htmlname . '" class="flat' . ($morecss ? ' ' . $morecss : '') . '"' . ($moreparam ? ' ' . $moreparam : '') . ' name="' . $htmlname . '">';

            $textifempty = '';
            // Do not use textifempty = ' ' or '&nbsp;' here, or search on key will search on ' key'.
            //if (! empty($conf->use_javascript_ajax) || $forcecombo) $textifempty='';
            if (!empty($usesearchtoselect)) {
                if ($showempty && !is_numeric($showempty)) $textifempty = $langs->trans($showempty);
                else $textifempty .= $langs->trans("All");
            }
            if ($showempty) $out .= '<option value="-1">' . $textifempty . '</option>' . "\n";

            $num = $this->db->num_rows($resql);
            $i = 0;
            if ($num)
            {
                while ($i < $num)
                {
                    $obj = $this->db->fetch_object($resql);

                    $out.= '<option value="' . $obj->rowid . '"';
                    if ($selected && $selected == $obj->rowid) $out .= ' selected';
                    $out .= '>';
                    $out .= $obj->ref;
                    $out .= '</option>';
                    $i++;
                }
            }
            else
            {
                $out .= '<option value="-1" disabled>' . $langs->trans("RequestManagerNoEquipement") . '</option>';
            }

            if ($options_only)
            {
                $out .= '</select>';
            }

            $this->num = $num;
            return $out;
        }
        else
        {
            dol_print_error($this->db);
            return -1;
        }
    }

    /**
     * Print contact add form
     *
     * @param   RequestManager  $requestManager     Request manager object
     * @param   int             $idContactType      Id of contact type
     * @return  void            Print contact add form
     */
	public function form_add_contact(RequestManager $requestManager, $idContactType)
    {
        global $conf, $langs, $user;

        $formCompany = NULL;
        if ($idContactType === RequestManager::CONTACT_TYPE_ID_WATCHER) {
            require_once DOL_DOCUMENT_ROOT . '/core/class/html.formcompany.class.php';
            $formCompany = new FormCompany($this->db);
            $excludes_contact = $requestManager->watcher_ids;
        } else {
            $excludes_contact = $requestManager->requester_ids;
        }
        $newCompanyId = $requestManager->socid;
        $contactTypeCodeHtmlName = RequestManager::getContactTypeCodeHtmlNameById($idContactType);

        // form to add requester contact
        print '<form name="form_add_contact_' . $contactTypeCodeHtmlName . '" action="' . $_SERVER["PHP_SELF"] . '" method="post">';
        print '<input type="hidden" name="token" value="' . $_SESSION['newtoken'] . '">';
        print '<input type="hidden" name="action" value="add_contact">';
        print '<input type="hidden" name="id" value="' . $requestManager->id . '">';
        print '<input type="hidden" name="add_contact_type_id" value="' . $idContactType . '">';
        print '<table class="nobordernopadding" width="100%">';
        print '<tr>';
        print '<td>';
        if ($formCompany !== NULL) {
            $selectCompaniesHtmlName = $contactTypeCodeHtmlName . '_newcompany';
            $newCompanyId = intval(GETPOST($selectCompaniesHtmlName, 'int')?GETPOST($selectCompaniesHtmlName, 'int'):$requestManager->socid);
            $formCompany->selectCompaniesForNewContact($requestManager,'id', $newCompanyId, $selectCompaniesHtmlName, '', 0, '', 'minwidth300');
        }
        $this->form->select_contacts($newCompanyId, '', $contactTypeCodeHtmlName . '_fk_socpeople', 1, $excludes_contact, '', 0, 'minwidth300');
        print '&nbsp;<input type="submit" class="button" value="' . $langs->trans('Add') . '">';

        // button create contact (only for requesters)
        if ($idContactType === RequestManager::CONTACT_TYPE_ID_REQUEST && $user->rights->societe->contact->creer)
        {
            $backToPage = $_SERVER["PHP_SELF"] . '?id=' . $requestManager->id;
            $btnCreateContactLabel = (!empty($conf->global->SOCIETE_ADDRESSES_MANAGEMENT) ? $langs->trans("AddContact") : $langs->trans("AddContactAddress"));
            $btnCreateContact = '<a class="addnewrecord" href="'.DOL_URL_ROOT.'/contact/card.php?socid='.$requestManager->socid . '&token=' . newToken().'&action=create&backtopage=' . urlencode($backToPage) . '">' . $btnCreateContactLabel;
            if (empty($conf->dol_optimize_smallscreen)) $btnCreateContact .= ' ' . img_picto($btnCreateContactLabel, 'filenew');
            $btnCreateContact .= '</a>'."\n";
            print '&nbsp;&nbsp;' . $btnCreateContact;
        }

        print '</td>';
        print '</tr>';
        print '</table>';
        print '</form>';
    }

    /**
     * 	Render list of categories linked to object with id $id and type $type
     *
     * 	@param		int		$id				Id of object
     * 	@param		string	$type			Type of category ('member', 'customer', 'supplier', 'product', 'contact', 'requestmanager')
     *  @param		int		$rendermode		0=Default, use multiselect. 1=Emulate multiselect (recommended)
     *  @param		int		$editMode		[=FALSE] for view mode, TRUE for edit mode (with rendermode=0 only)
     *	@param	    string	$htmlname		Name of select
     *	@param	    array	$arrayselected  Selected values
     * 	@return		string					String with categories
     */
	public function showCategories($id, $type, $rendermode=0, $editMode=FALSE, $htmlname='categories', $arrayselected=null)
    {
        global $db;

        dol_include_once('/requestmanager/class/categorierequestmanager.class.php');

        $cat = new CategorieRequestManager($db);
        $categories = $cat->containing($id, $type);

        if ($rendermode == 1)
        {
            $toprint = array();
            foreach($categories as $c)
            {
                $ways = $c->print_all_ways();       // $ways[0] = "ccc2 >> ccc2a >> ccc2a1" with html formated text
                foreach($ways as $way)
                {
                    $toprint[] = '<li class="select2-search-choice-dolibarr noborderoncategories"'.($c->color?' style="background: #'.$c->color.';"':' style="background: #aaa"').'>'.img_object('','category').' '.$way.'</li>';
                }
            }
            return '<div class="select2-container-multi-dolibarr" style="width: 90%;"><ul class="select2-choices-dolibarr">'.implode(' ', $toprint).'</ul></div>';
        }

        if ($rendermode == 0)
        {
            if (!isset($arrayselected)) {
                $arrayselected = array();
                foreach ($categories as $c) {
                    $arrayselected[] = $c->id;
                }
            }

            $selectMoreAttrib = 'disabled';
            $selectElementType = 'category';
            if ($editMode === TRUE) {
                $selectMoreAttrib = '';
                $selectElementType = '';
            }

            return $this->multiselect_categories($arrayselected, $htmlname, '', 0, '', 0, '100%', $selectMoreAttrib, $selectElementType);
        }

        return 'ErrorBadValueForParameterRenderMode';	// Should not happened
    }

	/**
	 * Prepare SQL request for lists to follow
	 *
	 * @param	DoliDB			$db						Doli DB object
	 * @param	RequestManager	$requestmanagerstatic	RequestManager object
	 * @param	string			$join					[=''] Join condition in SQL
	 * @param	string			$filter					[=''] Filter condition where in SQL
	 * @param	string			$sortfield				[=''] List of sort fields, separated by comma. Example: 't1.fielda, t2.fieldb'
	 * @param	string			$sortorder				[=''] List of sort order seprated by comma ('ASC'|'DESC')
	 * @return	string									SQL request
	 */
	private static function _listsFollowSqlPrepare(DoliDB $db, RequestManager &$requestmanagerstatic, $join = '', $filter = '', $sortfield = '', $sortorder = '')
	{
		global $extrafields;

		$v13p = version_compare(DOL_VERSION, '13.0.0', '>=');

		$sql = 'SELECT DISTINCT';
		$sql .= ' rm.rowid, rm.ref, rm.ref_ext, rm.fk_parent,';
		$sql .= ' rm.fk_soc, s.nom as soc_name_origin, s.name_alias as soc_name_alias_origin, s.client as soc_client_origin, s.fournisseur as soc_fournisseur_origin, s.code_client as soc_code_client_origin, s.code_fournisseur as soc_code_fournisseur_origin,';
		$sql .= ' GROUP_CONCAT(DISTINCT ec.fk_socpeople SEPARATOR \',\') as requester_contacts,';
		$sql .= ' rm.label, rm.description,';
		$sql .= ' rm.fk_type, crmrt.label as type_label,';
		$sql .= ' rm.fk_category, crmc.label as category_label,';
		$sql .= ' rm.fk_source, crms.label as source_label,';
		$sql .= ' rm.fk_urgency, crmu.label as urgency_label,';
		$sql .= ' rm.fk_impact, crmi.label as impact_label,';
		$sql .= ' rm.fk_priority, crmp.label as priority_label,';
		$sql .= ' rm.notify_requester_by_email, rm.notify_watcher_by_email, rm.notify_assigned_by_email,';
		$sql .= ' GROUP_CONCAT(DISTINCT rmau.fk_user SEPARATOR \',\') as assigned_users,';
		$sql .= ' GROUP_CONCAT(DISTINCT rmaug.fk_usergroup SEPARATOR \',\') as assigned_usergroups,';
		$sql .= ' rm.estimated_duration, rm.effective_duration, rm.total_effective_duration, rm.reported_progress, rm.effective_cost, rm.date_operation, rm.date_deadline, rm.date_resolved, rm.date_closed,';
		$sql .= ' rm.fk_user_resolved, ur.firstname as userresolvedfirstname, ur.lastname as userresolvedlastname, ur.email as userresolvedemail,';
		$sql .= ' rm.fk_user_closed, uc.firstname as userclosedfirstname, uc.lastname as userclosedlastname, uc.email as userclosedemail,';
		$sql .= ' rm.fk_status,';
		$sql .= ' rm.datec, rm.tms,';
		$sql .= ' rm.fk_user_author, ua.firstname as userauthorfirstname, ua.lastname as userauthorlastname, ua.email as userauthoremail,';
		$sql .= ' rm.fk_user_modif, um.firstname as usermodiffirstname, um.lastname as usermodiflastname, um.email as usermodifemail';
		// Add fields from extrafields
		if ($v13p) {
			if (!empty($extrafields->attributes[$requestmanagerstatic->table_element]['label'])) {
				foreach ($extrafields->attributes[$requestmanagerstatic->table_element]['label'] as $key => $val) {
					$sql .= ($extrafields->attributes[$requestmanagerstatic->table_element]['type'][$key] != 'separate' ? ", ef.".$key.' as options_'.$key : '');
				}
			}
		} else {
			foreach ($extrafields->attribute_label as $key => $val) $sql .= ($extrafields->attribute_type[$key] != 'separate' ? ", ef." . $key . ' as options_' . $key : '');
		}
		$sql .= ' FROM '.MAIN_DB_PREFIX.'requestmanager as rm';
		$sql .= ' LEFT JOIN '.MAIN_DB_PREFIX.'c_requestmanager_request_type as crmrt on (crmrt.rowid = rm.fk_type)';
		$sql .= ' LEFT JOIN '.MAIN_DB_PREFIX.'c_requestmanager_category as crmc on (crmc.rowid = rm.fk_category)';
		$sql .= ' LEFT JOIN '.MAIN_DB_PREFIX.'c_requestmanager_source as crms on (crms.rowid = rm.fk_source)';
		$sql .= ' LEFT JOIN '.MAIN_DB_PREFIX.'c_requestmanager_urgency as crmu on (crmu.rowid = rm.fk_urgency)';
		$sql .= ' LEFT JOIN '.MAIN_DB_PREFIX.'c_requestmanager_impact as crmi on (crmi.rowid = rm.fk_impact)';
		$sql .= ' LEFT JOIN '.MAIN_DB_PREFIX.'c_requestmanager_priority as crmp on (crmp.rowid = rm.fk_priority)';
		$sql .= ' LEFT JOIN '.MAIN_DB_PREFIX.'c_requestmanager_status as crmst on (crmst.rowid = rm.fk_status)';
		$sql .= ' LEFT JOIN '.MAIN_DB_PREFIX.'societe as s on (s.rowid = rm.fk_soc)';
		$sql .= ' LEFT JOIN '.MAIN_DB_PREFIX.'element_contact as ec ON (ec.fk_c_type_contact = 163018 AND ec.element_id = rm.rowid)';
		if ($v13p) {
			if (!empty($extrafields->attributes[$requestmanagerstatic->table_element]['label'])) {
				$sql .= " LEFT JOIN ".MAIN_DB_PREFIX."requestmanager_extrafields as ef on (rm.rowid = ef.fk_object)";
			}
		} else {
			if (is_array($extrafields->attribute_label) && count($extrafields->attribute_label)) {
				$sql .= " LEFT JOIN ".MAIN_DB_PREFIX."requestmanager_extrafields as ef on (rm.rowid = ef.fk_object)";
			}
		}
		$sql .= ' LEFT JOIN '.MAIN_DB_PREFIX.'user as ur ON ur.rowid = rm.fk_user_resolved';
		$sql .= ' LEFT JOIN '.MAIN_DB_PREFIX.'user as uc ON uc.rowid = rm.fk_user_closed';
		$sql .= ' LEFT JOIN '.MAIN_DB_PREFIX.'user as ua ON ua.rowid = rm.fk_user_author';
		$sql .= ' LEFT JOIN '.MAIN_DB_PREFIX.'user as um ON um.rowid = rm.fk_user_modif';
		$sql .= ' LEFT JOIN '.MAIN_DB_PREFIX.'requestmanager_assigned_user as rmau ON rmau.fk_requestmanager = rm.rowid';
		$sql .= ' LEFT JOIN '.MAIN_DB_PREFIX.'requestmanager_assigned_usergroup as rmaug ON rmaug.fk_requestmanager = rm.rowid';
		$sql .= $join;
		$sql .= ' WHERE rm.entity IN ('.getEntity('requestmanager').')';
		$sql .= $filter;
		$sql .= ' GROUP BY rm.rowid';
		$sql .= $db->order($sortfield, $sortorder);

		return $sql;
	}


    /**
     * Print a line for lists to follow
     *
     * @param   DoliDB          $db                         Doli DB object
     * @param   array           $arrayfields                Array of fields to show
     * @param   stdClass        $obj                        Standard object from db
     * @param   RequestManager  $requestmanagerstatic       RequestManager object
     * @param   Societe         $societestatic_origin       Societe origin object
     * @param   User            $userstatic                 User object
     * @param   int             $lists_follow_last_date     Last date when follow list is viewed
     */
    private static function _listsFollowPrintLineFrom(DoliDB $db, $arrayfields, $obj, RequestManager &$requestmanagerstatic, Societe $societestatic_origin, User $userstatic, $lists_follow_last_date=0)
    {
        global $langs, $conf, $form, $user, $hookmanager, $extrafields, $users_cache, $usergroups_cache, $contact_cache;

        require_once DOL_DOCUMENT_ROOT . '/user/class/user.class.php';
		require_once DOL_DOCUMENT_ROOT . '/user/class/usergroup.class.php';
		require_once DOL_DOCUMENT_ROOT . '/contact/class/contact.class.php';

		if (!isset($users_cache)) $users_cache = array();
		if (!isset($usergroups_cache)) $usergroups_cache = array();
		if (!isset($contact_cache)) $contact_cache = array();

        $now = dol_now();

        // TODO rajouter la Progression calculée et le Coût éffectif

        // societe origin
        $societestatic_origin->id = $obj->fk_soc;
        $societestatic_origin->name = $obj->soc_name_origin;
		$societestatic_origin->name_alias = $obj->soc_name_alias_origin;
        $societestatic_origin->client = $obj->soc_client_origin;
        $societestatic_origin->fournisseur = $obj->soc_fournisseur_origin;
        $societestatic_origin->code_client = $obj->soc_code_client_origin;
        $societestatic_origin->code_fournisseur = $obj->soc_code_fournisseur_origin;

        // requestmanager
        $requestmanagerstatic->id = $obj->rowid;
        $requestmanagerstatic->ref = $obj->ref;
        $requestmanagerstatic->ref_ext = $obj->ref_ext;
        $requestmanagerstatic->fk_parent = $obj->fk_parent;
        $requestmanagerstatic->fk_type = $obj->fk_type;
        $requestmanagerstatic->fk_urgency = $obj->fk_urgency;
        $requestmanagerstatic->label = $obj->label;
        $requestmanagerstatic->socid = $obj->fk_soc;
        $requestmanagerstatic->thirdparty = $societestatic_origin;
	    $requestmanagerstatic->date_deadline = $db->jdate($obj->date_deadline);

        // picto warning for deadline
        $pictoWarning = '';
        if ($requestmanagerstatic->date_deadline) {
            if ($requestmanagerstatic->date_deadline < $now) {
                // alert time is up
                $pictoWarning = ' ' . img_warning($langs->trans("Late"));
            }
        }

        $updatedLineClass = "";
        if ($lists_follow_last_date > 0 && $lists_follow_last_date <= $db->jdate($obj->tms) && $user->id != $obj->fk_user_modif) {
            $updatedLineClass = " rm_my_request_updated_line_color";
        }

        print '<tr class="oddeven">';

        // Ref
        if (!empty($arrayfields['rm.ref']['checked'])) {
            print '<td class="nowrap'.$updatedLineClass.'">';
            print $requestmanagerstatic->getNomUrl(1, 'parent_path', 24) . $pictoWarning;
            print '</td>';
        }

        //External Ref
        if (!empty($arrayfields['rm.ref_ext']['checked'])) {
            print '<td class="nowrap'.$updatedLineClass.'">';
            print $obj->ref_ext;
            print '</td>';
        }

        // Type
        if (!empty($arrayfields['rm.fk_type']['checked'])) {
            print '<td class="nowrap'.$updatedLineClass.'">';
            print $obj->type_label;
            print '</td>';
        }

        // Category
        if (!empty($arrayfields['rm.fk_category']['checked'])) {
            print '<td class="nowrap'.$updatedLineClass.'">';
            print $obj->category_label;
            print '</td>';
        }

        // Label
        if (!empty($arrayfields['rm.label']['checked'])) {
            print '<td class="nowrap'.$updatedLineClass.'">';
            $toprint = dol_trunc($obj->label, 24);
            if (empty($conf->global->MAIN_DISABLE_TRUNC)) {
                $toprint = $form->textwithtooltip($toprint, $obj->label);
            }
            print $toprint;
            print '</td>';
        }

        // Thridparty
        if (!empty($arrayfields['rm.fk_soc']['checked'])) {
            print '<td class="nowrap'.$updatedLineClass.'">';
            print $societestatic_origin->getNomUrl(1, '', 24);
            print '</td>';
        }

		// Requester contacts
		if (!empty($arrayfields['requester_contacts']['checked'])) {
			print '<td class="'.$updatedLineClass.'">';
			$requester_contacts = explode(',', $obj->requester_contacts);
			if (is_array($requester_contacts) && count($requester_contacts) > 0) {
				$toprint = array();
				foreach ($requester_contacts as $contact_id) {
					if ($contact_id > 0) {
						if (!isset($contact_cache[$contact_id])) {
							$contact = new Contact($db);
							$contact->fetch($contact_id);
							$contact_cache[$contact_id] = $contact;
						}
						$toprint[] = '<span class="nowrap">' . $contact_cache[$contact_id]->getNomUrl(1) . '</span>';
					}
				}
				print implode(', ', $toprint);
			}
			print '</td>';
		}

        // Description
        if (!empty($arrayfields['rm.description']['checked'])) {
            print '<td class="nowrap'.$updatedLineClass.'">';
            $toprint = dol_trunc($obj->description, 24);
            if (empty($conf->global->MAIN_DISABLE_TRUNC)) {
                $toprint = $form->textwithtooltip($toprint, $obj->description);
            }
            print $toprint;
            print '</td>';
        }

        // Source
        if (!empty($arrayfields['rm.fk_source']['checked'])) {
            print '<td class="nowrap'.$updatedLineClass.'">';
            print $obj->source_label;
            print '</td>';
        }

        // Urgency
        if (!empty($arrayfields['rm.fk_urgency']['checked'])) {
            $color = $requestmanagerstatic->getColorUrgency();
            print '<td class="nowrap' . $updatedLineClass . '"' . (empty($color) ? '' : ' style="background-color: ' . $color . ' !important;"') . '>';
            print $obj->urgency_label;
            print '</td>';
        }

        // Impact
        if (!empty($arrayfields['rm.fk_impact']['checked'])) {
            print '<td class="nowrap'.$updatedLineClass.'">';
            print $obj->impact_label;
            print '</td>';
        }

        // Priority
        if (!empty($arrayfields['rm.fk_priority']['checked'])) {
            print '<td class="nowrap'.$updatedLineClass.'">';
            print $obj->priority_label;
            print '</td>';
        }

        if (!empty($arrayfields['rm.estimated_duration']['checked']) ||
            !empty($arrayfields['rm.effective_duration']['checked']) ||
            !empty($arrayfields['rm.reported_progress']['checked'])) {
            $can_set_times = $requestmanagerstatic->isRequestTypeOption('display_set_times');
            $children_total_time_spent = $requestmanagerstatic->get_children_total_time_spent();
        }

        // Estimated Duration
        if (!empty($arrayfields['rm.estimated_duration']['checked'])) {
            print '<td class="nowrap'.$updatedLineClass.'">';
            $estimated_duration = $obj->estimated_duration + $children_total_time_spent['total_estimated_duration'];
            if ($estimated_duration > 0) print convertSecondToTime($estimated_duration, 'allhourmin');
            print '</td>';
        }

        // Effective Duration
        if (!empty($arrayfields['rm.effective_duration']['checked'])) {
            print '<td class="nowrap'.$updatedLineClass.'">';
            $effective_duration = $obj->effective_duration + $obj->total_effective_duration + $children_total_time_spent['total_effective_duration'];
            if ($effective_duration > 0) print convertSecondToTime($effective_duration, 'allhourmin');
            print '</td>';
        }

        // Reported Progress
        if (!empty($arrayfields['rm.reported_progress']['checked'])) {
            print '<td class="nowrap'.$updatedLineClass.'">';
            $reported_progress = $children_total_time_spent['total_reported_progress'] > 0 ? ($can_set_times && isset($obj->reported_progress) ? ($obj->reported_progress + $children_total_time_spent['total_reported_progress']) / 2 : $children_total_time_spent['total_reported_progress']) : $obj->reported_progress;
            if ($reported_progress > 0) print $reported_progress . ' %';
            print '</td>';
        }

        // Calculated progression
        if (!empty($arrayfields['calculated_progression']['checked'])) {
            print '<td class="nowrap">';
            $estimated_duration = $obj->estimated_duration + $children_total_time_spent['total_estimated_duration'];
            $effective_duration = $obj->effective_duration + $obj->total_effective_duration + $children_total_time_spent['total_effective_duration'];
            if ($estimated_duration > 0) {
                print ($effective_duration > 0 ? round($effective_duration / $estimated_duration * 100, 2) : 0) . ' %';
            } else {
                print '<span class="opacitymedium">' . $langs->trans('RequestManagerEffectiveDurationNotDefined') . '</span>';
            }
            print '</td>';
        }

        // Effective cost
        if (!empty($arrayfields['rm.effective_cost']['checked'])) {
            print '<td class="nowrap">';
            $effective_cost = $obj->effective_cost + $children_total_time_spent['total_effective_cost'];
            if (!empty($effective_cost)) print price($effective_cost, '', $langs, 1, 0, 0, $conf->currency);
            print '</td>';
        }

        // Date Operation
        if (!empty($arrayfields['rm.date_operation']['checked'])) {
            print '<td class="nowrap'.$updatedLineClass.'" align="center">';
            if ($obj->date_operation > 0) print dol_print_date($db->jdate($obj->date_operation), 'dayhour');
            print '</td>';
        }

        // Date Deadline
        if (!empty($arrayfields['rm.date_deadline']['checked'])) {
            print '<td class="nowrap'.$updatedLineClass.'" align="center">';
            if ($obj->date_deadline > 0) print dol_print_date($db->jdate($obj->date_deadline), 'dayhour');
            print '</td>';
        }

        // Notification requesters
        if (!empty($arrayfields['rm.notify_requester_by_email']['checked'])) {
            print '<td class="nowrap'.$updatedLineClass.'" align="center">';
            print yn($obj->notify_requester_by_email);
            print '</td>';
        }

        // Notification watchers
        if (!empty($arrayfields['rm.notify_watcher_by_email']['checked'])) {
            print '<td class="nowrap'.$updatedLineClass.'" align="center">';
            print yn($obj->notify_watcher_by_email);
            print '</td>';
        }

        // Assigned user
        if (!empty($arrayfields['assigned_users']['checked'])) {
            print '<td class="'.$updatedLineClass.'">';
            $assigned_users = explode(',', $obj->assigned_users);
            if (is_array($assigned_users) && count($assigned_users) > 0) {
                $toprint = array();
                foreach ($assigned_users as $user_id) {
                    if ($user_id > 0) {
                        if (!isset($users_cache[$user_id])) {
                            $assigned_user = new User($db);
                            $assigned_user->fetch($user_id);
                            $users_cache[$user_id] = $assigned_user;
                        }
						$toprint[] = '<span class="nowrap">' . $users_cache[$user_id]->getNomUrl(1) . '</span>';
                    }
                }
                print implode(', ', $toprint);
            }
            print '</td>';
        }

        // Assigned usergroup
        if (!empty($arrayfields['assigned_usergroups']['checked'])) {
            print '<td class="'.$updatedLineClass.'">';
            $assigned_usergroups = explode(',', $obj->assigned_usergroups);
            if (is_array($assigned_usergroups) && count($assigned_usergroups) > 0) {
                $toprint = array();
                foreach ($assigned_usergroups as $usergroup_id) {
                    if (!isset($usergroups_cache[$usergroup_id])) {
                        $assigned_usergroup = new UserGroup($db);
                        $assigned_usergroup->fetch($usergroup_id);
                        $usergroups_cache[$usergroup_id] = $assigned_usergroup;
                    }
					$toprint[] = '<span class="nowrap">' . $usergroups_cache[$usergroup_id]->getFullName($langs) . '</span>';
                }
                print implode(', ', $toprint);
            }
            print '</td>';
        }

        // Notification assigned
        if (!empty($arrayfields['rm.notify_assigned_by_email']['checked'])) {
            print '<td class="nowrap'.$updatedLineClass.'" align="center">';
            print yn($obj->notify_assigned_by_email);
            print '</td>';
        }

        // User resolved
        if (!empty($arrayfields['rm.fk_user_resolved']['checked'])) {
            print '<td class="nowrap'.$updatedLineClass.'">';
            if ($obj->fk_user_resolved > 0) {
                $userstatic->id = $obj->fk_user_resolved;
                $userstatic->firstname = $obj->userresolvedfirstname;
                $userstatic->lastname = $obj->userresolvedlastname;
                $userstatic->email = $obj->userresolvedemail;
                print $userstatic->getNomUrl(1);
            }
            print '</td>';
        }

        // User closed
        if (!empty($arrayfields['rm.fk_user_closed']['checked'])) {
            print '<td class="nowrap'.$updatedLineClass.'">';
            if ($obj->fk_user_closed > 0) {
                $userstatic->id = $obj->fk_user_closed;
                $userstatic->firstname = $obj->userclosedfirstname;
                $userstatic->lastname = $obj->userclosedlastname;
                $userstatic->email = $obj->userclosedemail;
                print $userstatic->getNomUrl(1);
            }
            print '</td>';
        }

        // Date resolved
        if (!empty($arrayfields['rm.date_resolved']['checked'])) {
            print '<td class="nowrap'.$updatedLineClass.'" align="center">';
            if ($obj->date_resolved > 0) print dol_print_date($db->jdate($obj->date_resolved), 'dayhour');
            print '</td>';
        }

        // Date closed
        if (!empty($arrayfields['rm.date_cloture']['checked'])) {
            print '<td class="nowrap'.$updatedLineClass.'" align="center">';
            if ($obj->date_closed > 0) print dol_print_date($db->jdate($obj->date_closed), 'dayhour');
            print '</td>';
        }

        // Author
        if (!empty($arrayfields['rm.fk_user_author']['checked'])) {
            print '<td class="nowrap'.$updatedLineClass.'">';
            if ($obj->fk_user_author > 0) {
                $userstatic->id = $obj->fk_user_author;
                $userstatic->firstname = $obj->userauthorfirstname;
                $userstatic->lastname = $obj->userauthorlastname;
                $userstatic->email = $obj->userauthoremail;
                print $userstatic->getNomUrl(1);
            }
            print '</td>';
        }

        // Modified by
        if (!empty($arrayfields['rm.fk_user_modif']['checked'])) {
            print '<td class="nowrap'.$updatedLineClass.'">';
            if ($obj->fk_user_modif > 0) {
                $userstatic->id = $obj->fk_user_modif;
                $userstatic->firstname = $obj->usermodiffirstname;
                $userstatic->lastname = $obj->usermodiflastname;
                $userstatic->email = $obj->usermodifemail;
                print $userstatic->getNomUrl(1);
            }
            print '</td>';
        }

		// Extra fields
		$extrafieldsobjectkey = 'requestmanager';
		include DOL_DOCUMENT_ROOT.'/core/tpl/extrafields_list_print_fields.tpl.php';

        // Fields from hook
        $parameters = array('arrayfields' => $arrayfields, 'obj' => $obj);
        $reshook = $hookmanager->executeHooks('printFieldListValue', $parameters);    // Note that $action and $object may have been modified by hook
        print $hookmanager->resPrint;

        // Date creation
        if (!empty($arrayfields['rm.datec']['checked'])) {
            print '<td align="center" class="nowrap'.$updatedLineClass.'">';
            print dol_print_date($db->jdate($obj->datec), 'dayhour');
            print '</td>';
        }

        // Date modification
        if (!empty($arrayfields['rm.tms']['checked'])) {
            print '<td align="center" class="nowrap'.$updatedLineClass.'">';
            print dol_print_date($db->jdate($obj->tms), 'dayhour');
            print '</td>';
        }

        // Status
        if (!empty($arrayfields['rm.fk_status']['checked'])) {
            print '<td align="right" class="nowrap'.$updatedLineClass.'">' . $requestmanagerstatic->LibStatut($obj->fk_status, 5, 0, !empty($conf->global->REQUESTMANAGER_SHOW_CHILDREN_REQUEST_STATUS_IN_LIST) ? 3 : -1) . '</td>';
        }

        print '<td class="nowrap'.$updatedLineClass.'"></td>';

        print "</tr>\n";
    }


    /**
     * Print a list to follow
     *
     * @param   DoliDB          $db                         Doli DB object
     * @param   array           $arrayfields                Array of fields to show
     * @param   RequestManager  $requestmanagerstatic       RequestManager object
     * @param   Societe         $societestatic_origin       Societe origin object
     * @param   User            $userstatic                 User object
     * @param   string          $join                       [=''] Join condition where in SQL
     * @param   string          $filter                     [=''] Filter condition where in SQL
     * @param   string          $sortfield                  [=''] List of sort fields, separated by comma. Example: 't1.fielda, t2.fieldb'
     * @param	string          $sortorder                  [=''] List of sort order seprated by comma ('ASC'|'DESC')
     * @param   string          $titleKey                   [=''] Traduction key for title of this list
     * @param   int             $nbCol                      [=1] Nb column to show
     * @param   int             $lists_follow_last_date     Last date when follow list is viewed
     */
    public static function listsFollowPrintListFrom(DoliDB $db, $arrayfields, RequestManager $requestmanagerstatic, Societe $societestatic_origin, User $userstatic, $join='', $filter='', $sortfield='', $sortorder='', $titleKey='', $nbCol=1, $lists_follow_last_date=0)
    {
        global $langs;

        $sql = self::_listsFollowSqlPrepare($db, $requestmanagerstatic, $join, $filter, $sortfield, $sortorder);

        $resql = $db->query($sql);
        if ($resql) {
            print '<tr class="liste_titre">';
            print '<td colspan="' . $nbCol . '">' . $langs->trans($titleKey) . '</td>';
            print '</tr>';

            $i = 0;
            $num = $db->num_rows($resql);
            while ($i < $num) {
                $obj = $db->fetch_object($resql);

                // print a line
                self::_listsFollowPrintLineFrom($db, $arrayfields, $obj, $requestmanagerstatic, $societestatic_origin, $userstatic, $lists_follow_last_date);

                $i++;
            }

            print '<tr>';
            print '<td colspan="' . $nbCol . '" align="center">';
            if ($i <= 0) {
                print $langs->trans('NoRecordFound');
            }
            print '</td>';
            print '</tr>';

            $db->free($resql);
        } else {
            dol_print_error($db);
        }
    }

    /**
     * Save the form in session
     *
     * @param   string      $htmlname           Name of the form
     * @param   string      $key                Key of the data in session
     * @param   string      $ajax_url           Url of the ajax page
     * @return  string                          Output sript js
     */
    public function saveFormToSession($htmlname, $key, $ajax_url='/requestmanager/ajax/save_data.php')
    {
        global $langs;

        $working_img = json_encode(img_picto($langs->trans('RequestManagerFormSavingToSession'), 'working.gif'));
        $saved_img = json_encode(img_picto($langs->trans('RequestManagerFormSavedToSession'), 'statut4'));
        $not_saved_img = json_encode(img_picto($langs->trans('RequestManagerFormNotSavedToSession'), 'statut5'));
        $initial_img = isset($_SESSION['rm_data_save_in_session'][$key]) ? $saved_img : $not_saved_img;

        $key = json_encode($key);
        $ajax_url = json_encode(dol_buildpath($ajax_url, 1));

        $out = <<<SCRIPT
            <script type="text/javascript" language="javascript">
                jQuery(document).ready(function () {
                    var rm_message_form = $('form#$htmlname');
                    if (rm_message_form.length == 0) rm_message_form = $('form[name="$htmlname"]');
                    
                    var rm_saving_status = $('#rm-saving-status');
                    if (rm_saving_status.length > 0) rm_saving_status.empty().append($initial_img);
                    
                    var rm_saving_timer = 0;
                    var rm_saving_delay = 1000;

                    // Save form in session when input of the form changed
                    rm_message_form.find(':input').on('change keyup', function() {
                        rm_event_saving_form_in_session();
                    });

                    // Save form in session when ckeditor of the form changed
                    if (typeof CKEDITOR == "object" && typeof CKEDITOR.instances != "undefined") {
                        $.map(rm_message_form.find(':input'), function(item, idx) {
                            var input = $(item);
                            var input_type = item.localName;
                            var name = input.attr("name");

                            if (input_type == 'textarea' && typeof CKEDITOR.instances[name] != "undefined") {
                                CKEDITOR.instances[name].on('change', function() {
                                    rm_event_saving_form_in_session();
                                });
                            }
                        });
                    }

                    // Save form in session delayed
                    function rm_event_saving_form_in_session() {
                        if (rm_saving_status.length > 0) rm_saving_status.empty().append($working_img);
                        if (rm_saving_timer > 0) { clearTimeout(rm_saving_timer); } 
                        rm_saving_timer = setTimeout(rm_delayed_saving_form_in_session, rm_saving_delay);
                    }

                    // Save form in session delayed
                    function rm_delayed_saving_form_in_session() {
                        rm_save_form_in_session('$htmlname', $key, $ajax_url);
                    }

                    // Save form in session
                    function rm_save_form_in_session(form_name, key, ajax_url) {
                        var form_element = $('form#' + form_name);
                        if (form_element.length == 0) form_element = $('form[name="'+form_name+'"]');
                        
                        if (form_element.length == 0) {
//                            alert('Form to save in session not found.');
                            console.error('Form to save in session not found.');
                            if (rm_saving_status.length > 0) rm_saving_status.empty().append($not_saved_img);
                        } else {
                            var save_data = {};
                            $.map(form_element.find(':input'), function(item, idx) {
                                var input = $(item);
                                var input_type = item.localName;
                                var type = input.attr("type");
                                var name = input.attr("name");
                                var value = '';

                                if (((input_type == 'input' && type != 'button' && type != 'submit') || input_type == 'textarea' || input_type == 'select') &&
                                    typeof name != "undefined" && name.length > 0 && !(name in save_data)
                                ) {
                                    if (type == "checkbox" || type == "radio") { value = $('input[name="' + name + '"]:checked').val(); }
                                    else if (input_type == 'textarea' && typeof CKEDITOR == "object" && typeof CKEDITOR.instances != "undefined" && typeof CKEDITOR.instances[name] != "undefined") { value = CKEDITOR.instances[name].getData(); }
                                    else { value = input.val(); }
    
                                    if (typeof value != "undefined") save_data[name] = value;
                                }
                            });

                            rm_save_data_in_session($.param(save_data), key, ajax_url);
                        }
                    }

                    // Save data in session
                    function rm_save_data_in_session(data_to_save, key, ajax_url) {
                        $.ajax({
                            url: ajax_url,
                            method: "POST",
                            data: {
                                key: key,
                                data: data_to_save
                            },
                            dataType: "json"
                        }).done(function(msg) {
                            if (!(msg.error.length == 0)) {
//                                alert('Error when save in session. msg: ' + msg);
                                console.error('Error when save in session. msg: ', msg);
                                if (rm_saving_status.length > 0) rm_saving_status.empty().append($not_saved_img);
                            } else {
                                if (rm_saving_status.length > 0) rm_saving_status.empty().append($saved_img);
                            }
                        }).fail(function(jqXHR) {
//                            alert('Error when save in session: ' + jqXHR);
                            console.error('Error when save in session. jqXHR: ', jqXHR);
                            if (rm_saving_status.length > 0) rm_saving_status.empty().append($not_saved_img);
                        });
                    }
                });
            </script>
SCRIPT;

        return $out;
    }

    /**
     *     Show a confirmation HTML form or AJAX popup.
     *     Easiest way to use this is with useajax=1.
     *     If you use useajax='xxx', you must also add jquery code to trigger opening of box (with correct parameters)
     *     just after calling this method. For example:
     *       print '<script type="text/javascript">'."\n";
     *       print 'jQuery(document).ready(function() {'."\n";
     *       print 'jQuery(".xxxlink").click(function(e) { jQuery("#aparamid").val(jQuery(this).attr("rel")); jQuery("#dialog-confirm-xxx").dialog("open"); return false; });'."\n";
     *       print '});'."\n";
     *       print '</script>'."\n";
     *
     *     @param  	string		$page        	   	Url of page to call if confirmation is OK
     *     @param	string		$title       	   	Title
     *     @param	string		$question    	   	Question
     *     @param 	string		$action      	   	Action
     *	   @param  	array		$formquestion	   	An array with complementary inputs to add into forms: array(array('label'=> ,'type'=> , ))
     * 	   @param  	string		$selectedchoice  	"" or "no" or "yes"
     * 	   @param  	int			$useajax		   	0=No, 1=Yes, 2=Yes but submit page with &confirm=no if choice is No, 'xxx'=Yes and preoutput confirm box with div id=dialog-confirm-xxx
     *     @param  	int			$height          	Force height of box
     *     @param	int			$width				Force width of box ('999' or '90%'). Ignored and forced to 90% on smartphones.
     *     @param	int			$post				Send by form POST.
     *     @return 	string      	    			HTML ajax code if a confirm ajax popup is required, Pure HTML code if it's an html form
     */
	public function formconfirm($page, $title, $question, $action, $formquestion=array(), $selectedchoice="", $useajax=0, $height=200, $width=500, $post=0)
    {
        global $langs, $conf, $form;
        global $useglobalvars;

        if (!is_object($form)) {
            require_once DOL_DOCUMENT_ROOT . '/core/class/html.form.class.php';
            $form = new Form($this->db);
        }

        $more = '';
        $formconfirm = '';
        $inputok = array();
        $inputko = array();

        // Clean parameters
        $newselectedchoice = empty($selectedchoice) ? "no" : $selectedchoice;
        if ($conf->browser->layout == 'phone') $width = '95%';

        if (is_array($formquestion) && !empty($formquestion)) {
            if ($post) {
                $more .= '<form id="form_dialog_confirm" name="form_dialog_confirm" action="'.$page.'" method="POST" enctype="multipart/form-data">';
                $more .= '<input type="hidden" id="confirm" name="confirm" value="yes">' . "\n";
                $more .= '<input type="hidden" id="action" name="action" value="'.$action.'">' . "\n";
            }
            // First add hidden fields and value
            foreach ($formquestion as $key => $input) {
                if (is_array($input) && !empty($input)) {
                    if ($post && ($input['name'] == "confirm" || $input['name'] == "action")) continue;
                    if ($input['type'] == 'hidden') {
                        $more .= '<input type="hidden" id="' . $input['name'] . '" name="' . $input['name'] . '" value="' . dol_escape_htmltag($input['value'], 1, 1) . '">' . "\n";
                    }
                }
            }

            // Now add questions
            $more .= '<table class="paddingtopbottomonly" width="100%">' . "\n";
            $more .= '<tr><td colspan="3">' . (!empty($formquestion['text']) ? $formquestion['text'] : '') . '</td></tr>' . "\n";
            foreach ($formquestion as $key => $input) {
                if (is_array($input) && !empty($input)) {
                    $size = (!empty($input['size']) ? ' size="' . $input['size'] . '"' : '');

                    if ($input['type'] == 'text') {
                        $more .= '<tr><td>' . $input['label'] . '</td><td colspan="2" align="left"><input type="text" class="flat" id="' . $input['name'] . '" name="' . $input['name'] . '"' . $size . ' value="' . $input['value'] . '" /></td></tr>' . "\n";
                    } else if ($input['type'] == 'password') {
                        $more .= '<tr><td>' . $input['label'] . '</td><td colspan="2" align="left"><input type="password" class="flat" id="' . $input['name'] . '" name="' . $input['name'] . '"' . $size . ' value="' . $input['value'] . '" /></td></tr>' . "\n";
                    } else if ($input['type'] == 'select') {
                        $more .= '<tr><td>';
                        if (!empty($input['label'])) $more .= $input['label'] . '</td><td valign="top" colspan="2" align="left">';
                        $more .= $form->selectarray($input['name'], $input['values'], $input['default'], 1);
                        $more .= '</td></tr>' . "\n";
                    } else if ($input['type'] == 'checkbox') {
                        $more .= '<tr>';
                        $more .= '<td>' . $input['label'] . ' </td><td align="left">';
                        $more .= '<input type="checkbox" class="flat" id="' . $input['name'] . '" name="' . $input['name'] . '"';
                        if (!is_bool($input['value']) && $input['value'] != 'false') $more .= ' checked';
                        if (is_bool($input['value']) && $input['value']) $more .= ' checked';
                        if (isset($input['disabled'])) $more .= ' disabled';
                        $more .= ' /></td>';
                        $more .= '<td align="left">&nbsp;</td>';
                        $more .= '</tr>' . "\n";
                    } else if ($input['type'] == 'radio') {
                        $i = 0;
                        foreach ($input['values'] as $selkey => $selval) {
                            $more .= '<tr>';
                            if ($i == 0) $more .= '<td class="tdtop">' . $input['label'] . '</td>';
                            else $more .= '<td>&nbsp;</td>';
                            $more .= '<td width="20"><input type="radio" class="flat" id="' . $input['name'] . '" name="' . $input['name'] . '" value="' . $selkey . '"';
                            if ($input['disabled']) $more .= ' disabled';
                            $more .= ' /></td>';
                            $more .= '<td align="left">';
                            $more .= $selval;
                            $more .= '</td></tr>' . "\n";
                            $i++;
                        }
                    } else if ($input['type'] == 'date') {
                        $more .= '<tr><td>' . $input['label'] . '</td>';
                        $more .= '<td colspan="2" align="left">';
                        $more .= $form->select_date($input['value'], $input['name'], $input['hour'] ? 1 : 0, $input['minute'] ? 1 : 0, 0, '', 1, $input['addnowlink'] ? 1 : 0, 1);
                        $more .= '</td></tr>' . "\n";
                        $formquestion[] = array('name' => $input['name'] . 'day');
                        $formquestion[] = array('name' => $input['name'] . 'month');
                        $formquestion[] = array('name' => $input['name'] . 'year');
                        $formquestion[] = array('name' => $input['name'] . 'hour');
                        $formquestion[] = array('name' => $input['name'] . 'min');
                    } else if ($input['type'] == 'other') {
                        $more .= '<tr><td>';
                        if (!empty($input['label'])) $more .= $input['label'] . '</td><td colspan="2" align="left">';
                        $more .= $input['value'];
                        $more .= '</td></tr>' . "\n";
                    } else if ($input['type'] == 'onecolumn') {
                        $more .= '<tr><td colspan="3" align="left">';
                        $more .= $input['value'];
                        $more .= '</td></tr>' . "\n";
                    }
                }
            }
            $more .= '</table>' . "\n";
            if ($post) $more .= '</form>';
        }

        // JQUI method dialog is broken with jmobile, we use standard HTML.
        // Note: When using dol_use_jmobile or no js, you must also check code for button use a GET url with action=xxx and check that you also output the confirm code when action=xxx
        // See page product/card.php for example
        if (!empty($conf->dol_use_jmobile)) $useajax = 0;
        if (empty($conf->use_javascript_ajax)) $useajax = 0;

        if ($useajax) {
            $autoOpen = true;
            $dialogconfirm = 'dialog-confirm';
            $button = '';
            if (!is_numeric($useajax)) {
                $button = $useajax;
                $useajax = 1;
                $autoOpen = false;
                $dialogconfirm .= '-' . $button;
            }
            $pageyes = $page . (preg_match('/\?/', $page) ? '&' : '?') . 'action=' . $action . '&token=' . newToken() . '&confirm=yes';
            $pageno = ($useajax == 2 ? $page . (preg_match('/\?/', $page) ? '&' : '?') . 'action=' . $action . '&token=' . newToken() . '&confirm=no' : '');
            // Add input fields into list of fields to read during submit (inputok and inputko)
            if (is_array($formquestion)) {
                foreach ($formquestion as $key => $input) {
                    //print "xx ".$key." rr ".is_array($input)."<br>\n";
                    if (is_array($input) && isset($input['name'])) {
                        // Modification Open-DSI - Begin
                        if (is_array($input['name'])) $inputok = array_merge($inputok, $input['name']);
                        else array_push($inputok, $input['name']);
                        // Modification Open-DSI - End
                    }
                    if (isset($input['inputko']) && $input['inputko'] == 1) array_push($inputko, $input['name']);
                }
            }
            // Show JQuery confirm box. Note that global var $useglobalvars is used inside this template
            $formconfirm .= '<div id="' . $dialogconfirm . '" title="' . dol_escape_htmltag($title) . '" style="display: none;">';
            if (!empty($more)) {
                $formconfirm .= '<div class="confirmquestions">' . $more . '</div>';
            }
            $formconfirm .= ($question ? '<div class="confirmmessage">' . img_help('', '') . ' ' . $question . '</div>' : '');
            $formconfirm .= '</div>' . "\n";

            $formconfirm .= "\n<!-- begin ajax form_confirm page=" . $page . " -->\n";
            $formconfirm .= '<script type="text/javascript">' . "\n";
            $formconfirm .= 'jQuery(document).ready(function() {
            $(function() {
            	$( "#' . $dialogconfirm . '" ).dialog(
            	{
                    autoOpen: ' . ($autoOpen ? "true" : "false") . ',';
            if ($newselectedchoice == 'no') {
                $formconfirm .= '
						open: function() {
            				$(this).parent().find("button.ui-button:eq(2)").focus();
						},';
            }
            if ($post) {
                $formconfirm .= '
                    resizable: false,
                    height: "' . $height . '",
                    width: "' . $width . '",
                    modal: true,
                    closeOnEscape: false,
                    buttons: {
                        "' . dol_escape_js($langs->transnoentities("Yes")) . '": function() {
                            var form_dialog_confirm = $("form#form_dialog_confirm");                            
                            form_dialog_confirm.find("input#confirm").val("yes");
                            form_dialog_confirm.submit();
                            $(this).dialog("close");
                        },
                        "' . dol_escape_js($langs->transnoentities("No")) . '": function() {
                            if (' . ($useajax == 2 ? '1' : '0') . ' == 1) {
                                var form_dialog_confirm = $("form#form_dialog_confirm");                            
                                form_dialog_confirm.find("input#confirm").val("no");
                                form_dialog_confirm.submit();
                            }
                            $(this).dialog("close");
                        }
                    }
                }
                );

            	var button = "' . $button . '";
            	if (button.length > 0) {
                	$( "#" + button ).click(function() {
                		$("#' . $dialogconfirm . '").dialog("open");
        			});
                }
            });
            });
            </script>';
            } else {
                $formconfirm .= '
                    resizable: false,
                    height: "' . $height . '",
                    width: "' . $width . '",
                    modal: true,
                    closeOnEscape: false,
                    buttons: {
                        "' . dol_escape_js($langs->transnoentities("Yes")) . '": function() {
                        	var options="";
                        	var inputok = ' . json_encode($inputok) . ';
                         	var pageyes = "' . dol_escape_js(!empty($pageyes) ? $pageyes : '') . '";
                         	if (inputok.length>0) {
                         		$.each(inputok, function(i, inputname) {
                         			var more = "";
                         			if ($("#" + inputname).attr("type") == "checkbox") { more = ":checked"; }
                         		    if ($("#" + inputname).attr("type") == "radio") { more = ":checked"; }
                         			var inputvalue = $("#" + inputname + more).val();
                         			if (typeof inputvalue == "undefined") { inputvalue=""; }
                         			options += "&" + inputname + "=" + urlencode(inputvalue);
                         		});
                         	}
                         	var urljump = pageyes + (pageyes.indexOf("?") < 0 ? "?" : "") + options;
                         	//alert(urljump);
            				if (pageyes.length > 0) { location.href = urljump; }
                            $(this).dialog("close");
                        },
                        "' . dol_escape_js($langs->transnoentities("No")) . '": function() {
                        	var options = "";
                         	var inputko = ' . json_encode($inputko) . ';
                         	var pageno="' . dol_escape_js(!empty($pageno) ? $pageno : '') . '";
                         	if (inputko.length>0) {
                         		$.each(inputko, function(i, inputname) {
                         			var more = "";
                         			if ($("#" + inputname).attr("type") == "checkbox") { more = ":checked"; }
                         			var inputvalue = $("#" + inputname + more).val();
                         			if (typeof inputvalue == "undefined") { inputvalue=""; }
                         			options += "&" + inputname + "=" + urlencode(inputvalue);
                         		});
                         	}
                         	var urljump=pageno + (pageno.indexOf("?") < 0 ? "?" : "") + options;
                         	//alert(urljump);
            				if (pageno.length > 0) { location.href = urljump; }
                            $(this).dialog("close");
                        }
                    }
                }
                );

            	var button = "' . $button . '";
            	if (button.length > 0) {
                	$( "#" + button ).click(function() {
                		$("#' . $dialogconfirm . '").dialog("open");
        			});
                }
            });
            });
            </script>';
            }
            $formconfirm .= "<!-- end ajax form_confirm -->\n";
        } else {
            $formconfirm .= "\n<!-- begin form_confirm page=" . $page . " -->\n";

            $formconfirm .= '<form method="POST" action="' . $page . '" class="notoptoleftroright">' . "\n";
            $formconfirm .= '<input type="hidden" name="action" value="' . $action . '">' . "\n";
            $formconfirm .= '<input type="hidden" name="token" value="' . $_SESSION['newtoken'] . '">' . "\n";

            $formconfirm .= '<table width="100%" class="valid">' . "\n";

            // Line title
            $formconfirm .= '<tr class="validtitre"><td class="validtitre" colspan="3">' . img_picto('', 'recent') . ' ' . $title . '</td></tr>' . "\n";

            // Line form fields
            if ($more) {
                $formconfirm .= '<tr class="valid"><td class="valid" colspan="3">' . "\n";
                $formconfirm .= $more;
                $formconfirm .= '</td></tr>' . "\n";
            }

            // Line with question
            $formconfirm .= '<tr class="valid">';
            $formconfirm .= '<td class="valid">' . $question . '</td>';
            $formconfirm .= '<td class="valid">';
            $formconfirm .= $form->selectyesno("confirm", $newselectedchoice);
            $formconfirm .= '</td>';
            $formconfirm .= '<td class="valid" align="center"><input class="button valignmiddle" type="submit" value="' . $langs->trans("Validate") . '"></td>';
            $formconfirm .= '</tr>' . "\n";

            $formconfirm .= '</table>' . "\n";

            $formconfirm .= "</form>\n";
            $formconfirm .= '<br>';

            $formconfirm .= "<!-- end form_confirm -->\n";
        }

        return $formconfirm;
    }

    /**
     * 	List of element type for the extrafields parameters
     *
     * 	@return		array		    List of element type for the extrafields parameters
     */
	public function element_type_array()
    {
        global $conf, $hookmanager, $langs;

        $langs->loadLangs(array('requestmanager@requestmanager', 'propal', 'commercial', 'orders', 'bills', 'supplier_proposal', 'suppliers', 'contracts', 'interventions', 'projects', 'trips', 'agenda'));

        // TODO debuger la creation d'une demande lié a une demande avec un champ obligatoire
        // TODO debuger la creation d'une evenement lié a une demande avec un champ obligatoire (lors de la création d'une demande lié a une demande) (a voir pour les event)
        // TODO debuger la modification d'une demande lié a une demande avec un champ obligatoire
        // TODO debuger l'ajout d'un message d'une demande lié a une demande avec un champ obligatoire
//        $element_type_list = array('requestmanager' => $langs->trans('RequestManagerRequest'), 'requestmanager_message' => $langs->trans('RequestManagerRequest') . ' (' . $langs->trans('RequestManagerMessage') . ')');
        $element_type_list = array();
        if (isModEnabled('propal')) {
            $element_type_list += array('propal' => $langs->trans('Proposal')/*, 'propaldet' => $langs->trans('Proposal') . ' (' . $langs->trans('Line') . ')'*/);
        }
        if (isModEnabled('commande')) {
            $element_type_list += array('commande' => $langs->trans('Order')/*, 'commandedet' => $langs->trans('Order') . ' (' . $langs->trans('Line') . ')'*/);
        }
        if (isModEnabled('facture')) {
            $element_type_list += array('facture' => $langs->trans('Invoice')/*, 'facturedet' => $langs->trans('Invoice') . ' (' . $langs->trans('Line') . ')'*/);
        }
        if (isModEnabled('supplier_proposal')) {
            $element_type_list += array('supplier_proposal' => $langs->trans('SupplierProposal')/*, 'supplier_proposaldet' => $langs->trans('SupplierProposal') . ' (' . $langs->trans('Line') . ')'*/);
        }
        if (isModEnabled('fournisseur')) {
            $element_type_list += array('commande_fournisseur' => $langs->trans('SupplierOrder')/*, 'commande_fournisseurdet' => $langs->trans('SupplierOrder') . ' (' . $langs->trans('Line') . ')'*/,
                'facture_fourn' => $langs->trans('SupplierInvoice')/*, 'facture_fourndet' => $langs->trans('SupplierInvoice') . ' (' . $langs->trans('Line') . ')'*/);
        }
        if (isModEnabled('contrat')) {
            $element_type_list += array('contrat' => $langs->trans('Contract')/*, 'contratdet' => $langs->trans('Contract') . ' (' . $langs->trans('Line') . ')'*/);
        }
        if (isModEnabled('ficheinter')) {
            $element_type_list += array('fichinter' => $langs->trans('Intervention')/*, 'fichinterdet' => $langs->trans('Intervention') . ' (' . $langs->trans('Line') . ')'*/);
        }
        if (isModEnabled('projet')) {
            $element_type_list += array('projet' => $langs->trans('Project'), 'projet_task' => $langs->trans('Project') . ' (' . $langs->trans('Task') . ')');
        }
		if (isModEnabled('deplacement')) {
			$element_type_list += array('expensereport' => $langs->trans('ExpenseReport'));
		}
		if (isModEnabled('deplacement')) {
			$element_type_list += array('expensereport' => $langs->trans('ExpenseReport'));
		}
		$element_type_list += array('requestmanager' => $langs->trans('RequestManagerRequest'));
//        if (isModEnabled('agenda')) {
//            $element_type_list += array('actioncomm' => $langs->trans('Event'));
//        }

		if (!is_object($hookmanager)) {
			require_once DOL_DOCUMENT_ROOT . '/core/class/hookmanager.class.php';
			$hookmanager = new HookManager($this->db);
		}

		// Add custom object
		$hookmanager2 = clone $hookmanager; // Génère des erreurs de resultat disparaissant si appelé dans une autre hooks donc on copie la hook
		$hookmanager2->initHooks(array('requestmanagerdao'));
        $parameters = array();
        $reshook = $hookmanager2->executeHooks('addRequestManagerExtraFieldsDictionaryElementType', $parameters); // Note that $action and $object may have been
        if ($reshook) $element_type_list += $hookmanager2->resArray;

        // Sort list
        asort($element_type_list);

        $filtered_element_type_list = array();
        $sql = "SELECT elementtype FROM " . MAIN_DB_PREFIX . "extrafields WHERE entity IN (0," . $conf->entity . ") GROUP BY elementtype";
        $resql = $this->db->query($sql);
        if ($resql) {
            while ($obj = $this->db->fetch_object($resql)) {
                if (isset($element_type_list[$obj->elementtype])) {
                    $filtered_element_type_list[$obj->elementtype] = $element_type_list[$obj->elementtype];
                }
            }
        }

        return $filtered_element_type_list;
    }

    /**
   	 *	Return HTML code of the multiselect of list of request status for the request type.
   	 *  This also set the number of value found into $this->num
   	 *
   	 * @param   array	            $request_types   	    Array of request type selected
     * @param   array|int|string	$selected   	        Array of ID of pre-selected value id
   	 * @param   string		        $htmlname  	            Name of HTML field ('none' for a not editable field)
     * @param	int		            $forcecombo		        Force to use combo box
     * @param	array	            $events			        Event options. Example: array(array('method'=>'getContacts', 'url'=>dol_buildpath('/core/ajax/contacts.php',1), 'htmlname'=>'contactid', 'params'=>array('add-customer-contact'=>'disabled')))
     * @param  	int		            $usesearchtoselect	    Minimum length of input string to start autocomplete
     * @param	int		            $outputmode		        0=HTML select string, 1=Array
     * @param	int		            $limit			        Limit number of answers
     * @param	string	            $morecss		        Add more css styles to the SELECT component
     * @param   string	            $moreparam              Add more parameters onto the select tag. For example 'style="width: 95%"' to avoid select2 component to go over parent container
     * @param   bool                $options_only           Return options only (for ajax treatment)
   	 * @return	int|array|string						    <0 if KO, select output if OK
   	 */
	public function multiselect_request_status($request_types, $selected=-1, $htmlname='', $forcecombo=0, $events=array(), $usesearchtoselect=0, $outputmode=0, $limit=0, $morecss='minwidth100', $moreparam='', $options_only=false)
    {
        return $this->formdictionary->select_dictionary_list('requestmanager', 'requestmanagerstatus', $selected, $htmlname, '', 'rowid', '{{code}} {{label}}', empty($request_types) ? null : array('request_type'=>$request_types), array('label'=>'ASC'), $forcecombo, $events, $usesearchtoselect, $outputmode, $limit, $morecss, $moreparam, $options_only, 1);
    }

    /**
   	 *	Return HTML code of the multiselect of list of extrafields for the element type.
   	 *  This also set the number of value found into $this->num
   	 *
   	 *	@param  array	            $element_types   	    Array of element type selected
     *	@param  array|int|string	$selected   	        Array of ID of pre-selected value id
   	 *	@param  string		        $htmlname  	            Name of HTML field ('none' for a not editable field)
   	 *	@param  string		        $exclude                List of contacts id to exclude
   	 *	@param	string		        $limitto		        Disable answers that are not id in this array list
   	 *	@param	string		        $moreclass		        Add more class to class style
   	 *	@param	bool		        $options_only	        Return options only (for ajax treatment)
   	 * 	@param	int			        $forcecombo		        Force to use combo box
   	 *  @param	array		        $events			        Event options. See FormPeep::add_select_events for more details
   	 *  @param	string		        $moreparam		        Add more parameters onto the select tag. For example 'style="width: 95%"' to avoid select2 component to go over parent container
   	 *  @param	string		        $htmlid			        Html id to use instead of htmlname
     *  @param  int		            $outputmode             0=HTML select string, 1=Array
     *  @param  array               $extrakeyvalue          Extra key for output mode array (ex: key1 => key1 in SQL Request)
     *  @param  int		            $exclude_not_included   0=Disable excluded, 1=Don't include exluded
   	 *	@return	 int|array|string						    <0 if KO, select output if OK
   	 */
	public function multiselect_extrafields($element_types, $selected=-1, $htmlname='', $exclude='', $limitto='', $moreclass='', $options_only=false, $forcecombo=0, $events=array(), $moreparam='', $htmlid='', $outputmode=0, $extrakeyvalue=array(), $exclude_not_included=0)
    {
        global $conf, $langs;

        $element_type_array = $this->element_type_array();

        $nb = 0;
        $sql = "SELECT CONCAT(elementtype, '_', `name`) AS rowid, label AS ref, langs, CONCAT(";
        foreach ($element_type_array as $element_type => $element_label) {
            if (!in_array($element_type, $element_types)) continue;
            $sql .= "IF(elementtype = '" . $this->db->escape($element_type) . "', '" . $this->db->escape($element_label) . "',";
            $nb++;
        }
        $sql .= "''" . str_pad('', $nb, ')');
        $sql .= ", ' - ') AS label, label AS extra_label FROM " . MAIN_DB_PREFIX . "extrafields WHERE entity IN (0," . $conf->entity . ") AND type != 'separate' AND " . (empty($element_types) ? "1=2" : "elementtype IN ('" . implode("', '", $element_types) . "')");
        $none_label = $this->db->escape($langs->trans('None'));
        $sql .= " UNION SELECT 'rm_none' AS rowid, '" . $none_label . "' AS ref, '' AS langs, '" . $none_label . "' AS label, ''";

        return $this->select_query($sql, $selected, $htmlname, 0, $exclude, $limitto, $moreclass, $options_only, $forcecombo, $events, $moreparam, $htmlid, $outputmode, $extrakeyvalue, 1, $exclude_not_included);
    }

    /**
     *	Return HTML code of the multiselect of list of requests.
     *  This also set the number of value found into $this->num
     *
     *	@param  array|int|string	$selected   	        Array of ID of pre-selected value id
     *	@param  string		        $htmlname  	            Name of HTML field ('none' for a not editable field)
     *	@param  int		            $requestofuser          0=Show all otherwise the ID of the user for show only his request
	 *	@param  int		            $onlytimesdefinable     0=Show all, 1=Only the request who the user can set the times
     *	@param  string		        $exclude                List of contacts id to exclude
     *	@param	string		        $limitto		        Disable answers that are not id in this array list
     *	@param	string		        $moreclass		        Add more class to class style
     *	@param	bool		        $options_only	        Return options only (for ajax treatment)
     * 	@param	int			        $forcecombo		        Force to use combo box
     *  @param	array		        $events			        Event options. See FormPeep::add_select_events for more details
     *  @param	string		        $moreparam		        Add more parameters onto the select tag. For example 'style="width: 95%"' to avoid select2 component to go over parent container
     *  @param	string		        $htmlid			        Html id to use instead of htmlname
     *  @param  int		            $outputmode             0=HTML select string, 1=Array
     *  @param  array               $extrakeyvalue          Extra key for output mode array (ex: key1 => key1 in SQL Request)
     *  @param  int		            $ismultiselect          0=Not multiselect, 1=Multiselect
     *  @param  int		            $exclude_not_included   0=Disable excluded, 1=Don't include exluded
     *	@return	 int|array|string						    <0 if KO, select output if OK
     */
	public function select_request($selected=-1, $htmlname='', $requestofuser=0, $onlytimesdefinable=0, $exclude='', $limitto='', $moreclass='', $options_only=false, $forcecombo=0, $events=array(), $moreparam='', $htmlid='', $outputmode=0, $extrakeyvalue=array(), $ismultiselect=0, $exclude_not_included=0)
    {
        global $langs;

        $sql = "SELECT rm.rowid, CONCAT(rm.ref, ' - ', rm.label, ' ( ', s.nom, ' )') AS label";
        $sql .= " FROM " . MAIN_DB_PREFIX . "requestmanager AS rm";
        $sql .= " LEFT JOIN " . MAIN_DB_PREFIX . "c_requestmanager_status as crmst ON crmst.rowid = rm.fk_status";
        $sql .= " LEFT JOIN " . MAIN_DB_PREFIX . "societe as s ON s.rowid = rm.fk_soc";
        if ($onlytimesdefinable > 0) {
			$sql .= " LEFT JOIN " . MAIN_DB_PREFIX . "c_requestmanager_request_type as crmrt ON crmrt.rowid = rm.fk_type";
		}
		if ($requestofuser > 0) {
            $sql .= ' INNER JOIN (';
            $sql .= '   SELECT rm.rowid';
            $sql .= '   FROM ' . MAIN_DB_PREFIX . 'requestmanager as rm';
            $sql .= '   LEFT JOIN ' . MAIN_DB_PREFIX . 'requestmanager_assigned_user as rmau ON rmau.fk_requestmanager = rm.rowid';
            $sql .= '   LEFT JOIN ' . MAIN_DB_PREFIX . 'requestmanager_assigned_usergroup as rmaug ON rmaug.fk_requestmanager = rm.rowid';
            $sql .= '   WHERE rm.entity IN (' . getEntity('requestmanager') . ')';
            require_once DOL_DOCUMENT_ROOT . '/user/class/usergroup.class.php';
            $usergroup_static = new UserGroup($this->db);
            $groupslist = $usergroup_static->listGroupsForUser($requestofuser);
            $sql .= ' AND (rmau.fk_user = ' . $requestofuser;
            if (!empty($groupslist)) {
                $sql .= ' OR rmaug.fk_usergroup IN (' . implode(',', array_keys($groupslist)) . ')';
            }
            $sql .= ')';
            $sql .= '   GROUP BY rm.rowid';
            $sql .= ' ) as assigned ON assigned.rowid = rm.rowid';
        }
        $sql .= " WHERE rm.entity IN (" . getEntity('requestmanager') . ")";
        $sql .= " AND crmst.type IN (" . RequestManager::STATUS_TYPE_IN_PROGRESS . ", " . RequestManager::STATUS_TYPE_RESOLVED . ")";
		if ($onlytimesdefinable > 0) {
			$sql .= " AND crmrt.display_set_times = 1";
		}

        return $this->select_query($sql, $selected, $htmlname, 1, $exclude, $limitto, $moreclass, $options_only, $forcecombo, $events, $moreparam, $htmlid, $outputmode, $extrakeyvalue, $ismultiselect, $exclude_not_included);
    }
}

