<?php
/* Copyright (C) 2007-2012  Laurent Destailleur <eldy@users.sourceforge.net>
 * Copyright (C) 2014       Juanjo Menent       <jmenent@2byte.es>
 * Copyright (C) 2015       Florian Henry       <florian.henry@open-concept.pro>
 * Copyright (C) 2015       Raphaël Doursenaud  <rdoursenaud@gpcsolutions.fr>
 * Copyright (C) 2018       Open-DSI            <support@open-dsi.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * \file    requestmanager/class/requestmanagertimespent.class.php
 * \ingroup requestmanager
 * \brief
 */

require_once DOL_DOCUMENT_ROOT . '/core/class/commonobject.class.php';

/**
 * Class RequestManagerTimeSpent
 */
class RequestManagerTimeSpent extends CommonObject
{
	public $element = 'requestmanager_requestmanagertimespent';
	public $table_element = 'requestmanager_time_spent';
    public $table_parent_element = 'requestmanager';
    public $picto = 'requestmanager@requestmanager';

    /**
     * Array of whitelist of properties keys for this object used for the API
     * @var  array
     *      array('properties_name'=> '' or array('properties_name'=> '' or array(...), ...)
     *      if property is a object and this properties_name value is not a array then get whitelist of his object element
     *      if property is a object and this properties_name value is a array then get whitelist set in the array
     *      if property is a array and this properties_name value is not a array then get all values
     *      if property is a array and this properties_name value is a array then get whitelist set in the array
     */
    static public $API_WHITELIST_OF_PROPERTIES = array(
        "id" => '', "fk_requestmanager" => '', "time_date" => '', "fk_user" => '',
        "duration" => '', "thm" => '', "cost" => '', "note" => '',
        "date_creation" => '', "date_modification" => '',
        "user_creation_id" => '', "user_modification_id" => '',
    );

    /**
     * Array of whitelist of properties keys for this object when is a linked object used for the API
     * @var  array
     *      if empty array then equal at $api_whitelist_of_properties
     *      array('properties_name'=> '' or array('properties_name'=> '' or array(...), ...)
     *      if property is a object and this properties_name value is not a array then get whitelist of his object element
     *      if property is a object and this properties_name value is a array then get whitelist set in the array
     *      if property is a array and this properties_name value is not a array then get all values
     *      if property is a array and this properties_name value is a array then get whitelist set in the array
     */
    static public $API_WHITELIST_OF_PROPERTIES_LINKED_OBJECT = array(
    );

    /**
     * Error message
     * @var string
     */
    public $error;
    /**
     * List of error message
     * @var array
     */
    public $errors;
    /**
     * Old request object before update
     * @var RequestManager
     */
    public $oldcopy;

    /**
     * ID of the request
     * @var int
     */
    public $id;
    /**
     * ID of the request
     * @var int
     */
    public $fk_requestmanager;

    /**
     * Date of the time spent
     * @var int
     */
    public $time_date;
    /**
     * ID of the user of the time spent
     * @var int
     */
    public $fk_user;
    /**
     * Duration of the time spent (in second)
     * @var int
     */
    public $duration;
    /**
     * Average hourly rate of the user of the time spent
     * @var double
     */
    public $thm;
    /**
     * Cost of the time spent
   	 * @var double
   	 */
   	public $cost;
    /**
     * Note of the time spent
     * @var string
     */
    public $note;

    /**
     * Date created of the request
     * @var int
     */
    public $date_creation;
    /**
     * Date modified of the request
     * @var int
     */
    public $date_modification;
    /**
     * Id of the user who created the request
     * @var int
     */
    public $user_creation_id;
    /**
     * User who created the request
     * @var User
     */
    public $user_creation;
    /**
     * Id of the user who modified the request
     * @var int
     */
    public $user_modification_id;
    /**
     * User who modified the request
     * @var User
     */
    public $user_modification;

    /**
	 * Request manager cached
	 * @var RequestManager[int]
	 */
	public static $request_cached = array();


    /**
	 * Constructor
	 *
	 * @param   DoliDb      $db     Database handler
	 */
	public function __construct(DoliDB $db)
	{
		$this->db = $db;
	}

    /**
     * Get all errors
     *
     * @return array    Array of errors
     */
	public function getErrors() {
	    $errors = is_array($this->errors) ? $this->errors : array();
	    $errors = array_merge($errors, (!empty($this->error) ? array($this->error) : array()));

	    return $errors;
    }

	/**
	 *  Create time spent into database
	 *
	 * @param   User    $user           User that creates
	 * @param   bool    $notrigger      false=launch triggers after, true=disable triggers
	 * @return  int                     <0 if KO, Id of created object if OK
	 */
	public function create(User $user, $notrigger = false)
    {
        global $langs;
        $error = 0;
        $this->errors = array();
        $langs->load("requestmanager@requestmanager");

        dol_syslog(__METHOD__ . " user(ID)={$user->id}, fk_requestmanager={$this->fk_requestmanager}, date={$this->time_date}, user_id={$this->fk_user}, note={$this->note}, duration={$this->duration}, notrigger=$notrigger", LOG_DEBUG);

        // Clean parameters
        $this->fk_requestmanager = $this->fk_requestmanager > 0 ? $this->fk_requestmanager : 0;
        $this->fk_user = $this->fk_user > 0 ? $this->fk_user : 0;
        $this->duration = $this->duration > 0 ? $this->duration : 0;
        $this->thm = $this->thm > 0 ? $this->thm : 0;
        $this->note = trim($this->note);

        // Check parameters
        if (!($this->fk_requestmanager > 0)) {
            $this->errors[] = $langs->trans("ErrorFieldRequired", $langs->transnoentitiesnoconv("TechnicalID") . ' - ' . $langs->transnoentitiesnoconv("RequestManagerRequest"));
            $error++;
        }
        if (!($this->fk_user > 0)) {
            $this->errors[] = $langs->trans("ErrorFieldRequired", $langs->transnoentitiesnoconv("User"));
            $error++;
        }
        if (!($this->duration > 0)) {
            $this->errors[] = $langs->trans("ErrorFieldRequired", $langs->transnoentitiesnoconv("NewTimeSpent"));
            $error++;
        }
        if ($error) {
            dol_syslog(__METHOD__ . " Errors check parameters: " . $this->errorsToString(), LOG_ERR);
            return -3;
        }

        if (empty($this->thm)) {
            $user_static = new User($this->db);
            $result = $user_static->fetch($this->fk_user);
            if ($result < 0) {
                $this->error = $user->error;
                $this->errors = $user->errors;
                return -1;
            } elseif ($result == 0) {
                $langs->load("errors");
                $this->errors[] = $langs->trans("User") . ' (ID: ' . $this->fk_user . ') : ' . $langs->trans('ErrorRecordNotFound');
                return -1;
            }
            $this->thm = $user_static->thm > 0 ? $user_static->thm : 0;
        }

        $now = dol_now();
        $this->db->begin();

        // Add time spent
        $sql = 'INSERT INTO ' . MAIN_DB_PREFIX . $this->table_element . ' (';
        $sql .= '  fk_requestmanager, time_date, fk_user, note, duration, thm, datec, fk_user_author';
        $sql .= ') VALUES (' . $this->fk_requestmanager;
        $sql .= ", '" . $this->db->idate($this->time_date) . "'";
        $sql .= ", " . $this->fk_user;
        $sql .= ", " . (!empty($this->note) ? "'" . $this->db->escape($this->note) . "'" : "NULL");
        $sql .= ", " . $this->duration;
        $sql .= ", '" . $this->db->escape($this->thm) . "'";
        $sql .= ", '" . $this->db->idate($now) . "'";
        $sql .= ", " . $user->id;
        $sql .= ")";

        $resql = $this->db->query($sql);
        if (!$resql) {
            $error++;
            $this->errors[] = 'Error ' . $this->db->lasterror();
            dol_syslog(__METHOD__ . " SQL: " . $sql . "; Error: " . $this->db->lasterror(), LOG_ERR);
        }

        if (!$error) {
            $this->id = $this->db->last_insert_id(MAIN_DB_PREFIX . $this->table_element);

            if (!$error && !$notrigger) {
                // Call trigger
                $result = $this->call_trigger('REQUESTMANAGERTIMESPENT_CREATE', $user);
                if ($result < 0) {
                    $error++;
                    dol_syslog(__METHOD__ . " Errors call trigger: " . $this->errorsToString(), LOG_ERR);
                }
                // End call triggers
            }
        }

        if (!$error) {
            $result = $this->update_total_time_spent($user, $notrigger);
            if ($result < 0) $error++;
        }

        // Commit or rollback
        if ($error) {
            $this->db->rollback();
            return -1 * $error;
        } else {
            $this->cost = price2num($this->duration * $this->thm / 3600, 'MT');

            $this->db->commit();
            dol_syslog(__METHOD__ . " success", LOG_DEBUG);
            return $this->id;
        }
    }

	/**
	 *  Load time spent in memory from the database
	 *
     * @param   int     $id         Id object
     * @param   string	$ref		Reference of request
     * @param   string	$refext		External reference of request
     * @param   int     $entity     Force entity (list id separate by coma)
	 * @return  int                 <0 if KO, 0 if not found, >0 if OK
	 */
	public function fetch($id)
    {
        global $langs;
        $this->errors = array();
        $langs->load("requestmanager@requestmanager");

        dol_syslog(__METHOD__ . " id=$id", LOG_DEBUG);

        $sql = 'SELECT';
        $sql .= ' t.rowid';
        $sql .= ', t.fk_requestmanager';
        $sql .= ', t.time_date';
        $sql .= ', t.fk_user';
        $sql .= ', t.duration';
        $sql .= ', t.thm';
        $sql .= ', t.note';
        $sql .= ', t.datec';
        $sql .= ', t.tms';
        $sql .= ', t.fk_user_author';
        $sql .= ', t.fk_user_modif';
        $sql .= ' FROM ' . MAIN_DB_PREFIX . $this->table_element . ' as t';
        $sql .= ' WHERE t.rowid = ' . $id;

        $resql = $this->db->query($sql);
        if ($resql) {
            if ($obj = $this->db->fetch_object($resql)) {
                $this->id                   = $obj->rowid;
                $this->fk_requestmanager    = $obj->fk_requestmanager;
                $this->time_date            = $this->db->jdate($obj->time_date);
                $this->fk_user              = $obj->fk_user;
                $this->duration             = $obj->duration;
                $this->thm                  = $obj->thm;
                $this->note                 = $obj->note;
                $this->cost                 = price2num($this->duration * $this->thm / 3600, 'MT');
                $this->date_creation        = $this->db->jdate($obj->datec);
                $this->date_modification    = $this->db->jdate($obj->tms);
                $this->user_creation_id     = $obj->fk_user_author;
                $this->user_modification_id = $obj->fk_user_modif;

                $this->db->free($resql);
                return 1;
            } else {
                return 0;
            }
        } else {
            $this->errors[] = 'Error ' . $this->db->lasterror();
            dol_syslog(__METHOD__ . " SQL: " . $sql . "; Error: " . $this->db->lasterror(), LOG_ERR);

            return -1;
        }
    }

    /**
	 *  Update request into database
	 *
	 * @param   User    $user           User that modifies
	 * @param   bool    $notrigger      false=launch triggers after, true=disable triggers
	 * @return  int                     <0 if KO, >0 if OK
	 */
	public function update(User $user, $notrigger = false)
    {
        global $langs;
        $error = 0;
        $this->errors = array();
        $langs->load("requestmanager@requestmanager");

        dol_syslog(__METHOD__ . " user(ID)={$user->id}, id={$this->id}, fk_requestmanager={$this->fk_requestmanager}, date={$this->time_date}, user_id={$this->fk_user}, note={$this->note}, duration={$this->duration}, notrigger=$notrigger", LOG_DEBUG);

        // Clean parameters
        $this->id = $this->id > 0 ? $this->id : 0;
        $this->fk_requestmanager = $this->fk_requestmanager > 0 ? $this->fk_requestmanager : 0;
        $this->fk_user = $this->fk_user > 0 ? $this->fk_user : 0;
        $this->duration = $this->duration > 0 ? $this->duration : 0;
        $this->thm = $this->thm > 0 ? $this->thm : 0;
        $this->note = trim($this->note);

        // Check parameters
        if (!($this->id > 0)) {
            $this->errors[] = $langs->trans("ErrorFieldRequired", $langs->transnoentitiesnoconv("TechnicalID"));
            $error++;
        }
        if (!($this->fk_requestmanager > 0)) {
            $this->errors[] = $langs->trans("ErrorFieldRequired", $langs->transnoentitiesnoconv("TechnicalID") . ' - ' . $langs->transnoentitiesnoconv("RequestManagerRequest"));
            $error++;
        }
        if (!($this->fk_user > 0)) {
            $this->errors[] = $langs->trans("ErrorFieldRequired", $langs->transnoentitiesnoconv("User"));
            $error++;
        }
        if (!($this->duration > 0)) {
            $this->errors[] = $langs->trans("ErrorFieldRequired", $langs->transnoentitiesnoconv("NewTimeSpent"));
            $error++;
        }
        if ($error) {
            dol_syslog(__METHOD__ . " Errors check parameters: " . $this->errorsToString(), LOG_ERR);
            return -3;
        }

        if (empty($this->thm)) {
            $user_static = new User($this->db);
            $result = $user_static->fetch($this->fk_user);
            if ($result < 0) {
                $this->error = $user->error;
                $this->errors = $user->errors;
                return -1;
            } elseif ($result == 0) {
                $langs->load("errors");
                $this->errors[] = $langs->trans("User") . ' (ID: ' . $this->fk_user . ') : ' . $langs->trans('ErrorRecordNotFound');
                return -1;
            }
            $this->thm = $user_static->thm > 0 ? $user_static->thm : 0;
        }

        $now = dol_now();
        $this->db->begin();

        // Update time spent
        $sql = 'UPDATE ' . MAIN_DB_PREFIX . $this->table_element . ' SET' .
            "  time_date = '" . $this->db->idate($this->time_date) . "'" .
			", fk_requestmanager = " . $this->fk_requestmanager .
            ", fk_user = " . $this->fk_user .
            ", note = " . (!empty($this->note) ? "'" . $this->db->escape($this->note) . "'" : "NULL") .
            ", duration = " . $this->duration .
            ", thm = '" . $this->db->escape($this->thm) . "'" .
            ", fk_user_modif = " . $user->id .
            ", tms = '" . $this->db->idate($now) . "'" .
            " WHERE rowid = " . $this->id;

        $resql = $this->db->query($sql);
        if (!$resql) {
            $error++;
            $this->errors[] = 'Error ' . $this->db->lasterror();
            dol_syslog(__METHOD__ . " SQL: " . $sql . "; Error: " . $this->db->lasterror(), LOG_ERR);
        }

        if (!$error && !$notrigger) {
            // Call trigger
            $result = $this->call_trigger('REQUESTMANAGERTIMESPENT_MODIFY', $user);
            if ($result < 0) {
                $error++;
                dol_syslog(__METHOD__ . " Errors call trigger: " . $this->errorsToString(), LOG_ERR);
            }
            // End call triggers
        }

		if (!$error && isset($this->oldcopy->id) && $this->oldcopy->id > 0) {
			$result = $this->oldcopy->update_total_time_spent($user, $notrigger);
			if ($result < 0) {
				$this->error = $this->oldcopy->error;
				$this->errors = $this->oldcopy->errors;
				$error++;
			}
		}

		if (!$error) {
            $result = $this->update_total_time_spent($user, $notrigger);
            if ($result < 0) $error++;
        }

        // Commit or rollback
        if ($error) {
            $this->db->rollback();

            return -1 * $error;
        } else {
            $this->db->commit();
            dol_syslog(__METHOD__ . " success", LOG_DEBUG);

            return 1;
        }
    }

	/**
	 *  Delete request in database
	 *
     * @param   User    $user           User that deletes
	 * @param   bool    $notrigger      false=launch triggers after, true=disable triggers
	 * @return  int                     <0 if KO, >0 if OK
	 */
	public function delete(User $user, $notrigger = false)
	{
        global $langs;
        $error = 0;
        $this->errors = array();
        $langs->load("requestmanager@requestmanager");

        dol_syslog(__METHOD__ . " user(ID)={$user->id}, id={$this->id}, fk_requestmanager={$this->fk_requestmanager}, notrigger=$notrigger", LOG_DEBUG);

        // Clean parameters
        $this->id = $this->id > 0 ? $this->id : 0;

        // Check parameters
        if (!($this->id > 0)) {
            $this->errors[] = $langs->trans("ErrorFieldRequired", $langs->transnoentitiesnoconv("TechnicalID"));
            $error++;
        }
        if ($error) {
            dol_syslog(__METHOD__ . " Errors check parameters: " . $this->errorsToString(), LOG_ERR);
            return -3;
        }

		$this->db->begin();

		// User is mandatory for trigger call
		if (!$notrigger) {
			// Call trigger
			$result = $this->call_trigger('REQUESTMANAGERTIMESPENT_DELETE', $user);
			if ($result < 0) {
				$error++;
				dol_syslog(__METHOD__ . " Errors call trigger: " . $this->errorsToString(), LOG_ERR);
			}
			// End call triggers
		}

		// Remove request
		if (!$error) {
            $sql = "DELETE FROM " . MAIN_DB_PREFIX . $this->table_element .
                " WHERE rowid = " . $this->id;

			$resql = $this->db->query($sql);
			if (!$resql) {
				$error++;
				$this->errors[] = 'Error ' . $this->db->lasterror();
				dol_syslog(__METHOD__ . " SQL: " . $sql . "; Error: " . $this->db->lasterror(), LOG_ERR);
			}
		}

        if (!$error) {
            $result = $this->update_total_time_spent($user, $notrigger);
            if ($result < 0) $error++;
        }

		if (!$error) {
			$this->db->commit();
			dol_syslog(__METHOD__ . " success", LOG_DEBUG);

			return 1;
		} else {
			$this->db->rollback();

			return -1;
		}
	}

    /**
     * 	Update information on the total time spent for the request
     *
     * @param   User    $user           User that modifies
     * @param   int		$notrigger	    0=launch triggers after, 1=disable triggers
     * @return  int                     <0 si ko, >0 si ok
     */
	public function update_total_time_spent(User $user, $notrigger=0)
    {
        global $langs;
        $error = 0;
        $this->errors = array();
        $langs->load("requestmanager@requestmanager");

        dol_syslog(__METHOD__ . " user(ID)={$user->id}, fk_requestmanager={$this->fk_requestmanager}, notrigger=$notrigger", LOG_DEBUG);

        // Clean parameters
        $this->fk_requestmanager = $this->fk_requestmanager > 0 ? $this->fk_requestmanager : 0;

        // Check parameters
        if (!($this->fk_requestmanager > 0)) {
            $this->errors[] = $langs->trans("ErrorFieldRequired", $langs->transnoentitiesnoconv("TechnicalID") . ' - ' . $langs->transnoentitiesnoconv("RequestManagerRequest"));
            $error++;
        }
        if ($error) {
            dol_syslog(__METHOD__ . " Errors check parameters: " . $this->errorsToString(), LOG_ERR);
            return -3;
        }

        $now = dol_now();
        $this->db->begin();

        // Update total time spent on request
        $sql = "UPDATE " . MAIN_DB_PREFIX . $this->table_parent_element . " As rm " .
            " LEFT JOIN (" .
            "   SELECT fk_requestmanager, SUM(duration) AS total_duration, SUM(duration * thm / 3600) AS total_effective_cost" .
            "   FROM " . MAIN_DB_PREFIX . $this->table_element .
            "   GROUP BY fk_requestmanager" .
            " ) AS t ON t.fk_requestmanager = rm.rowid" .
            " SET rm.total_effective_duration = IF(t.total_duration IS NULL, 0, t.total_duration)" .
            ", rm.effective_cost = IF(t.total_effective_cost IS NULL, 0, t.total_effective_cost)" .
            ", rm.fk_user_modif = " . $user->id .
            ", rm.tms = '" . $this->db->idate($now) . "'" .
            " WHERE rm.rowid = " . $this->fk_requestmanager;

        $resql = $this->db->query($sql);
        if (!$resql) {
            $error++;
            $this->errors[] = 'Error ' . $this->db->lasterror();
            dol_syslog(__METHOD__ . " SQL: " . $sql . "; Error: " . $this->db->lasterror(), LOG_ERR);
        }

        if (!$notrigger) {
            // Call trigger
            $result = $this->call_trigger('REQUESTMANAGERTIMESPENT_UPDATETOTAL', $user);
            if ($result < 0) $error++;
            // End call triggers
        }

        // Commit or rollback
        if ($error) {
            $this->db->rollback();

            return -1 * $error;
        } else {
            $this->db->commit();
            dol_syslog(__METHOD__ . " success", LOG_DEBUG);

            return 1;
        }
    }

    /**
     *  Get the total times spent for each user
     *
     * @param   int		    $fk_requestmanager	Id of the request
     * @param   int		    $with_object	    0=only user id, 1=include the object of the user
     * @return  int|array                       <0 if KO otherwise the list of total times spent for each user
     */
	public function get_sum_time_spent($fk_requestmanager=0, $with_object=0)
    {
        global $langs;
        $error = 0;
        $this->errors = array();
        $langs->load("requestmanager@requestmanager");

        dol_syslog(__METHOD__ . " id={$this->id}, with_object=$with_object", LOG_DEBUG);

        // Clean parameters
        $this->fk_requestmanager = $fk_requestmanager > 0 ? $fk_requestmanager : ($this->fk_requestmanager > 0 ? $this->fk_requestmanager : 0);

        // Check parameters
        if (!($this->fk_requestmanager > 0)) {
            $this->errors[] = $langs->trans("ErrorFieldRequired", $langs->transnoentitiesnoconv("TechnicalID") . ' - ' . $langs->transnoentitiesnoconv("RequestManagerRequest"));
            $error++;
        }
        if ($error) {
            dol_syslog(__METHOD__ . " Errors check parameters: " . $this->errorsToString(), LOG_ERR);
            return -3;
        }

        $sql = 'SELECT fk_user, SUM(duration) AS total_duration, SUM(duration * thm / 3600) AS total_cost' .
            ' FROM ' . MAIN_DB_PREFIX . $this->table_element .
            ' WHERE fk_requestmanager = ' . $this->fk_requestmanager .
            " GROUP BY fk_user";
        $resql = $this->db->query($sql);
        if (!$resql) {
            $this->errors[] = 'Error ' . $this->db->lasterror();
            dol_syslog(__METHOD__ . " SQL: " . $sql . "; Error: " . $this->db->lasterror(), LOG_ERR);
            return -1;
        }

        $times_spent = array();
        while ($obj = $this->db->fetch_object($resql)) {
            $times_spent[$obj->fk_user] = array(
                'user_id' => $obj->fk_user,
                'duration' => $obj->total_duration,
                'cost' => price2num($obj->total_cost, 'MT'),
            );
            if ($with_object) {
                require_once DOL_DOCUMENT_ROOT . '/user/class/user.class.php';
                $user = new User($this->db);
                $user->fetch($obj->fk_user);
                $times_spent[$obj->fk_user]['user'] = $user;
            }
        }
        $this->db->free($resql);

        return $times_spent;
    }

    /**
     *  Get the total times spent for a user in a period
     *
     * @param   int		    $fk_requestmanager	Id of the request
     * @param   int		    $user_id	        Id of the user
     * @param   int		    $date_nav_start	    Date begin
     * @param   int		    $date_nav_end	    Date end
     * @return  int                             <0 if KO otherwise duration in second
     */
	public function get_sum_consumed_time($fk_requestmanager, $user_id, $date_nav_start, $date_nav_end)
    {
        global $langs;
        $error = 0;
        $this->errors = array();
        $langs->load("requestmanager@requestmanager");

        dol_syslog(__METHOD__ . " id={$this->id}, fk_requestmanager=$fk_requestmanager, user_id=$user_id, date_nav_start=$date_nav_start, date_nav_end=$date_nav_end", LOG_DEBUG);

        // Clean parameters
        $this->fk_requestmanager = $fk_requestmanager > 0 ? $fk_requestmanager : ($this->fk_requestmanager > 0 ? $this->fk_requestmanager : 0);
        $this->fk_user = $user_id > 0 ? $user_id : ($this->fk_user > 0 ? $this->fk_user : 0);

        // Check parameters
        if (!($this->fk_user > 0)) {
            $this->errors[] = $langs->trans("ErrorFieldRequired", $langs->transnoentitiesnoconv("User"));
            $error++;
        }
        if ($error) {
            dol_syslog(__METHOD__ . " Errors check parameters: " . $this->errorsToString(), LOG_ERR);
            return -3;
        }

        $sql = "SELECT SUM(duration) AS total_duration" .
            " FROM " . MAIN_DB_PREFIX . $this->table_element .
            " WHERE fk_user = " . $this->fk_user .
            " AND DATE(time_date) BETWEEN DATE('" . $this->db->idate($date_nav_start) . "') AND DATE('" . $this->db->idate($date_nav_end) . "')" .
            ($this->fk_requestmanager > 0 ? " AND fk_requestmanager = " . $this->fk_requestmanager : '') .
            " GROUP BY fk_user";
        $resql = $this->db->query($sql);
        if (!$resql) {
            $this->errors[] = 'Error ' . $this->db->lasterror();
            dol_syslog(__METHOD__ . " SQL: " . $sql . "; Error: " . $this->db->lasterror(), LOG_ERR);
            return -1;
        }

        $times_spent = 0;
        if ($obj = $this->db->fetch_object($resql)) {
            $times_spent = $obj->total_duration;
        }
        $this->db->free($resql);

        return $times_spent;
    }

	/**
	 *  Can write this line ?
	 *
	 * @param	bool	$include_assigned	Include assigned users for validated the user
	 * @param	int		$force_reload		1=Force reload
	 * @return  int							<0 if KO, ==0 if No, >0 if Yes
	 */
	public function canWriteLine($include_assigned = false, $force_reload = 0)
	{
		global $user;

		if ($this->id > 0 && $this->fk_requestmanager > 0 && $this->fk_user > 0) {
			$request = $this->getRequestManger($this->fk_requestmanager, $force_reload);
			if (!is_object($request)) {
				return -1;
			}

			$can_edit = $request->isRequestTypeOption('manage_times') && $request->isRequestTypeOption('display_set_times') &&
				($request->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS || $request->statut_type == RequestManager::STATUS_TYPE_RESOLVED) &&
				($user->rights->requestmanager->timesspent->manage || $user->admin || $user->id == $this->fk_user || ($include_assigned && in_array($user->id, $request->assigned_user_ids)));

			return $can_edit ? 1 : 0;
		}

		return 0;
	}

	/**
	 *  Get and loaded request from cache
	 *
	 * @param	int						$request_id		Request ID
	 * @param	int						$force_reload	1=Force reload
	 * @return	int|RequestManager						<0 if KO, 0 if not found, Object if OK
	 */
	public function getRequestManger($request_id, $force_reload = 0)
	{
		global $langs;

		if (!isset(self::$request_cached[$request_id]) || !empty($force_reload)) {
			dol_include_once('/requestmanager/class/requestmanager.class.php');
			$request = new RequestManager($this->db);
			$result = $request->fetch($request_id);
			if ($result < 0) {
				$this->error = $request->error;
				$this->errors = $request->errors;
				return -1;
			} elseif ($result == 0) {
				$langs->loadLangs(array("errors", "requestmanager@requestmanager"));
				$this->errors[] = $langs->trans('ErrorRecordNotFound') . ' - ' . $langs->trans('RequestManagerRequest') . ' ( ID: ' . $request_id . ' )';
				return -1;
			}

			self::$request_cached[$request_id] = $request;
		}

		return self::$request_cached[$request_id];
	}
}