<?php
/* Copyright (C) 2003      Rodolphe Quiedeville <rodolphe@quiedeville.org>
 * Copyright (C) 2004-2015 Laurent Destailleur  <eldy@users.sourceforge.net>
 * Copyright (C) 2005-2016 Regis Houssin        <regis.houssin@capnetworks.com>
 * Copyright (C) 2018      Open-DSI             <support@open-dsi.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * 	\defgroup   requestmanager     Module Request Manager
 *  \brief      Example of a module descriptor.
 *				Such a file must be copied into htdocs/requestmanager/core/modules directory.
 *  \file       htdocs/requestmanager/core/modules/modRequestManager.class.php
 *  \ingroup    requestmanager
 *  \brief      Description and activation file for module Request Manager
 */
include_once DOL_DOCUMENT_ROOT . '/core/modules/DolibarrModules.class.php';
dol_include_once('/requestmanager/class/requestmanager.class.php');

/**
 *  Description and activation class for module Request Manager
 */
class modRequestManager extends DolibarrModules
{
	/**
	 * Constructor. Define names, constants, directories, boxes, permissions
	 *
	 * @param DoliDB $db Database handler
	 */
	public function __construct($db)
	{
		global $langs, $conf;

		$this->db = $db;
		$langs->load('opendsi@requestmanager');

		// Id for module (must be unique).
		// Use here a free id (See in Home -> System information -> Dolibarr for list of used modules id).
		$this->numero = 163018;        // TODO Go on page http://wiki.dolibarr.org/index.php/List_of_modules_id to reserve id number for your module
		// Key text used to identify module (for permissions, menus, etc...)
		$this->rights_class = 'requestmanager';

		// Family can be 'crm','financial','hr','projects','products','ecm','technic','interface','other'
		// It is used to group modules by family in module setup page
		$this->family = 'easya';
		// Module position in the family
		$this->module_position = 500;
		// Gives the possibility to the module, to provide his own family info and position of this family (Overwrite $this->family and $this->module_position. Avoid this)
		$this->familyinfo = array('easya' => array('position' => '001', 'label' => $langs->trans("easyaFamily")));

		// Module label (no space allowed), used if translation string 'ModuleXXXName' not found (where XXX is value of numeric property 'numero' of module)
		$this->name = preg_replace('/^mod/i', '', get_class($this));
		// Module description, used if translation string 'ModuleXXXDesc' not found (where XXX is value of numeric property 'numero' of module)
		$this->description = "Description of module Request Manager";
		$this->descriptionlong = "";
		$this->editor_name = '<b>Easya Solutions</b>';
		$this->editor_web = 'https://easya.solutions';
		$this->editor_url = "https://easya.solutions";
		$this->editor_email = 'support@easya.solutions';

		// Possible values for version are: 'development', 'experimental', 'dolibarr', 'dolibarr_deprecated' or a version string like 'x.y.z'
		$this->version = trim(file_get_contents(__DIR__.'/../../VERSION'));
		$this->url_last_version = 'https://git.open-dsi.fr/dolibarr-extension/'.strtolower($this->name).'/-/raw/2024/VERSION';
		// Key used in llx_const table to save module status enabled/disabled (where MYMODULE is value of property name of module in uppercase)
		$this->const_name = 'MAIN_MODULE_' . strtoupper($this->name);

		$easya_info = json_decode(file_get_contents(__DIR__ . '/../../.easya_info.json'));
		$this->phpmin = explode('.', $easya_info->php_min_version);                    // Minimum version of PHP required by module
		$this->need_dolibarr_version = explode('.', $easya_info->dlb_min_version);    // Minimum version of Dolibarr required by module

		// Name of image file used for this module.
		// If file is in theme/yourtheme/img directory under name object_pictovalue.png, use this->picto='pictovalue'
		// If file is in module/img directory under name object_pictovalue.png, use this->picto='pictovalue@module'
		if ((float) DOL_VERSION <= 11.0) {
			$this->picto = 'opendsi@' . strtolower($this->name);
		} else {
			$this->picto = 'opendsi_big@' . strtolower($this->name);
		}

		// Defined all module parts (triggers, login, substitutions, menus, css, etc...)
		// for default path (eg: /mymodule/core/xxxxx) (0=disable, 1=enable)
		// for specific path of parts (eg: /mymodule/core/modules/barcode)
		// for specific css file (eg: /mymodule/css/mymodule.css.php)
		//$this->module_parts = array(
		//                        	'triggers' => 0,                                 	// Set this to 1 if module has its own trigger directory (core/triggers)
		//							'login' => 0,                                    	// Set this to 1 if module has its own login method directory (core/login)
		//							'substitutions' => 0,                            	// Set this to 1 if module has its own substitution function file (core/substitutions)
		//							'menus' => 0,                                    	// Set this to 1 if module has its own menus handler directory (core/menus)
		//							'theme' => 0,                                    	// Set this to 1 if module has its own theme directory (theme)
		//                        	'tpl' => 0,                                      	// Set this to 1 if module overwrite template dir (core/tpl)
		//							'barcode' => 0,                                  	// Set this to 1 if module has its own barcode directory (core/modules/barcode)
		//							'models' => 0,                                   	// Set this to 1 if module has its own models directory (core/modules/xxx)
		//							'css' => array('/mymodule/css/mymodule.css.php'),	// Set this to relative path of css file if module has its own css file
		//							'js' => array('/mymodule/js/mymodule.js'),          // Set this to relative path of js file if module must load a js on all pages
		//							'hooks' => array('hookcontext1','hookcontext2',...) // Set here all hooks context managed by module. You can also set hook context 'all'
		//							'dir' => array('output' => 'othermodulename'),      // To force the default directories names
		//							'workflow' => array('WORKFLOW_MODULE1_YOURACTIONTYPE_MODULE2'=>array('enabled'=>'! empty($conf->module1->enabled) && ! empty($conf->module2->enabled)', 'picto'=>'yourpicto@mymodule')) // Set here all workflow context managed by module
		//                        );
		$this->module_parts = array(
			'dictionaries' => 1,
			'substitutions' => 1,
			'css' => array('/requestmanager/css/requestmanager.css.php'),
			'hooks' => array('main', 'login'),
			'models' => 1,
			'triggers' => 1
		);

		// Data directories to create when module is enabled.
		// Example: this->dirs = array("/mymodule/temp");
		$this->dirs = array('/requestmanager/', '/doctemplates/requestmanager/');

		// Config pages. Put here list of php page, stored into mymodule/admin directory, to use to setup module.
		$this->config_page_url = array("setup.php@requestmanager");

		// Dependencies
		$this->hidden = false;            // A condition to hide module
		$this->depends = array('modAgenda', 'modAdvanceDictionaries');        // List of modules id that must be enabled if this module is enabled
		$this->requiredby = array();    // List of modules id to disable if this one is disabled
		$this->conflictwith = array();    // List of modules id this module is in conflict with
		// $this->phpmin = array(5, 0);                    // Minimum version of PHP required by module
		// $this->need_dolibarr_version = array(4, 0);    // Minimum version of Dolibarr required by module
		$this->langfiles = array("requestmanager@requestmanager", "opendsi@requestmanager");
		$langs->load('requestmanager@requestmanager');

		// Constants
		// List of particular constants to add when module is enabled (key, 'chaine', value, desc, visible, 'current' or 'allentities', deleteonunactive)
		// Example: $this->const=array(0=>array('MYMODULE_MYNEWCONST1','chaine','myvalue','This is a constant to add',1),
		//                             1=>array('MYMODULE_MYNEWCONST2','chaine','myvalue','This is another constant to add',0, 'current', 1)
		// );
		$this->const = array(
			0 => array('REQUESTMANAGER_REF_ADDON', 'chaine', 'mod_requestmanager_ref_simple', '', 0, 'current'),
			1 => array('REQUESTMANAGER_REF_UNIVERSAL_MASK', 'chaine', 'RI{yy}{mm}-{000000}', '', 0, 'current'),
			2 => array('REQUESTMANAGER_REFEXT_ADDON', 'chaine', 'mod_requestmanager_refext_simple', '', 0, 'current'),
			3 => array('REQUESTMANAGER_REFEXT_UNIVERSAL_MASK', 'chaine', 'RE{yy}{mm}-{000000}', '', 0, 'current'),
			4 => array('REQUESTMANAGER_ADDON_PDF_ODT_PATH', 'chaine', 'DOL_DATA_ROOT/doctemplates/requestmanager', '', 0, 'current'),
			5 => array('REQUESTMANAGER_POSITION_BLOC_OBJECT_LINKED', 'chaine', 'top', '', 0, 'current'),
			6 => array('REQUESTMANAGER_POSITION_LINK_NEW_OBJECT_LINKED', 'chaine', 'top', '', 0, 'current'),
			7 => array('REQUESTMANAGER_POSITION_BLOC_DOCUMENTS', 'chaine', 'top', '', 0, 'current'),
			//8 => array('REQUESTMANAGER_CREATE_NOTIFY', 'chaine', '1', '', 0, 'current'),
			//9 => array('REQUESTMANAGERMESSAGE_CREATE_NOTIFY', 'chaine', '1', '', 0, 'current'),
			//10 => array('REQUESTMANAGER_SET_ASSIGNED_NOTIFY', 'chaine', '1', '', 0, 'current'),
			//11 => array('REQUESTMANAGER_STATUS_MODIFY_NOTIFY', 'chaine', '1', '', 0, 'current'),
			//12 => array('REQUESTMANAGER_NOTIFICATION_ASSIGNED_BY_EMAIL', 'chaine', '1', '', 0, 'current'),
			13 => array('REQUESTMANAGER_TIMESLOTS_ACTIVATE', 'chaine', '0', '', 0, 'current'),
			14 => array('REQUESTMANAGER_TIMESLOTS_PERIODS', 'chaine', '1W 08h00 - 5W 12h00, 1W 14h00 - 5W 18h00', '', 0, 'current'),
		);

		// Array to add new pages in new tabs
		// Example: $this->tabs = array('objecttype:+tabname1:Title1:mylangfile@mymodule:$user->rights->mymodule->read:/mymodule/mynewtab1.php?id=__ID__',  					// To add a new tab identified by code tabname1
		//                              'objecttype:+tabname2:SUBSTITUTION_Title2:mylangfile@mymodule:$user->rights->othermodule->read:/mymodule/mynewtab2.php?id=__ID__',  	// To add another new tab identified by code tabname2. Label will be result of calling all substitution functions on 'Title2' key.
		//                              'objecttype:-tabname:NU:conditiontoremove');                                                     										// To remove an existing tab identified by code tabname
		// where objecttype can be
		// 'categories_x'	  to add a tab in category view (replace 'x' by type of category (0=product, 1=supplier, 2=customer, 3=member)
		// 'contact'          to add a tab in contact view
		// 'contract'         to add a tab in contract view
		// 'group'            to add a tab in group view
		// 'intervention'     to add a tab in intervention view
		// 'invoice'          to add a tab in customer invoice view
		// 'invoice_supplier' to add a tab in supplier invoice view
		// 'member'           to add a tab in fundation member view
		// 'opensurveypoll'	  to add a tab in opensurvey poll view
		// 'order'            to add a tab in customer order view
		// 'order_supplier'   to add a tab in supplier order view
		// 'payment'		  to add a tab in payment view
		// 'payment_supplier' to add a tab in supplier payment view
		// 'product'          to add a tab in product view
		// 'propal'           to add a tab in propal view
		// 'project'          to add a tab in project view
		// 'stock'            to add a tab in stock view
		// 'thirdparty'       to add a tab in third party view
		// 'user'             to add a tab in user view
		$this->tabs = array(
			'thirdparty:+rm_request_list:SUBSTITUTION_RMLISTTABLABEL:requestmanager@requestmanager:$user->rights->requestmanager->lire:/requestmanager/list.php?socid=__ID__',
			'thirdparty:+rm_request_planning_list:SUBSTITUTION_RMPLANNINGLISTTABLABEL:requestmanager@requestmanager:$conf->global->REQUESTMANAGER_PLANNING_ACTIVATE && $user->rights->requestmanager->planning->lire:/requestmanager/list.php?planning=1&socid=__ID__',
			'contact:+rm_request_list:SUBSTITUTION_RMLISTTABLABEL:requestmanager@requestmanager:$user->rights->requestmanager->lire:/requestmanager/list.php?contact_id=__ID__',
			'requestmanager:+history:History:history@history:$user->rights->history->read&&$conf->history->enabled:/history/history.php?type_object=requestmanager&id=__ID__',
			'questionnaire:-invitation:NU:$conf->questionnaire->enabled',
			'questionnaire:-answer:NU:$conf->questionnaire->enabled',
			//'product:+requestmanager_product:SUBSTITUTION_requestmanagerTABLABEL:requestmanager@requestmanager:$user->rights->requestmanager->lire:/requestmanager/product_tab.php?id=__ID__',
		);

		if (!isset($conf->requestmanager) || !isset($conf->requestmanager->enabled)) {
			$conf->requestmanager = new stdClass();
			$conf->requestmanager->enabled = 0;
		}

		// Dictionaries
		$this->dictionaries = array();
		/* Example:
		$this->dictionaries=array(
			'langs'=>'mylangfile@mymodule',
			'tabname'=>array(MAIN_DB_PREFIX."table1",MAIN_DB_PREFIX."table2",MAIN_DB_PREFIX."table3"),		// List of tables we want to see into dictonnary editor
			'tablib'=>array("Table1","Table2","Table3"),													// Label of tables
			'tabsql'=>array('SELECT f.rowid as rowid, f.code, f.label, f.active FROM '.MAIN_DB_PREFIX.'table1 as f','SELECT f.rowid as rowid, f.code, f.label, f.active FROM '.MAIN_DB_PREFIX.'table2 as f','SELECT f.rowid as rowid, f.code, f.label, f.active FROM '.MAIN_DB_PREFIX.'table3 as f'),	// Request to select fields
			'tabsqlsort'=>array("label ASC","label ASC","label ASC"),																					// Sort order
			'tabfield'=>array("code,label","code,label","code,label"),																					// List of fields (result of select to show dictionary)
			'tabfieldvalue'=>array("code,label","code,label","code,label"),																				// List of fields (list of fields to edit a record)
			'tabfieldinsert'=>array("code,label","code,label","code,label"),																			// List of fields (list of fields for insert)
			'tabrowid'=>array("rowid","rowid","rowid"),																									// Name of columns with primary key (try to always name it 'rowid')
			'tabcond'=>array($conf->mymodule->enabled,$conf->mymodule->enabled,$conf->mymodule->enabled)												// Condition to show each dictionary
		);
		*/

		// Boxes
		// Add here list of php file(s) stored in core/boxes that contains class to show a box.
		$this->boxes = array();            // List of boxes
		// Example:
		//$this->boxes=array(
		//    0=>array('file'=>'myboxa.php@mymodule','note'=>'','enabledbydefaulton'=>'Home'),
		//    1=>array('file'=>'myboxb.php@mymodule','note'=>''),
		//    2=>array('file'=>'myboxc.php@mymodule','note'=>'')
		//);

		// Cronjobs
		$this->cronjobs = array();            // List of cron jobs entries to add
		// Example: $this->cronjobs=array(0=>array('label'=>'My label', 'jobtype'=>'method', 'class'=>'/dir/class/file.class.php', 'objectname'=>'MyClass', 'method'=>'myMethod', 'parameters'=>'', 'comment'=>'Comment', 'frequency'=>2, 'unitfrequency'=>3600, 'test'=>true),
		//                                1=>array('label'=>'My label', 'jobtype'=>'command', 'command'=>'', 'parameters'=>'', 'comment'=>'Comment', 'frequency'=>1, 'unitfrequency'=>3600*24, 'test'=>true)
		// );

		// Permissions
		$this->rights = array();        // Permission array used by this module
		$r = 0;

		$this->rights[$r][0] = 163032;
		$this->rights[$r][1] = 'Consulter les demandes';
		$this->rights[$r][3] = 0;
		$this->rights[$r][4] = 'lire';
		$r++;

		$this->rights[$r][0] = 163033;
		$this->rights[$r][1] = 'Créer/modifier les demandes';
		$this->rights[$r][3] = 0;
		$this->rights[$r][4] = 'creer';
		$r++;

		$this->rights[$r][0] = 163034;
		$this->rights[$r][1] = 'Supprimer les demandes';
		$this->rights[$r][3] = 0;
		$this->rights[$r][4] = 'supprimer';
		$r++;

		$this->rights[$r][0] = 163035;
		$this->rights[$r][1] = 'Reouvrir les demandes cloturées';
		$this->rights[$r][3] = 0;
		$this->rights[$r][4] = 'cloturer';
		$r++;

		$this->rights[$r][0] = 163036;
		$this->rights[$r][1] = 'Voir les documents des demandes';
		$this->rights[$r][3] = 0;
		$this->rights[$r][4] = 'read_file';
		$r++;

		$this->rights[$r][0] = 163052;
		$this->rights[$r][1] = 'Voir le(s) groupe(s) utilisateur(s) en charge sur les tiers pour la planification des demandes';
		$this->rights[$r][3] = 0;
		$this->rights[$r][4] = 'usergroup_in_charge';
		$this->rights[$r][5] = 'lire';
		$r++;

		$this->rights[$r][0] = 163053;
		$this->rights[$r][1] = 'Creer/Modifier le(s) groupe(s) utilisateur(s) en charge sur les tiers pour la planification des demandes';
		$this->rights[$r][3] = 0;
		$this->rights[$r][4] = 'usergroup_in_charge';
		$this->rights[$r][5] = 'manage';
		$r++;

		$this->rights[$r][0] = 163054;
		$this->rights[$r][1] = 'Voir la planification des demandes';
		$this->rights[$r][3] = 0;
		$this->rights[$r][4] = 'planning';
		$this->rights[$r][5] = 'lire';
		$r++;

		$this->rights[$r][0] = 163055;
		$this->rights[$r][1] = 'Gère la planification des demandes';
		$this->rights[$r][3] = 0;
		$this->rights[$r][4] = 'planning';
		$this->rights[$r][5] = 'manage';
		$r++;

		$this->rights[$r][0] = 163058;
		$this->rights[$r][1] = 'Affiche les boutons des statuts précedant des demandes';
		$this->rights[$r][3] = 0;
		$this->rights[$r][4] = 'last_status';
		$r++;

		$this->rights[$r][0] = 163088;
		$this->rights[$r][1] = 'Gère les temps consommés des demandes';
		$this->rights[$r][3] = 0;
		$this->rights[$r][4] = 'timesspent';
		$this->rights[$r][5] = 'manage';
		$r++;

		// Add here list of permission defined by an id, a label, a boolean and two constant strings.
		// Example:
		// $this->rights[$r][0] = $this->numero + $r;	// Permission id (must not be already used)
		// $this->rights[$r][1] = 'Permision label';	// Permission label
		// $this->rights[$r][3] = 1; 					// Permission by default for new user (0/1)
		// $this->rights[$r][4] = 'level1';				// In php code, permission will be checked by test if ($user->rights->permkey->level1->level2)
		// $this->rights[$r][5] = 'level2';				// In php code, permission will be checked by test if ($user->rights->permkey->level1->level2)
		// $r++;

		// Main menu entries
		$this->menu = array();            // List of menus to add
		$r = 0;

		$this->menu[$r] = array(
			'fk_menu' => '',
			'type' => 'top',
			'titre' => 'RequestManagerMenuTop',
			'mainmenu' => 'requestmanager',
			'url' => !empty($conf->global->REQUESTMANAGER_FOLLOW_LIST_ON_MAIN_MENU) ? '/requestmanager/lists_follow.php' : '/requestmanager/list.php',
			'langs' => 'requestmanager@requestmanager',
			'position' => 360,
			'enabled' => '$conf->requestmanager->enabled',
			'perms' => '$user->rights->requestmanager->lire',
			'target' => '',
			'user' => 0
		);
		$r++;

		$this->menu[$r] = array(
			'fk_menu' => '',
			'type' => 'top',
			'titre' => 'RequestManagerMenuTopCreateFast',
			'mainmenu' => 'requestmanager',
			'url' => '/requestmanager/card.php?action=create',
			'langs' => 'requestmanager@requestmanager',
			'position' => 361,
			'enabled' => '$conf->requestmanager->enabled&&!empty($conf->global->REQUESTMANAGER_DISPLAY_BUTTON_CREATE_REQ_IN_TOP_MENU)&&empty($conf->global->REQUESTMANAGER_DISPLAY_BUTTON_CREATE_REQ_IN_TOP_RIGHT_MENU)',
			'perms' => '$user->rights->requestmanager->creer',
			'target' => '',
			'user' => 0
		);
		$r++;

		$this->menu[$r] = array(
			'fk_menu' => 'fk_mainmenu=requestmanager',
			'type' => 'left',
			'titre' => 'RequestManagerMenuLeftTitle',
			'leftmenu' => 'requestmanager',
			'url' => !empty($conf->global->REQUESTMANAGER_FOLLOW_LIST_ON_MAIN_MENU) ? '/requestmanager/lists_follow.php' : '/requestmanager/list.php',
			'langs' => 'requestmanager@requestmanager',
			'position' => 100,
			'enabled' => '$conf->requestmanager->enabled',
			'perms' => '$user->rights->requestmanager->lire',
			'target' => '',
			'user' => 0
		);
		$r++;

		$this->menu[$r] = array(
			'fk_menu' => 'fk_mainmenu=requestmanager,fk_leftmenu=requestmanager',
			'type' => 'left',
			'titre' => 'RequestManagerMenuLeftNewRequest',
			'leftmenu' => 'requestmanager_new',
			'url' => '/requestmanager/card.php?action=create',
			'langs' => 'requestmanager@requestmanager',
			'position' => 200,
			'enabled' => '$conf->requestmanager->enabled',
			'perms' => '$user->rights->requestmanager->creer',
			'target' => '',
			'user' => 0
		);
		$r++;

		$this->menu[$r] = array(
			'fk_menu' => 'fk_mainmenu=requestmanager,fk_leftmenu=requestmanager',
			'type' => 'left',
			'titre' => 'RequestManagerMenuLeftList',
			'leftmenu' => 'requestmanager_list',
			'url' => '/requestmanager/list.php',
			'langs' => 'requestmanager@requestmanager',
			'position' => 300,
			'enabled' => '$conf->requestmanager->enabled',
			'perms' => '$user->rights->requestmanager->lire',
			'target' => '',
			'user' => 0
		);
		$r++;

		$this->menu[$r] = array(
			'fk_menu' => 'fk_mainmenu=requestmanager,fk_leftmenu=requestmanager_list',
			'type' => 'left',
			'titre' => 'RequestManagerMenuLeftListNotAssigned',
			'leftmenu' => 'requestmanager_list_not_assigned',
			'url' => '/requestmanager/list.php?notassigned=1',
			'langs' => 'requestmanager@requestmanager',
			'position' => 310,
			'enabled' => '$conf->requestmanager->enabled',
			'perms' => '$user->rights->requestmanager->lire',
			'target' => '',
			'user' => 0
		);
		$r++;

		$this->menu[$r] = array(
			'fk_menu' => 'fk_mainmenu=requestmanager,fk_leftmenu=requestmanager_list',
			'type' => 'left',
			'titre' => 'RequestManagerMenuLeftListNotResolved',
			'leftmenu' => 'requestmanager_list_not_resolved',
			'url' => '/requestmanager/list.php?status_type=-2',
			'langs' => 'requestmanager@requestmanager',
			'position' => 320,
			'enabled' => '$conf->requestmanager->enabled',
			'perms' => '$user->rights->requestmanager->lire',
			'target' => '',
			'user' => 0
		);
		$r++;

		$this->menu[$r] = array(
			'fk_menu' => 'fk_mainmenu=requestmanager,fk_leftmenu=requestmanager_list',
			'type' => 'left',
			'titre' => 'RequestManagerMenuLeftListClosed',
			'leftmenu' => 'requestmanager_list_closed',
			'url' => '/requestmanager/list.php?status_type=' . RequestManager::STATUS_TYPE_CLOSED,
			'langs' => 'requestmanager@requestmanager',
			'position' => 330,
			'enabled' => '$conf->requestmanager->enabled',
			'perms' => '$user->rights->requestmanager->lire',
			'target' => '',
			'user' => 0
		);
		$r++;

		$this->menu[$r] = array(
			'fk_menu' => 'fk_mainmenu=requestmanager,fk_leftmenu=requestmanager',
			'type' => 'left',
			'titre' => 'RequestManagerMenuLeftMyList',
			'leftmenu' => 'requestmanager_mylist',
			'url' => '/requestmanager/lists_follow.php',
			'langs' => 'requestmanager@requestmanager',
			'position' => 400,
			'enabled' => '$conf->requestmanager->enabled',
			'perms' => '$user->rights->requestmanager->lire',
			'target' => '',
			'user' => 0
		);
		$r++;

		$this->menu[$r] = array(
			'fk_menu' => 'fk_mainmenu=requestmanager,fk_leftmenu=requestmanager',
			'type' => 'left',
			'titre' => 'RequestManagerMenuLeftPlanningList',
			'leftmenu' => 'requestmanager_plist',
			'url' => '/requestmanager/list.php?planning=1',
			'langs' => 'requestmanager@requestmanager',
			'position' => 500,
			'enabled' => '$conf->requestmanager->enabled && !empty($conf->global->REQUESTMANAGER_PLANNING_ACTIVATE)',
			'perms' => '$user->rights->requestmanager->planning->lire',
			'target' => '',
			'user' => 0
		);
		$r++;

		$this->menu[$r] = array(
			'fk_menu' => 'fk_mainmenu=requestmanager,fk_leftmenu=requestmanager_plist',
			'type' => 'left',
			'titre' => 'RequestManagerMenuLeftPlanningListInCharge',
			'leftmenu' => 'requestmanager_plist_in_charge',
			'url' => '/requestmanager/list.php?planning=1&mylist=1',
			'langs' => 'requestmanager@requestmanager',
			'position' => 510,
			'enabled' => '$conf->requestmanager->enabled && !empty($conf->global->REQUESTMANAGER_PLANNING_ACTIVATE)',
			'perms' => '$user->rights->requestmanager->planning->lire',
			'target' => '',
			'user' => 0
		);
		$r++;

		$this->menu[$r] = array(
			'fk_menu' => 'fk_mainmenu=requestmanager,fk_leftmenu=requestmanager',
			'type' => 'left',
			'titre' => 'RequestManagerMenuLeftFollowSurvey',
			'leftmenu' => 'requestmanager_survey_list',
			'url' => '/requestmanager/survey.php',
			'langs' => 'requestmanager@requestmanager',
			'position' => 600,
			'enabled' => '$conf->requestmanager->enabled && $conf->questionnaire->enabled',
			'perms' => '$user->rights->requestmanager->lire && $user->rights->questionnaire->read',
			'target' => '',
			'user' => 0
		);
		$r++;

		$this->menu[$r] = array(
			'fk_menu' => 'fk_mainmenu=requestmanager,fk_leftmenu=requestmanager',
			'type' => 'left',
			'titre' => 'RequestManagerMenuLeftTimeSpent',
			'leftmenu' => 'requestmanager_time_spent',
			'url' => '/requestmanager/time.php',
			'langs' => 'requestmanager@requestmanager',
			'position' => 700,
			'enabled' => '$conf->requestmanager->enabled',
			'perms' => '$user->rights->requestmanager->lire',
			'target' => '',
			'user' => 0
		);
		$r++;

		// Add here entries to declare new menus
		//
		// Example to declare a new Top Menu entry and its Left menu entry:
		// $this->menu[$r]=array(	'fk_menu'=>'',			                // '' if this is a top menu. For left menu, use 'fk_mainmenu=xxx' or 'fk_mainmenu=xxx,fk_leftmenu=yyy' where xxx is mainmenucode and yyy is a leftmenucode
		//							'type'=>'top',			                // This is a Top menu entry
		//							'titre'=>'MyModule top menu',
		//							'mainmenu'=>'mymodule',
		//							'leftmenu'=>'mymodule',
		//							'url'=>'/mymodule/pagetop.php',
		//							'langs'=>'mylangfile@mymodule',	        // Lang file to use (without .lang) by module. File must be in langs/code_CODE/ directory.
		//							'position'=>100,
		//							'enabled'=>'$conf->mymodule->enabled',	// Define condition to show or hide menu entry. Use '$conf->mymodule->enabled' if entry must be visible if module is enabled.
		//							'perms'=>'1',			                // Use 'perms'=>'$user->rights->mymodule->level1->level2' if you want your menu with a permission rules
		//							'target'=>'',
		//							'user'=>2);				                // 0=Menu for internal users, 1=external users, 2=both
		// $r++;
		//
		// Example to declare a Left Menu entry into an existing Top menu entry:
		// $this->menu[$r]=array(	'fk_menu'=>'fk_mainmenu=xxx',		    // '' if this is a top menu. For left menu, use 'fk_mainmenu=xxx' or 'fk_mainmenu=xxx,fk_leftmenu=yyy' where xxx is mainmenucode and yyy is a leftmenucode
		//							'type'=>'left',			                // This is a Left menu entry
		//							'titre'=>'MyModule left menu',
		//							'mainmenu'=>'xxx',
		//							'leftmenu'=>'mymodule',
		//							'url'=>'/mymodule/pagelevel2.php',
		//							'langs'=>'mylangfile@mymodule',	        // Lang file to use (without .lang) by module. File must be in langs/code_CODE/ directory.
		//							'position'=>100,
		//							'enabled'=>'$conf->mymodule->enabled',  // Define condition to show or hide menu entry. Use '$conf->mymodule->enabled' if entry must be visible if module is enabled. Use '$leftmenu==\'system\'' to show if leftmenu system is selected.
		//							'perms'=>'1',			                // Use 'perms'=>'$user->rights->mymodule->level1->level2' if you want your menu with a permission rules
		//							'target'=>'',
		//							'user'=>2);				                // 0=Menu for internal users, 1=external users, 2=both
		// $r++;

		// Exports
		$r = 1;

		// Example:
		// $this->export_code[$r]=$this->rights_class.'_'.$r;
		// $this->export_label[$r]='MyModule';	// Translation key (used only if key ExportDataset_xxx_z not found)
		// $this->export_enabled[$r]='1';                               // Condition to show export in list (ie: '$user->id==3'). Set to 1 to always show when module is enabled.
		// $this->export_icon[$r]='generic:MyModule';					// Put here code of icon then string for translation key of module name
		// $this->export_permission[$r]=array(array("mymodule","level1","level2"));
		// $this->export_fields_array[$r]=array('s.rowid'=>"IdCompany",'s.nom'=>'CompanyName','s.address'=>'Address','s.zip'=>'Zip','s.town'=>'Town','s.fk_pays'=>'Country','s.phone'=>'Phone','s.siren'=>'ProfId1','s.siret'=>'ProfId2','s.ape'=>'ProfId3','s.idprof4'=>'ProfId4','s.code_compta'=>'CustomerAccountancyCode','s.code_compta_fournisseur'=>'SupplierAccountancyCode','f.rowid'=>"InvoiceId",'f.facnumber'=>"InvoiceRef",'f.datec'=>"InvoiceDateCreation",'f.datef'=>"DateInvoice",'f.total'=>"TotalHT",'f.total_ttc'=>"TotalTTC",'f.tva'=>"TotalVAT",'f.paye'=>"InvoicePaid",'f.fk_statut'=>'InvoiceStatus','f.note'=>"InvoiceNote",'fd.rowid'=>'LineId','fd.description'=>"LineDescription",'fd.price'=>"LineUnitPrice",'fd.tva_tx'=>"LineVATRate",'fd.qty'=>"LineQty",'fd.total_ht'=>"LineTotalHT",'fd.total_tva'=>"LineTotalTVA",'fd.total_ttc'=>"LineTotalTTC",'fd.date_start'=>"DateStart",'fd.date_end'=>"DateEnd",'fd.fk_product'=>'ProductId','p.ref'=>'ProductRef');
		// $this->export_TypeFields_array[$r]=array('t.date'=>'Date', 't.qte'=>'Numeric', 't.poids'=>'Numeric', 't.fad'=>'Numeric', 't.paq'=>'Numeric', 't.stockage'=>'Numeric', 't.fadparliv'=>'Numeric', 't.livau100'=>'Numeric', 't.forfait'=>'Numeric', 's.nom'=>'Text','s.address'=>'Text','s.zip'=>'Text','s.town'=>'Text','c.code'=>'Text','s.phone'=>'Text','s.siren'=>'Text','s.siret'=>'Text','s.ape'=>'Text','s.idprof4'=>'Text','s.code_compta'=>'Text','s.code_compta_fournisseur'=>'Text','s.tva_intra'=>'Text','f.facnumber'=>"Text",'f.datec'=>"Date",'f.datef'=>"Date",'f.date_lim_reglement'=>"Date",'f.total'=>"Numeric",'f.total_ttc'=>"Numeric",'f.tva'=>"Numeric",'f.paye'=>"Boolean",'f.fk_statut'=>'Status','f.note_private'=>"Text",'f.note_public'=>"Text",'fd.description'=>"Text",'fd.subprice'=>"Numeric",'fd.tva_tx'=>"Numeric",'fd.qty'=>"Numeric",'fd.total_ht'=>"Numeric",'fd.total_tva'=>"Numeric",'fd.total_ttc'=>"Numeric",'fd.date_start'=>"Date",'fd.date_end'=>"Date",'fd.special_code'=>'Numeric','fd.product_type'=>"Numeric",'fd.fk_product'=>'List:product:label','p.ref'=>'Text','p.label'=>'Text','p.accountancy_code_sell'=>'Text');
		// $this->export_entities_array[$r]=array('s.rowid'=>"company",'s.nom'=>'company','s.address'=>'company','s.zip'=>'company','s.town'=>'company','s.fk_pays'=>'company','s.phone'=>'company','s.siren'=>'company','s.siret'=>'company','s.ape'=>'company','s.idprof4'=>'company','s.code_compta'=>'company','s.code_compta_fournisseur'=>'company','f.rowid'=>"invoice",'f.facnumber'=>"invoice",'f.datec'=>"invoice",'f.datef'=>"invoice",'f.total'=>"invoice",'f.total_ttc'=>"invoice",'f.tva'=>"invoice",'f.paye'=>"invoice",'f.fk_statut'=>'invoice','f.note'=>"invoice",'fd.rowid'=>'invoice_line','fd.description'=>"invoice_line",'fd.price'=>"invoice_line",'fd.total_ht'=>"invoice_line",'fd.total_tva'=>"invoice_line",'fd.total_ttc'=>"invoice_line",'fd.tva_tx'=>"invoice_line",'fd.qty'=>"invoice_line",'fd.date_start'=>"invoice_line",'fd.date_end'=>"invoice_line",'fd.fk_product'=>'product','p.ref'=>'product');
		// $this->export_dependencies_array[$r]=array('invoice_line'=>'fd.rowid','product'=>'fd.rowid'); // To add unique key if we ask a field of a child to avoid the DISTINCT to discard them
		// $this->export_sql_start[$r]='SELECT DISTINCT ';
		// $this->export_sql_end[$r]  =' FROM ('.MAIN_DB_PREFIX.'facture as f, '.MAIN_DB_PREFIX.'facturedet as fd, '.MAIN_DB_PREFIX.'societe as s)';
		// $this->export_sql_end[$r] .=' LEFT JOIN '.MAIN_DB_PREFIX.'product as p on (fd.fk_product = p.rowid)';
		// $this->export_sql_end[$r] .=' WHERE f.fk_soc = s.rowid AND f.rowid = fd.fk_facture';
		// $this->export_sql_order[$r] .=' ORDER BY s.nom';
		// $r++;
	}

	/**
	 * Function called when module is enabled.
	 * The init function add constants, boxes, permissions and menus (defined in constructor) into Dolibarr database.
	 * It also creates data directories
	 *
	 * @param	string		$options	Options when enabling module ('', 'noboxes')
	 * @return	int						1 if OK, 0 if KO
	 */
	public function init($options = '')
	{
		global $langs, $conf;
		$langs->load('requestmanager@requestmanager');

		$sql = array();
		dol_include_once('/requestmanager/lib/requestmanager_init.lib.php');
		$sql = array_merge($sql, requestmanager_sql());

		// Create extrafields
		include_once DOL_DOCUMENT_ROOT . '/core/class/extrafields.class.php';
		$extrafields = new ExtraFields($this->db);
		$result = $extrafields->addExtraField('rm_ipbx_id', $langs->trans('RequestManagerIPBX'), 'varchar', 1, 255, 'actioncomm', 1, 0, '', null, 0, '0', 0, 1, '', '');
		$result = $extrafields->addExtraField('rm_timeslots_separator', $langs->trans('RequestManagerTimeSlotsSeparatorTitle'), 'separate', 400, '', 'contrat', 0, 0, '', null, 1, '', 1, 0, '');
		$result = $extrafields->addExtraField('rm_timeslots_periods', $langs->trans('RequestManagerTimeSlotsPeriodsName'), 'text', 401, 2000, 'contrat', 0, 0, '', null, 1, '', 1, 0, '');
		$result = $extrafields->addExtraField('rm_last_check_follow_list_date', $langs->trans('RequestManagerLastCheckFollowListDate'), 'datetime', 100, '', 'user', 0, 0, '', null, 0, '', 0, 1, '');
		$result = $extrafields->addExtraField('rm_default_time_date_nav_mode', $langs->trans('RequestManagerDefaultTimeDateNavMode'), 'select', 101, '', 'user', 0, 0, '', [ 'options' => [ 'all' => 'All', 'day' => 'Day', 'week' => 'Week', 'month' => 'Month', 'year' => 'Year' ]], 1, '', 3, 0, '', '', 'requestmanager@requestmanager');

		// Intervention
		if (isModEnabled('extendedintervention')) {
			$langs->load('extendedintervention@extendedintervention');
			$result = $extrafields->addExtraField('ei_type', $langs->trans('ExtendedInterventionType'), 'int', 0, '10', 'requestmanager', 0, 0, '', '', 0, '', 0, 1, '', '');
		}

		$this->_load_tables('/requestmanager/sql/');

		// Create functions and views SQL
		$sql_files = array(
			'function_rm_global_trim.sql',
		);
		dol_include_once('/requestmanager/lib/opendsi_common.lib.php');
		foreach ($sql_files as $file) {
			opendsi_execute_sql_file(dol_buildpath('/requestmanager/sql/' . $file));
		}

		// Create tables of all dictionaries
		dol_include_once('/advancedictionaries/class/dictionary.class.php');
		$dictionaries = Dictionary::fetchAllDictionaries($this->db, 'requestmanager');
		foreach ($dictionaries as $dictionary) {
			if ($dictionary->createTables() < 0) {
				setEventMessage('Error create dictionary table: ' . $dictionary->errorsToString(), 'errors');
			}
		}

		// Delete extrafields
		$extrafields->delete('rm_ipbx', 'actioncomm');

		// activate event type in modAgenda
		dolibarr_set_const($this->db, 'AGENDA_USE_EVENT_TYPE', 1, 'chaine', 0, '', $conf->entity);

		//ODT template
		$src = dol_buildpath('/requestmanager/doctemplates/requestmanager') . '/template_requestmanager.odt';
		$dirodt = DOL_DATA_ROOT . '/doctemplates/requestmanager';
		$dest = $dirodt . '/template_requestmanager.odt';

		if (file_exists($src) && !file_exists($dest)) {
			require_once DOL_DOCUMENT_ROOT . '/core/lib/files.lib.php';
			dol_mkdir($dirodt);
			$result = dol_copy($src, $dest, 0, 0);
			if ($result < 0) {
				$langs->load("errors");
				$this->error = $langs->trans('ErrorFailToCopyFile', $src, $dest);
				return 0;
			}
		}

		return $this->_init($sql, $options);
	}

	/**
	 * Function called when module is disabled.
	 * Remove from database constants, boxes and permissions from Dolibarr database.
	 * Data directories are not deleted
	 *
	 * @param      string	$options    Options when enabling module ('', 'noboxes')
	 * @return     int             	1 if OK, 0 if KO
	 */
	public function remove($options = '')
	{
		$sql = array();

		return $this->_remove($sql, $options);
	}
}

