<?php
/* Copyright (C) 2010-2012 	Laurent Destailleur <eldy@users.sourceforge.net>
 * Copyright (C) 2012		Juanjo Menent		<jmenent@2byte.es>
 * Copyright (C) 2016		Charlie Benke		<charlie@patas-monkey.com>
 * Copyright (C) 2018       Frédéric France         <frederic.france@netlogic.fr>
 *
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
* or see http://www.gnu.org/
*/

/**
 *	\file       htdocs/custom/requestmanager/core/modules/requestmanager/doc/doc_generic_requestmanager_odt.modules.php
 *	\ingroup    requestmanager
 *	\brief      File of class to build ODT documents for request
 */

dol_include_once('/requestmanager/core/modules/requestmanager/modules_requestmanager.php');
dol_include_once('/requestmanager/class/requestmanager.class.php');
require_once DOL_DOCUMENT_ROOT.'/product/class/product.class.php';
require_once DOL_DOCUMENT_ROOT.'/core/lib/company.lib.php';
require_once DOL_DOCUMENT_ROOT.'/core/lib/functions2.lib.php';
require_once DOL_DOCUMENT_ROOT.'/core/lib/files.lib.php';
require_once DOL_DOCUMENT_ROOT.'/core/lib/doc.lib.php';
require_once DOL_DOCUMENT_ROOT.'/core/lib/date.lib.php';


/**
 *	Class to build documents using ODF templates generator
 */
class doc_generic_requestmanager_odt extends ModelePDFRequestManager
{
    /**
     * @var Company Issuer object that emits
     */
    public $emetteur;

    /**
     * @var array Minimum version of PHP required by module.
     * e.g.: PHP ≥ 5.5 = array(5, 5)
     */
	public $phpmin = array(5, 5);

    /**
     * @var string Dolibarr version of the loaded document
     */
    public $version = 'dolibarr';


	/**
	 *	Constructor
	 *
	 *  @param		DoliDB		$db      Database handler
	 */
	public function __construct($db)
	{
		global $conf, $langs, $mysoc;

		// Load translation files required by the page
		$langs->loadLangs(array("main", "companies"));

		$this->db = $db;
		$this->name = "ODT templates";
		$this->description = $langs->trans("DocumentModelOdt");
		$this->scandir = 'REQUESTMANAGER_ADDON_PDF_ODT_PATH';    // Name of constant that is used to save list of directories to scan

		// Dimension page pour format A4
		$this->type = 'odt';
		$this->page_largeur = 0;
		$this->page_hauteur = 0;
		$this->format = array($this->page_largeur, $this->page_hauteur);
		$this->marge_gauche = 0;
		$this->marge_droite = 0;
		$this->marge_haute = 0;
		$this->marge_basse = 0;

		$this->option_logo = 1;                    // Affiche logo
		$this->option_tva = 0;                     // Gere option tva REQUESTMANAGER_TVAOPTION
		$this->option_modereg = 0;                 // Affiche mode reglement
		$this->option_condreg = 0;                 // Affiche conditions reglement
		$this->option_codeproduitservice = 0;      // Affiche code produit-service
		$this->option_multilang = 1;               // Dispo en plusieurs langues
		$this->option_escompte = 0;                // Affiche si il y a eu escompte
		$this->option_credit_note = 0;             // Support credit notes
		$this->option_freetext = 1;                   // Support add of a personalised text
		$this->option_draft_watermark = 0;           // Support add of a watermark on drafts

		// Recupere emetteur
		$this->emetteur = $mysoc;
		if (!$this->emetteur->country_code) $this->emetteur->country_code = substr($langs->defaultlang, -2);    // By default if not defined
	}


	/**
	 *	Return description of a module
	 *
	 *	@param	Translate	$langs      Lang object to use for output
	 *	@return string       			Description
	 */
	public function info($langs)
	{
		global $conf, $langs;

		// Load translation files required by the page
		$langs->loadLangs(array("errors", "companies"));

		$form = new Form($this->db);

		$texte = $this->description . ".<br>\n";
		$texte .= '<form action="' . $_SERVER["PHP_SELF"] . '" method="POST">';
		$texte .= '<input type="hidden" name="token" value="' . $_SESSION['newtoken'] . '">';
		$texte .= '<input type="hidden" name="action" value="setModuleOptions">';
		$texte .= '<input type="hidden" name="param1" value="REQUESTMANAGER_ADDON_PDF_ODT_PATH">';
		$texte .= '<table class="nobordernopadding" width="100%">';

		// List of directories area
		$texte .= '<tr><td>';
		$texttitle = $langs->trans("ListOfDirectories");
		$listofdir = explode(',', preg_replace('/[\r\n]+/', ',', trim($conf->global->REQUESTMANAGER_ADDON_PDF_ODT_PATH)));
		$listoffiles = array();
		foreach ($listofdir as $key => $tmpdir) {
			$tmpdir = trim($tmpdir);
			$tmpdir = preg_replace('/DOL_DATA_ROOT/', DOL_DATA_ROOT, $tmpdir);
			if (!$tmpdir) {
				unset($listofdir[$key]);
				continue;
			}
			if (!is_dir($tmpdir)) $texttitle .= img_warning($langs->trans("ErrorDirNotFound", $tmpdir), 0);
			else {
				$tmpfiles = dol_dir_list($tmpdir, 'files', 0, '\.(ods|odt)');
				if (count($tmpfiles)) $listoffiles = array_merge($listoffiles, $tmpfiles);
			}
		}
		$texthelp = $langs->trans("ListOfDirectoriesForModelGenODT");
		// Add list of substitution keys
		$texthelp .= '<br>' . $langs->trans("FollowingSubstitutionKeysCanBeUsed") . '<br>';
		$texthelp .= $langs->transnoentitiesnoconv("FullListOnOnlineDocumentation");    // This contains an url, we don't modify it

		$texte .= $form->textwithpicto($texttitle, $texthelp, 1, 'help', '', 1);
		$texte .= '<div><div style="display: inline-block; min-width: 100px; vertical-align: middle;">';
		$texte .= '<textarea class="flat" cols="60" name="value1">';
		$texte .= $conf->global->REQUESTMANAGER_ADDON_PDF_ODT_PATH;
		$texte .= '</textarea>';
		$texte .= '</div><div style="display: inline-block; vertical-align: middle;">';
		$texte .= '<input type="submit" class="button" value="' . $langs->trans("Modify") . '" name="Button">';
		$texte .= '<br></div></div>';

		// Scan directories
		$nbofiles = count($listoffiles);
		if (!empty($conf->global->REQUESTMANAGER_ADDON_PDF_ODT_PATH)) {
			$texte .= $langs->trans("NumberOfModelFilesFound") . ': <b>';
			//$texte.=$nbofiles?'<a id="a_'.get_class($this).'" href="#">':'';
			$texte .= count($listoffiles);
			//$texte.=$nbofiles?'</a>':'';
			$texte .= '</b>';
		}

		if ($nbofiles) {
			$texte .= '<div id="div_' . get_class($this) . '" class="hidden">';
			foreach ($listoffiles as $file) {
				$texte .= $file['name'] . '<br>';
			}
			$texte .= '<div id="div_' . get_class($this) . '">';
		}

		$texte .= '</td>';

		$texte .= '<td valign="top" rowspan="2" class="hideonsmartphone">';
		$texte .= $langs->trans("ExampleOfDirectoriesForModelGen");
		$texte .= '</td>';
		$texte .= '</tr>';

		$texte .= '</table>';
		$texte .= '</form>';

		return $texte;
	}

    // phpcs:disable PEAR.NamingConventions.ValidFunctionName.ScopeNotCamelCaps
	/**
	 *	Function to build a document on disk using the generic odt module.
	 *
	 *	@param		RequestManager	$object				Object source to build document
	 *	@param		Translate		$outputlangs		Lang output object
	 * 	@param		string			$srctemplatepath	Full path of source filename for generator using a template file
	 *  @param		int				$hidedetails		Do not show line details
	 *  @param		int				$hidedesc			Do not show desc
	 *  @param		int				$hideref			Do not show ref
	 *	@return		int         						1 if OK, <=0 if KO
	 */
	public function write_file($object, $outputlangs, $srctemplatepath, $hidedetails = 0, $hidedesc = 0, $hideref = 0)
	{
		// phpcs:enable
		global $user, $langs, $conf, $mysoc, $hookmanager;

		if (empty($srctemplatepath)) {
			dol_syslog("doc_generic_odt::write_file parameter srctemplatepath empty", LOG_WARNING);
			return -1;
		}

		// Add odtgeneration hook
		if (!is_object($hookmanager)) {
			include_once DOL_DOCUMENT_ROOT . '/core/class/hookmanager.class.php';
			$hookmanager = new HookManager($this->db);
		}
		$hookmanager2 = clone $hookmanager; // Génère des erreurs de resultat disparaissant si appelé dans une autre hooks donc on copie la hook
		$hookmanager2->initHooks(array('odtgeneration'));
		global $action;

		if (!is_object($outputlangs)) $outputlangs = $langs;
		$sav_charset_output = $outputlangs->charset_output;
		$outputlangs->charset_output = 'UTF-8';

		// Load translation files required by the page
		$outputlangs->loadLangs(array("main", "requestmanager@requestmanager", "dict", "companies", "products", "bills"));

		if ($conf->requestmanager->multidir_output[$conf->entity]) {
			// If $object is id instead of object
			if (!is_object($object)) {
				$id = $object;
				$object = new RequestManager($this->db);
				$result = $object->fetch($id);
				if ($result < 0) {
					dol_print_error($this->db, $object->error);
					return -1;
				}
			}

			$dir = $conf->requestmanager->multidir_output[$object->entity];
			$objectref = dol_sanitizeFileName($object->ref);
			if (!preg_match('/specimen/i', $objectref)) $dir .= "/" . $objectref;
			$file = $dir . "/" . $objectref . ".odt";

			if (!file_exists($dir)) {
				if (dol_mkdir($dir) < 0) {
					$this->error = $langs->transnoentities("ErrorCanNotCreateDir", $dir);
					return -1;
				}
			}

			if (file_exists($dir)) {
				//print "srctemplatepath=".$srctemplatepath;	// Src filename
				$newfile = basename($srctemplatepath);
				$newfiletmp = preg_replace('/\.od(t|s)/i', '', $newfile);
				$newfiletmp = preg_replace('/template_/i', '', $newfiletmp);
				$newfiletmp = preg_replace('/modele_/i', '', $newfiletmp);

				$newfiletmp = $objectref . '_' . $newfiletmp;

				// Get extension (ods or odt)
				$newfileformat = substr($newfile, strrpos($newfile, '.') + 1);
				if (!empty($conf->global->MAIN_DOC_USE_TIMING)) {
					$format = $conf->global->MAIN_DOC_USE_TIMING;
					if ($format == '1') $format = '%Y%m%d%H%M%S';
					$filename = $newfiletmp . '-' . dol_print_date(dol_now(), $format) . '.' . $newfileformat;
				} else {
					$filename = $newfiletmp . '.' . $newfileformat;
				}
				$file = $dir . '/' . $filename;
				//print "newdir=".$dir;
				//print "newfile=".$newfile;
				//print "file=".$file;
				//print "conf->requestmanager->dir_temp=".$conf->requestmanager->dir_temp;

				dol_mkdir($conf->requestmanager->multidir_temp[$object->entity]);

				// Make substitution
				$substitutionarray = array(
					'__FROM_NAME__' => $this->emetteur->name,
					'__FROM_EMAIL__' => $this->emetteur->email,
				);
				complete_substitutions_array($substitutionarray, $langs, $object);
				// Call the ODTSubstitution hook
				$parameters = array('file' => $file, 'object' => $object, 'outputlangs' => $outputlangs, 'substitutionarray' => &$substitutionarray);
				$reshook = $hookmanager2->executeHooks('ODTSubstitution', $parameters, $this, $action);    // Note that $action and $object may have been modified by some hooks

				// Line of free text
				$newfreetext = '';
				$paramfreetext = 'REQUESTMANAGER_FREE_TEXT';
				if (!empty($conf->global->$paramfreetext)) {
					$newfreetext = make_substitutions($conf->global->$paramfreetext, $substitutionarray);
				}

				// Open and load template
				require_once ODTPHP_PATH . 'odf.php';
				try {
					$odfHandler = new odf(
						$srctemplatepath,
						array(
							'PATH_TO_TMP' => $conf->requestmanager->multidir_temp[$object->entity],
							'ZIP_PROXY' => 'PclZipProxy',    // PhpZipProxy or PclZipProxy. Got "bad compression method" error when using PhpZipProxy.
							'DELIMITER_LEFT' => '{',
							'DELIMITER_RIGHT' => '}'
						)
					);
				} catch (Exception $e) {
					$this->error = $e->getMessage();
					dol_syslog($e->getMessage(), LOG_INFO);
					return -1;
				}
				// After construction $odfHandler->contentXml contains content and
				// [!-- BEGIN row.lines --]*[!-- END row.lines --] has been replaced by
				// [!-- BEGIN lines --]*[!-- END lines --]
				//print html_entity_decode($odfHandler->__toString());
				//print exit;

				// Reload all infos
				// Todo manage parent and children request ?
//				$object->fetch_parent();
//				$object->fetch_children_request(null, 1);
				$object->fetch_thirdparty();
				$object->fetch_optionals();
				$object->fetch_requesters(1);
				$object->fetch_watchers(1);
				$object->fetch_assigned(1);
				$can_set_times = false;
				$children_total_time_spent = array();
				if ($object->isRequestTypeOption('manage_times')) {
					$can_set_times = $object->isRequestTypeOption('display_set_times');
					$children_total_time_spent = $object->get_children_total_time_spent();
				}
				if (!is_object($object->user_resolved)) {
					$object->user_resolved = new User($this->db);
					if ($object->user_resolved_id > 0) $object->user_resolved->fetch($object->user_resolved_id);
				}
				if (!is_object($object->user_cloture)) {
					$object->user_cloture = new User($this->db);
					if ($object->user_cloture_id > 0) $object->user_cloture->fetch($object->user_cloture_id);
				}

				// Make substitutions into odt of freetext
				try {
					$odfHandler->setVars('free_text', $newfreetext, true, 'UTF-8');
				} catch (OdfException $e) {
					dol_syslog($e->getMessage(), LOG_INFO);
				}

				// Define substitution array
				$substitutionarray = getCommonSubstitutionArray($outputlangs, 0, null, $object);
				$array_object_from_properties = $this->get_substitutionarray_each_var_object($object, $outputlangs);
				$array_object = $this->get_substitutionarray_request_object($object, $outputlangs, 'object', $can_set_times, $children_total_time_spent);
				$array_user = $this->get_substitutionarray_user($user, $outputlangs);
				$array_soc = $this->get_substitutionarray_mysoc($mysoc, $outputlangs);
				$array_thirdparty = $this->get_substitutionarray_thirdparty($object->thirdparty, $outputlangs);
				$array_user_resolved = $this->get_substitutionarray_user_object($object->user_resolved, $outputlangs, 'user_resolved');
				$array_user_cloture = $this->get_substitutionarray_user_object($object->user_cloture, $outputlangs, 'user_cloture');
				$array_other = $this->get_substitutionarray_other($outputlangs);

				$tmparray = array_merge($substitutionarray, $array_object_from_properties, $array_user, $array_soc, $array_thirdparty, $array_object, $array_user_resolved, $array_user_cloture, $array_other);
				complete_substitutions_array($tmparray, $outputlangs, $object);

				// Call the ODTSubstitution hook
				$parameters = array('odfHandler' => &$odfHandler, 'file' => $file, 'object' => $object, 'outputlangs' => $outputlangs, 'substitutionarray' => &$tmparray);
				$reshook = $hookmanager2->executeHooks('ODTSubstitution', $parameters, $this, $action);    // Note that $action and $object may have been modified by some hooks

				foreach ($tmparray as $key => $value) {
					try {
						if (preg_match('/logo$/', $key)) { // Image
							if (file_exists($value)) $odfHandler->setImage($key, $value);
							else $odfHandler->setVars($key, 'ErrorFileNotFound', true, 'UTF-8');
						} else {   // Text
							$odfHandler->setVars($key, $value, true, 'UTF-8');
						}
					} catch (OdfException $e) {
						dol_syslog($e->getMessage(), LOG_INFO);
					}
				}
				// Replace tags of requester
				try {
					$foundtagforrequesters = 1;
					try {
						$listlines = $odfHandler->setSegment('requesters');
					} catch (OdfException $e) {
						// We may arrive here if tags for lines not present into template
						$foundtagforrequesters = 0;
						dol_syslog($e->getMessage(), LOG_INFO);
					}
					if ($foundtagforrequesters) {
						foreach ($object->requester_list as $contact) {
							$tmparray = $this->get_substitutionarray_contact($contact, $outputlangs, 'requester');
							complete_substitutions_array($tmparray, $outputlangs, $object, $contact, "completesubstitutionarray_requesters");
							// Call the ODTSubstitutionLine hook
							$parameters = array('odfHandler' => &$odfHandler, 'file' => $file, 'object' => $object, 'outputlangs' => $outputlangs, 'substitutionarray' => &$tmparray, 'contact' => $contact);
							$reshook = $hookmanager2->executeHooks('ODTSubstitutionRequester', $parameters, $this, $action);    // Note that $action and $object may have been modified by some hooks
							foreach ($tmparray as $key => $val) {
								try {
									$listlines->setVars($key, $val, true, 'UTF-8');
								} catch (OdfException $e) {
									dol_syslog($e->getMessage(), LOG_INFO);
								} catch (SegmentException $e) {
									dol_syslog($e->getMessage(), LOG_INFO);
								}
							}
							$listlines->merge();
						}
						$odfHandler->mergeSegment($listlines);
					}
				} catch (OdfException $e) {
					$this->error = $e->getMessage();
					dol_syslog($this->error, LOG_WARNING);
					return -1;
				}
				// Replace tags of watchers
				try {
					$foundtagforwatchers = 1;
					try {
						$listlines = $odfHandler->setSegment('watchers');
					} catch (OdfException $e) {
						// We may arrive here if tags for lines not present into template
						$foundtagforwatchers = 0;
						dol_syslog($e->getMessage(), LOG_INFO);
					}
					if ($foundtagforwatchers) {
						foreach ($object->watcher_list as $contact) {
							$tmparray = $this->get_substitutionarray_contact($contact, $outputlangs, 'watcher');
							complete_substitutions_array($tmparray, $outputlangs, $object, $contact, "completesubstitutionarray_watchers");
							// Call the ODTSubstitutionLine hook
							$parameters = array('odfHandler' => &$odfHandler, 'file' => $file, 'object' => $object, 'outputlangs' => $outputlangs, 'substitutionarray' => &$tmparray, 'contact' => $contact);
							$reshook = $hookmanager2->executeHooks('ODTSubstitutionWatcher', $parameters, $this, $action);    // Note that $action and $object may have been modified by some hooks
							foreach ($tmparray as $key => $val) {
								try {
									$listlines->setVars($key, $val, true, 'UTF-8');
								} catch (OdfException $e) {
									dol_syslog($e->getMessage(), LOG_INFO);
								} catch (SegmentException $e) {
									dol_syslog($e->getMessage(), LOG_INFO);
								}
							}
							$listlines->merge();
						}
						$odfHandler->mergeSegment($listlines);
					}
				} catch (OdfException $e) {
					$this->error = $e->getMessage();
					dol_syslog($this->error, LOG_WARNING);
					return -1;
				}
				// Replace tags of assigned users
				try {
					$foundtagforassignedusers = 1;
					try {
						$listlines = $odfHandler->setSegment('assigned_users');
					} catch (OdfException $e) {
						// We may arrive here if tags for lines not present into template
						$foundtagforassignedusers = 0;
						dol_syslog($e->getMessage(), LOG_INFO);
					}
					if ($foundtagforassignedusers) {
						foreach ($object->assigned_user_list as $user) {
							$tmparray = $this->get_substitutionarray_user_object($user, $outputlangs, 'assigned_user');
							complete_substitutions_array($tmparray, $outputlangs, $object, $user, "completesubstitutionarray_assigned_users");
							// Call the ODTSubstitutionLine hook
							$parameters = array('odfHandler' => &$odfHandler, 'file' => $file, 'object' => $object, 'outputlangs' => $outputlangs, 'substitutionarray' => &$tmparray, 'user' => $user);
							$reshook = $hookmanager2->executeHooks('ODTSubstitutionAssignedUser', $parameters, $this, $action);    // Note that $action and $object may have been modified by some hooks
							foreach ($tmparray as $key => $val) {
								try {
									$listlines->setVars($key, $val, true, 'UTF-8');
								} catch (OdfException $e) {
									dol_syslog($e->getMessage(), LOG_INFO);
								} catch (SegmentException $e) {
									dol_syslog($e->getMessage(), LOG_INFO);
								}
							}
							$listlines->merge();
						}
						$odfHandler->mergeSegment($listlines);
					}
				} catch (OdfException $e) {
					$this->error = $e->getMessage();
					dol_syslog($this->error, LOG_WARNING);
					return -1;
				}
				// Replace tags of assigned user groups
				try {
					$foundtagforassignedusergroups = 1;
					try {
						$listlines = $odfHandler->setSegment('assigned_usergroups');
					} catch (OdfException $e) {
						// We may arrive here if tags for lines not present into template
						$foundtagforassignedusergroups = 0;
						dol_syslog($e->getMessage(), LOG_INFO);
					}
					if ($foundtagforassignedusergroups) {
						foreach ($object->assigned_usergroup_list as $usergroup) {
							$tmparray = $this->get_substitutionarray_usergroup_object($usergroup, $outputlangs, 'assigned_usergroup');
							complete_substitutions_array($tmparray, $outputlangs, $object, $usergroup, "completesubstitutionarray_assigned_usergroups");
							// Call the ODTSubstitutionLine hook
							$parameters = array('odfHandler' => &$odfHandler, 'file' => $file, 'object' => $object, 'outputlangs' => $outputlangs, 'substitutionarray' => &$tmparray, 'usergroup' => $usergroup);
							$reshook = $hookmanager2->executeHooks('ODTSubstitutionAssignedUserGroup', $parameters, $this, $action);    // Note that $action and $object may have been modified by some hooks
							foreach ($tmparray as $key => $val) {
								try {
									$listlines->setVars($key, $val, true, 'UTF-8');
								} catch (OdfException $e) {
									dol_syslog($e->getMessage(), LOG_INFO);
								} catch (SegmentException $e) {
									dol_syslog($e->getMessage(), LOG_INFO);
								}
							}
							$listlines->merge();
						}
						$odfHandler->mergeSegment($listlines);
					}
				} catch (OdfException $e) {
					$this->error = $e->getMessage();
					dol_syslog($this->error, LOG_WARNING);
					return -1;
				}
				// Replace tags of tags/categories
				try {
					$foundtagforcategories = 1;
					try {
						$listlines = $odfHandler->setSegment('tags');
					} catch (OdfException $e) {
						// We may arrive here if tags for lines not present into template
						$foundtagforcategories = 0;
						dol_syslog($e->getMessage(), LOG_INFO);
					}
					if ($foundtagforcategories) {
						$object->fetch_tags(1);
						foreach ($object->tag_list as $tag) {
							$tmparray = $this->get_substitutionarray_tag_object($tag, $outputlangs, 'tag');
							complete_substitutions_array($tmparray, $outputlangs, $object, $tag, "completesubstitutionarray_tags");
							// Call the ODTSubstitutionLine hook
							$parameters = array('odfHandler' => &$odfHandler, 'file' => $file, 'object' => $object, 'outputlangs' => $outputlangs, 'substitutionarray' => &$tmparray, 'tag' => $tag);
							$reshook = $hookmanager2->executeHooks('ODTSubstitutionTag', $parameters, $this, $action);    // Note that $action and $object may have been modified by some hooks
							foreach ($tmparray as $key => $val) {
								try {
									$listlines->setVars($key, $val, true, 'UTF-8');
								} catch (OdfException $e) {
									dol_syslog($e->getMessage(), LOG_INFO);
								} catch (SegmentException $e) {
									dol_syslog($e->getMessage(), LOG_INFO);
								}
							}
							$listlines->merge();
						}
						$odfHandler->mergeSegment($listlines);
					}
				} catch (OdfException $e) {
					$this->error = $e->getMessage();
					dol_syslog($this->error, LOG_WARNING);
					return -1;
				}
				// Replace tags of lines
				try {
					$foundtagforlines = 1;
					try {
						$listlines = $odfHandler->setSegment('lines');
					} catch (OdfException $e) {
						// We may arrive here if tags for lines not present into template
						$foundtagforlines = 0;
						dol_syslog($e->getMessage(), LOG_INFO);
					}
					if ($foundtagforlines) {
						$object->fetch_lines();
						foreach ($object->lines as $line) {
							$tmparray = $this->get_substitutionarray_lines($line, $outputlangs);
							complete_substitutions_array($tmparray, $outputlangs, $object, $line, "completesubstitutionarray_lines");
							// Call the ODTSubstitutionLine hook
							$parameters = array('odfHandler' => &$odfHandler, 'file' => $file, 'object' => $object, 'outputlangs' => $outputlangs, 'substitutionarray' => &$tmparray, 'line' => $line);
							$reshook = $hookmanager2->executeHooks('ODTSubstitutionLine', $parameters, $this, $action);    // Note that $action and $object may have been modified by some hooks
							foreach ($tmparray as $key => $val) {
								try {
									$listlines->setVars($key, $val, true, 'UTF-8');
								} catch (OdfException $e) {
									dol_syslog($e->getMessage(), LOG_INFO);
								} catch (SegmentException $e) {
									dol_syslog($e->getMessage(), LOG_INFO);
								}
							}
							$listlines->merge();
						}
						$odfHandler->mergeSegment($listlines);
					}
				} catch (OdfException $e) {
					$this->error = $e->getMessage();
					dol_syslog($this->error, LOG_WARNING);
					return -1;
				}

				// Replace labels translated
				$tmparray = $outputlangs->get_translations_for_substitutions();
				foreach ($tmparray as $key => $value) {
					try {
						$odfHandler->setVars($key, $value, true, 'UTF-8');
					} catch (OdfException $e) {
						dol_syslog($e->getMessage(), LOG_INFO);
					}
				}

				// Call the beforeODTSave hook
				$parameters = array('odfHandler' => &$odfHandler, 'file' => $file, 'object' => $object, 'outputlangs' => $outputlangs, 'substitutionarray' => &$tmparray);
				$reshook = $hookmanager2->executeHooks('beforeODTSave', $parameters, $this, $action);    // Note that $action and $object may have been modified by some hooks

				// Write new file
				if (!empty($conf->global->MAIN_ODT_AS_PDF)) {
					try {
						$odfHandler->exportAsAttachedPDF($file);
					} catch (Exception $e) {
						$this->error = $e->getMessage();
						dol_syslog($e->getMessage(), LOG_INFO);
						return -1;
					}
				} else {
					try {
						$odfHandler->saveToDisk($file);
					} catch (Exception $e) {
						$this->error = $e->getMessage();
						dol_syslog($e->getMessage(), LOG_INFO);
						return -1;
					}
				}
				$parameters = array('odfHandler' => &$odfHandler, 'file' => $file, 'object' => $object, 'outputlangs' => $outputlangs, 'substitutionarray' => &$tmparray);
				$reshook = $hookmanager2->executeHooks('afterODTCreation', $parameters, $this, $action);    // Note that $action and $object may have been modified by some hooks

				if (!empty($conf->global->MAIN_UMASK))
					@chmod($file, octdec($conf->global->MAIN_UMASK));

				$odfHandler = null;    // Destroy object

				$this->result = array('fullpath' => $file);

				return 1;   // Success
			} else {
				$this->error = $langs->transnoentities("ErrorCanNotCreateDir", $dir);
				return -1;
			}
		}

		return -1;
	}

	// phpcs:disable PEAR.NamingConventions.ValidFunctionName.ScopeNotCamelCaps
	/**
	 * Define array with couple subtitution key => subtitution value
	 *
	 * @param	Object		$object			Object
	 * @param   Translate	$outputlangs    Language object for output
	 * @param   string		$array_key	    Name of the key for return array
	 * @return	array						Array of substitution key->code
	 */
	public function get_substitutionarray_thirdparty_object($object, $outputlangs, $array_key = 'object')
	{
		// phpcs:enable
		global $conf;

		if (empty($object->country) && !empty($object->country_code)) {
			$object->country = $outputlangs->transnoentitiesnoconv("Country" . $object->country_code);
		}
		if (empty($object->state) && !empty($object->state_code)) {
			$object->state = getState($object->state_code, 0);
		}

		$array_thirdparty = array(
			$array_key . '_name' => $object->name,
			$array_key . '_name_alias' => $object->name_alias,
			$array_key . '_email' => $object->email,
			$array_key . '_phone' => $object->phone,
			$array_key . '_fax' => $object->fax,
			$array_key . '_address' => $object->address,
			$array_key . '_zip' => $object->zip,
			$array_key . '_town' => $object->town,
			$array_key . '_country' => $object->country,
			$array_key . '_country_code' => $object->country_code,
			$array_key . '_state' => $object->state,
			$array_key . '_state_code' => $object->state_code,
			$array_key . '_web' => $object->url,
			$array_key . '_barcode' => $object->barcode,
			$array_key . '_vatnumber' => $object->tva_intra,
			$array_key . '_customercode' => $object->code_client,
			$array_key . '_suppliercode' => $object->code_fournisseur,
			$array_key . '_customeraccountancycode' => $object->code_compta,
			$array_key . '_supplieraccountancycode' => $object->code_compta_fournisseur,
			$array_key . '_juridicalstatus' => $object->forme_juridique,
			$array_key . '_outstanding_limit' => $object->outstanding_limit,
			$array_key . '_capital' => $object->capital,
			$array_key . '_idprof1' => $object->idprof1,
			$array_key . '_idprof2' => $object->idprof2,
			$array_key . '_idprof3' => $object->idprof3,
			$array_key . '_idprof4' => $object->idprof4,
			$array_key . '_idprof5' => $object->idprof5,
			$array_key . '_idprof6' => $object->idprof6,
			$array_key . '_note_public' => $object->note_public,
			$array_key . '_note_private' => $object->note_private,
			$array_key . '_default_bank_iban' => $object->bank_account->iban,
			$array_key . '_default_bank_bic' => $object->bank_account->bic
		);

		// Retrieve extrafields
		if (is_array($object->array_options) && count($object->array_options)) {
			require_once DOL_DOCUMENT_ROOT . '/core/class/extrafields.class.php';

			$v13p = version_compare(DOL_VERSION, '13.0.0', '>=');

			$extrafields = new ExtraFields($this->db);
			$extralabels = $extrafields->fetch_name_optionals_label('societe', true);
			$object->fetch_optionals();

			if ($v13p) {
				if (!empty($extrafields->attributes['societe']['label'])) {
					foreach ($extrafields->attributes['societe']['label'] as $key => $label) {
						if ($extrafields->attributes['societe']['type'][$key] == 'price') {
							$object->array_options['options_'.$key] = price($object->array_options['options_'.$key], 0, $outputlangs, 0, 0, -1, $conf->currency);
						} elseif ($extrafields->attributes['societe']['type'][$key] == 'select' || $extrafields->attributes['societe']['type'][$key] == 'checkbox') {
							$object->array_options['options_'.$key] = $extrafields->attributes['societe']['param'][$key]['options'][$object->array_options['options_'.$key]];
						}
						$array_thirdparty = array_merge($array_thirdparty, array($array_key.'_options_'.$key => $object->array_options['options_'.$key]));
					}
				}
			} else {
				foreach ($extrafields->attribute_label as $key => $label) {
					if ($extrafields->attribute_type[$key] == 'price') {
						$object->array_options['options_'.$key] = price($object->array_options['options_'.$key], 0, $outputlangs, 0, 0, -1, $conf->currency);
					} elseif ($extrafields->attribute_type[$key] == 'select' || $extrafields->attribute_type[$key] == 'checkbox') {
						$object->array_options['options_'.$key] = $extrafields->attribute_param[$key]['options'][$object->array_options['options_'.$key]];
					}
					$array_thirdparty = array_merge($array_thirdparty, array($array_key.'_options_'.$key => $object->array_options['options_'.$key]));
				}
			}
		}
		return $array_thirdparty;
	}

	// phpcs:disable PEAR.NamingConventions.ValidFunctionName.ScopeNotCamelCaps
	/**
	 * Define array with couple subtitution key => subtitution value
	 *
	 * @param	User 		$object        	user
	 * @param	Translate 	$outputlangs   	object for output
	 * @param   string		$array_key	    Name of the key for return array
	 * @return	array 						Array of substitution key->code
	 */
	public function get_substitutionarray_user_object($object, $outputlangs, $array_key = 'object')
	{
		// phpcs:enable
		global $conf;

		if (empty($object->country) && !empty($object->country_code)) {
			$object->country = $outputlangs->transnoentitiesnoconv("Country" . $object->country_code);
		}
		if (empty($object->state) && !empty($object->state_code)) {
			$object->state = getState($object->state_code, 0);
		}

		$logotouse = $conf->user->dir_output . '/' . get_exdir($object->id, 2, 0, 1, $object, 'user') . '/' . $object->photo;

		$array_user = array(
			$array_key . '_fullname' => $object->getFullName($outputlangs, 1),
			$array_key . '_lastname' => $object->lastname,
			$array_key . '_firstname' => $object->firstname,
			$array_key . '_login' => $object->login,
			$array_key . '_address' => $object->address,
			$array_key . '_zip' => $object->zip,
			$array_key . '_town' => $object->town,
			$array_key . '_state_id' => $object->state_id,
			$array_key . '_state_code' => $object->state_code,
			$array_key . '_state' => $object->state,
			$array_key . '_country_id' => $object->country_id,
			$array_key . '_country_code' => $object->country_code,
			$array_key . '_country' => $object->country,
			$array_key . '_phone' => $object->office_phone,
			$array_key . '_fax' => $object->office_fax,
			$array_key . '_mobile' => $object->user_mobile,
			$array_key . '_email' => $object->email,
			$array_key . '_logo' => $logotouse,
			$array_key . '_job' => $object->job,
			$array_key . '_web' => '',
		);

		// Retrieve extrafields
		require_once DOL_DOCUMENT_ROOT . '/core/class/extrafields.class.php';

		$v13p = version_compare(DOL_VERSION, '13.0.0', '>=');

		$extrafields = new ExtraFields($this->db);
		$extralabels = $extrafields->fetch_name_optionals_label('user', true);
		$object->fetch_optionals();

		if ($v13p) {
			if (!empty($extrafields->attributes['user']['label'])) {
				foreach ($extrafields->attributes['user']['label'] as $key => $label) {
					if ($extrafields->attributes['user']['type'][$key] == 'price') {
						$object->array_options['options_'.$key] = price($object->array_options ['options_'.$key], 0, $outputlangs, 0, 0, -1, $conf->currency);
					} elseif ($extrafields->attributes['user']['type'][$key] == 'select' || $extrafields->attributes['user']['type'][$key] == 'checkbox') {
						$object->array_options['options_'.$key] = $extrafields->attributes['user']['param'][$key]['options'][$object->array_options['options_'.$key]];
					}
					$array_user = array_merge($array_user, array($array_key.'_options_'.$key => $object->array_options['options_'.$key] ?? ''));
				}
			}
		} else {
			foreach ($extrafields->attribute_label as $key => $label) {
				if ($extrafields->attribute_type[$key] == 'price') {
					$object->array_options['options_'.$key] = price($object->array_options ['options_'.$key], 0, $outputlangs, 0, 0, -1, $conf->currency);
				} elseif ($extrafields->attribute_type[$key] == 'select' || $extrafields->attribute_type[$key] == 'checkbox') {
					$object->array_options['options_'.$key] = $extrafields->attribute_param[$key]['options'][$object->array_options['options_'.$key]];
				}
				$array_user = array_merge($array_user, array($array_key.'_options_'.$key => $object->array_options['options_'.$key]));
			}
		}

		return $array_user;
	}

	// phpcs:disable PEAR.NamingConventions.ValidFunctionName.ScopeNotCamelCaps
	/**
	 * Define array with couple subtitution key => subtitution value
	 *
	 * @param	UserGroup 	$object        	usergroup
	 * @param	Translate 	$outputlangs   	object for output
	 * @param   string		$array_key	    Name of the key for return array
	 * @return	array 						Array of substitution key->code
	 */
	public function get_substitutionarray_usergroup_object($object, $outputlangs, $array_key = 'object')
	{
		// phpcs:enable
		global $conf;

		$array_usergroup = array(
			$array_key . '_ref' => $object->ref,
			$array_key . '_name' => $object->name,
			$array_key . '_note' => $object->note,
		);

		// Retrieve extrafields
		require_once DOL_DOCUMENT_ROOT . '/core/class/extrafields.class.php';

		$v13p = version_compare(DOL_VERSION, '13.0.0', '>=');

		$extrafields = new ExtraFields($this->db);
		$extralabels = $extrafields->fetch_name_optionals_label('usergroup', true);
		$object->fetch_optionals();

		if ($v13p) {
			if (!empty($extrafields->attributes['usergroup']['label'])) {
				foreach ($extrafields->attributes['usergroup']['label'] as $key => $label) {
					if ($extrafields->attributes['usergroup']['type'][$key] == 'price') {
						$object->array_options['options_'.$key] = price($object->array_options ['options_'.$key], 0, $outputlangs, 0, 0, -1, $conf->currency);
					} elseif ($extrafields->attributes['usergroup']['type'][$key] == 'select' || $extrafields->attributes['usergroup']['type'][$key] == 'checkbox') {
						$object->array_options['options_'.$key] = $extrafields->attributes['usergroup']['param'][$key]['options'][$object->array_options['options_'.$key]];
					}
					$array_usergroup = array_merge($array_usergroup, array($array_key.'_options_'.$key => $object->array_options['options_'.$key]));
				}
			}
		} else {
			foreach ($extrafields->attribute_label as $key => $label) {
				if ($extrafields->attribute_type[$key] == 'price') {
					$object->array_options['options_'.$key] = price($object->array_options ['options_'.$key], 0, $outputlangs, 0, 0, -1, $conf->currency);
				} elseif ($extrafields->attribute_type[$key] == 'select' || $extrafields->attribute_type[$key] == 'checkbox') {
					$object->array_options['options_'.$key] = $extrafields->attribute_param[$key]['options'][$object->array_options['options_'.$key]];
				}
				$array_usergroup = array_merge($array_usergroup, array($array_key.'_options_'.$key => $object->array_options['options_'.$key]));
			}
		}

		return $array_usergroup;
	}

	// phpcs:disable PEAR.NamingConventions.ValidFunctionName.ScopeNotCamelCaps
	/**
	 * Define array with couple subtitution key => subtitution value
	 *
	 * @param	Categorie	$object        	category
	 * @param	Translate 	$outputlangs   	object for output
	 * @param   string		$array_key	    Name of the key for return array
	 * @return	array 						Array of substitution key->code
	 */
	public function get_substitutionarray_tag_object($object, $outputlangs, $array_key = 'object')
	{
		// phpcs:enable
		global $conf;

		$array_tag = array(
			$array_key . '_label' => $object->label,
			$array_key . '_description' => $object->description,
			$array_key . '_color' => $object->color,
		);

		// Retrieve extrafields
		require_once DOL_DOCUMENT_ROOT . '/core/class/extrafields.class.php';

		$v13p = version_compare(DOL_VERSION, '13.0.0', '>=');

		$extrafields = new ExtraFields($this->db);
		$extralabels = $extrafields->fetch_name_optionals_label('categorie', true);
		$object->fetch_optionals();

		if ($v13p) {
			if (!empty($extrafields->attributes['categorie']['label'])) {
				foreach ($extrafields->attributes['categorie']['label'] as $key => $label) {
					if ($extrafields->attributes['categorie']['type'][$key] == 'price') {
						$object->array_options['options_'.$key] = price($object->array_options ['options_'.$key], 0, $outputlangs, 0, 0, -1, $conf->currency);
					} elseif ($extrafields->attributes['categorie']['type'][$key] == 'select' || $extrafields->attributes['categorie']['type'][$key] == 'checkbox') {
						$object->array_options['options_'.$key] = $extrafields->attributes['categorie']['param'][$key]['options'][$object->array_options['options_'.$key]];
					}
					$array_tag = array_merge($array_tag, array($array_key.'_options_'.$key => $object->array_options['options_'.$key]));
				}
			}
		} else {
			foreach ($extrafields->attribute_label as $key => $label) {
				if ($extrafields->attribute_type[$key] == 'price') {
					$object->array_options['options_'.$key] = price($object->array_options ['options_'.$key], 0, $outputlangs, 0, 0, -1, $conf->currency);
				} elseif ($extrafields->attribute_type[$key] == 'select' || $extrafields->attribute_type[$key] == 'checkbox') {
					$object->array_options['options_'.$key] = $extrafields->attribute_param[$key]['options'][$object->array_options['options_'.$key]];
				}
				$array_tag = array_merge($array_tag, array($array_key.'_options_'.$key => $object->array_options['options_'.$key]));
			}
		}

		return $array_tag;
	}

	// phpcs:disable PEAR.NamingConventions.ValidFunctionName.ScopeNotCamelCaps
	/**
	 * Define array with couple subtitution key => subtitution value
	 *
	 * @param	RequestManager 		$object        				request
	 * @param	Translate 			$outputlangs   				object for output
	 * @param   string				$array_key	    			Name of the key for return array
	 * @param 	bool				$can_set_times				Can set times
	 * @param   array				$children_total_time_spent	Children total time spent
	 * @return	array 								Array of substitution key->code
	 */
	public function get_substitutionarray_request_object($object, $outputlangs, $array_key = 'object', $can_set_times = false, $children_total_time_spent = array())
	{
		// phpcs:enable
		global $conf;
		$resarray = $this->get_substitutionarray_object($object, $outputlangs, $array_key);

		$resarray[$array_key . '_label'] = $object->label;
		$resarray[$array_key . '_description'] = $object->description;
		$resarray[$array_key . '_type'] = $object->getLibType();
		$resarray[$array_key . '_category'] = $object->getLibCategory();
		$resarray[$array_key . '_source'] = $object->getLibSource();
		$resarray[$array_key . '_urgency'] = $object->getLibUrgency();
		$resarray[$array_key . '_impact'] = $object->getLibImpact();
		$resarray[$array_key . '_priority'] = $object->getLibPriority();
		$resarray[$array_key . '_notify_requester_by_email'] = yn($object->notify_requester_by_email);
		$resarray[$array_key . '_notify_watcher_by_email'] = yn($object->notify_watcher_by_email);
		$resarray[$array_key . '_notify_assigned_by_email'] = yn($object->notify_assigned_by_email);
		$resarray[$array_key . '_reason_resolution'] = $object->getLibReasonResolution();
		$resarray[$array_key . '_reason_resolution_details'] = $object->reason_resolution_details;
		$estimated_duration = $object->estimated_duration;
		if (isset($children_total_time_spent['total_estimated_duration'])) {
			$estimated_duration += $children_total_time_spent['total_estimated_duration'];
		}
		$resarray[$array_key . '_estimated_duration'] = $estimated_duration > 0 ? convertSecondToTime($estimated_duration, 'allhourmin') : '';
		$effective_duration = $object->effective_duration + $object->total_effective_duration;
		if (isset($children_total_time_spent['total_effective_duration'])) {
			$effective_duration += $children_total_time_spent['total_effective_duration'];
		}
		$resarray[$array_key . '_effective_duration'] = $effective_duration > 0 ? convertSecondToTime($effective_duration, 'allhourmin') : '';
		$reported_progress = (isset($children_total_time_spent['total_reported_progress']) && $children_total_time_spent['total_reported_progress'] > 0) ? ($can_set_times && isset($object->reported_progress) ? ($object->reported_progress + $children_total_time_spent['total_reported_progress']) / 2 : $children_total_time_spent['total_reported_progress']) : $object->reported_progress;
		$resarray[$array_key . '_reported_progress'] = $reported_progress > 0 ? $reported_progress . ' %' : '';
		$resarray[$array_key . '_calculated_progress'] = $estimated_duration > 0 ? ($effective_duration > 0 ? round($effective_duration / $estimated_duration * 100, 2) : 0) . ' %' : '';
		$effective_cost = $object->effective_cost ;
		if (isset($children_total_time_spent['total_effective_cost'])) {
			$effective_cost += $children_total_time_spent['total_effective_cost'];
		}
		$resarray[$array_key . '_effective_cost'] = $effective_cost > 0 ? price($effective_cost, '', $outputlangs, 1, 0, 0, $conf->currency) : '';
		$resarray[$array_key . '_date_operation'] = dol_print_date($object->date_operation, 'day');
		$resarray[$array_key . '_date_deadline'] = dol_print_date($object->date_deadline, 'day');
		$resarray[$array_key . '_date_resolved'] = dol_print_date($object->date_resolved, 'day');
		$resarray[$array_key . '_date_cloture'] = dol_print_date($object->date_cloture, 'day');
		$resarray[$array_key . '_status'] = $object->getLibStatut();
		$resarray[$array_key . '_status_type'] = $object->getLibStatut(12);
		$resarray[$array_key . '_status_and_status_type'] = $object->getLibStatut(13);
		$to_print = array();
		foreach ($object->requester_list as $requester) {
			$to_print[] = $requester->getFullName($outputlangs, 1);
		}
		$resarray[$array_key . '_requesters_fullname'] = implode(', ', $to_print);
		$to_print = array();
		foreach ($object->watcher_list as $watcher) {
			$to_print[] = $watcher->getFullName($outputlangs, 1);
		}
		$resarray[$array_key . '_watchers_fullname'] = implode(', ', $to_print);
		$to_print = array();
		foreach ($object->assigned_user_list as $assigned_user) {
			$to_print[] = $assigned_user->getFullName($outputlangs, 1);
		}
		$resarray[$array_key . '_assigned_users_fullname'] = implode(', ', $to_print);
		$to_print = array();
		foreach ($object->assigned_usergroup_list as $assigned_usergroup) {
			$to_print[] = $assigned_usergroup->name;
		}
		$resarray[$array_key . '_assigned_usergroups_fullname'] = implode(', ', $to_print);

		return $resarray;
	}
}
