<?php
/*  Copyright (C) 2018      Open-DSI             <support@open-dsi.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *	\file       htdocs/requestmanager/core/triggers/interface_99_modRequestManager_RequestManager.class.php
 *  \ingroup    requestmanager
 *	\brief      File of class of triggers for requestmanager module
 */


require_once DOL_DOCUMENT_ROOT.'/core/triggers/dolibarrtriggers.class.php';


/**
 *  Class of triggers for requestmanager module
 */
class InterfaceRequestManager extends DolibarrTriggers
{
	public $family = 'requestmanager';
	public $description = "Triggers of this module catch triggers event for RequestManager module.";
	public $version = self::VERSION_DOLIBARR;
	public $picto = 'technic';


	/**
	 * Function called when a Dolibarrr business event is done.
	 * All functions "runTrigger" are triggered if file is inside directory htdocs/core/triggers or htdocs/module/code/triggers (and declared)
	 *
	 * @param string		$action		Event action code
	 * @param Object		$object     Object
	 * @param User		    $user       Object user
	 * @param Translate 	$langs      Object langs
	 * @param conf		    $conf       Object conf
	 * @return int         				<0 if KO, 0 if no triggered ran, >0 if OK
	 */
	public function runTrigger($action, $object, User $user, Translate $langs, Conf $conf)
    {
        if (!isModEnabled('requestmanager')) return 0;     // Module not active, we do nothing

        switch ($action) {
	        // Contracts
	        case 'CONTRACT_CREATE':
		        if (!empty($conf->global->REQUESTMANAGER_TIMESLOTS_ACTIVATE)) {
			        dol_include_once('/requestmanager/lib/requestmanagertimeslots.lib.php');
			        $res = requestmanagertimeslots_get_periods($object->array_options['options_rm_timeslots_periods']);
			        if (!is_array($res)) {
				        $this->errors[] = $langs->trans('RequestManagerTimeSlotsPeriodsName') . ': ' . $res;
				        return -1;
			        }
		        }

		        dol_syslog("Trigger '" . $this->name . "' for action '$action' launched by " . __FILE__ . ". id=" . $object->id);
		        break;
			// Company
			case 'COMPANY_MODIFY':
				// Merge case
				if (!empty($object->context['merge']) && $object->context['mergefromid'] > 0) {
					$soc_id_src = (int)$object->context['mergefromid'];
					$soc_id_dst = (int)$object->id;
					$sqls = array(
						"UPDATE " . MAIN_DB_PREFIX . "requestmanager SET fk_soc = " . $soc_id_dst . " WHERE fk_soc = " . $soc_id_src,
						"UPDATE " . MAIN_DB_PREFIX . "societe_rm_usergroup_in_charge SET fk_soc = " . $soc_id_dst . " WHERE fk_soc = " . $soc_id_src,
					);

					foreach ($sqls as $sql) {
						$resql = $this->db->query($sql);
						if (!$resql) {
							$this->errors[] = 'Error ' . $this->db->lasterror();
							dol_syslog(__METHOD__ . " SQL: " . $sql . "; Error: " . $this->db->lasterror(), LOG_ERR);
							return -1;
						}
					}
				}
				dol_syslog("Trigger '" . $this->name . "' for action '$action' launched by " . __FILE__ . ". id=" . $object->id);
				break;
	        // Projects
	        case 'PROJECT_CREATE':
		        $origin = GETPOST('origin', 'alpha');
		        $originid = GETPOST('originid', 'int');

		        if ($origin && $originid) {
			        $ret = $object->add_object_linked($origin, $originid);
			        if (!$ret) {
				        $this->errors[] = $langs->trans('RequestManagerErrorAddLinkToRequest', $origin, $originid) . ': ' . $this->db->lasterror();
				        return -1;
			        }
		        }

		        dol_syslog("Trigger '" . $this->name . "' for action '$action' launched by " . __FILE__ . ". id=" . $object->id);
		        break;
	        case 'PROJECT_DELETE':
		        // Delete linked object
				$res = $object->deleteObjectLinked();
				if ($res < 0) return -1;

		        dol_syslog("Trigger '" . $this->name . "' for action '$action' launched by " . __FILE__ . ". id=" . $object->id);
		        break;
			case 'INVITATIONUSER_DELETE':
				// Delete linked request
				$sql = "DELETE FROM " . MAIN_DB_PREFIX . "requestmanager_quest_invitation_user WHERE `fk_quest_invitation_user` = " . $object->id;

				$resql = $this->db->query($sql);
				if (!$resql) {
					$this->errors[] = 'Error ' . $this->db->lasterror();
					dol_syslog(__METHOD__ . " SQL: " . $sql . "; Error: " . $this->db->lasterror(), LOG_ERR);
					return -1;
				}

				dol_syslog("Trigger '" . $this->name . "' for action '$action' launched by " . __FILE__ . ". id=" . $object->id);
				break;
			case 'ACTION_CREATE':
				// Management of the creation of the same event for x requester and/or each request children
				//---------------------------------------------------------------------------------------------
				if ($object->elementtype == "requestmanager" && $object->fk_element > 0) {
					$request = new RequestManager($object->db);
					$request->fetch($object->fk_element);

					$create_same_for_each_requester = $request->isRequestTypeOption('display_events_create_same_for_each_requester') && GETPOST('create_same_for_each_requester', 'aZ09');
					$create_same_for_each_request_children = $request->isRequestTypeOption('display_events_create_same_for_each_request_children') && GETPOST('create_same_for_each_request_children', 'aZ09');

					if (empty($object->context['requestmanager_create_same_event']) && (!empty($create_same_for_each_requester) || !empty($create_same_for_each_request_children))) {
						// Cancel last event created
						$transaction_opened = $object->db->db->transaction_opened;
						$object->db->db->transaction_opened = 1;
						$object->db->rollback();
						$object->db->begin();
						$object->db->db->transaction_opened = $transaction_opened;

						if (!empty($create_same_for_each_request_children)) {
							$request->fetch_children_request(null, 1, false);
							$request_list = $request->children_request_list;
						} else {
							$request_list = array($request);
						}

						if (!empty($request_list)) {
							foreach ($request_list as $request) {
								$socpeople_assigned_list = array();
								if (!empty($create_same_for_each_requester)) {
									foreach ($request->requester_ids as $requester_id) {
										$socpeople_assigned_list[][$requester_id] = array('id' => $requester_id);
									}
								} else {
									foreach ($request->requester_ids as $requester_id) {
										$socpeople_assigned_list[$requester_id] = array('id' => $requester_id);
									}
									$socpeople_assigned_list = array($socpeople_assigned_list);
								}

								if (!empty($socpeople_assigned_list)) {
									foreach ($socpeople_assigned_list as $socpeople_assigned) {
										$event = clone $object;
										$event->id = null;
										$event->context['requestmanager_create_same_event'] = true;

										$event->fk_element = $request->id;
										$event->socid = $request->socid;
										$event->socpeopleassigned = $socpeople_assigned;
										$event->contactid = key($socpeople_assigned);

										$result = $event->create($user);
										if ($result < 0) {
											$this->error = $event->error;
											$this->errors = $event->errors;
											return -1;
										}
									}
								}
							}

							$object->id = $event->id;
						}
					}

					dol_syslog("Trigger '" . $this->name . "' for action '$action' launched by " . __FILE__ . ". id=" . $object->id);
					break;
				}
        }

        // Management the edition of the extrafields by status of the request
        //----------------------------------------------------------------------
        if (!empty($conf->global->REQUESTMANAGER_DISPLAY_EXTRAFIELDS_ACTIVATE)) {
            $langs->load('requestmanager@requestmanager');
            dol_include_once('/requestmanager/class/html.formrequestmanager.class.php');
            $formrequestmanager = new FormRequestManager($this->db);

            $element_types = $formrequestmanager->element_type_array();
            $element_types = array_keys($element_types);

            if (in_array($object->table_element, $element_types) && (preg_match('/_CREATE$/', $action) || preg_match('/_MODIFY$/', $action) || preg_match('/_UPDATE$/', $action))) {
				if ($object->element == 'requestmanager') {
					/**
					 * @var RequestManager $object
					 */
					if (!empty($conf->global->REQUESTMANAGER_DISPLAY_EXTRAFIELDS_ON_REQUEST_CHILDREN) && $object->fk_parent > 0) {
						$object->fetch_parent();
						$linkedObjects['requestmanager'] = [$object->parent];
						$linkedObjectsIds['requestmanager'] = [$object->parent->id];
					} else {
						$linkedObjects['requestmanager'] = [$object];
						$linkedObjectsIds['requestmanager'] = [$object->id];
					}
				} else {
					$linkedObjectsIds = array();
					$linkedObjects = array();
					if ($object->element == 'project_task') {
						if (preg_match('/_CREATE$/', $action)) {
							$project_id = GETPOST('id', 'int');
						} else {
							$project_id = $object->fk_project;
						}
						if ($project_id > 0) {
							require_once DOL_DOCUMENT_ROOT . '/projet/class/project.class.php';
							$proj = new Project($this->db);
							$proj->fetch($project_id);
							$proj->fetchObjectLinked();
							$linkedObjectsIds = $proj->linkedObjectsIds;
							$linkedObjects = $proj->linkedObjects;
						}
					} elseif ($object->id > 0) {
						$object->fetchObjectLinked();
						$linkedObjectsIds = $object->linkedObjectsIds;
						$linkedObjects = $object->linkedObjects;
					}

					$origin = GETPOST('origin', 'alpha');
					$originid = GETPOST('originid', 'int');
					if ($origin == 'requestmanager' && $originid > 0 &&
						(empty($linkedObjectsIds['requestmanager']) || !in_array($originid, $linkedObjectsIds['requestmanager']))
					) {
						dol_include_once('/requestmanager/class/requestmanager.class.php');
						$requestmanager_tmp = new RequestManager($this->db);
						$requestmanager_tmp->fetch($originid);
						$linkedObjects['requestmanager'] = [$requestmanager_tmp];
						$linkedObjectsIds['requestmanager'][] = $originid;
					}
				}

				global $extrafields, $extrafields_task;
				$ef = &$extrafields;
				if ($object->table_element == 'projet_task' && is_object($extrafields_task)) {
					$ef = &$extrafields_task;
				}

				$element_table = $object->table_element;
				$extra_field_management = array();
				if (!empty($linkedObjects['requestmanager'])) {
					foreach ($linkedObjects['requestmanager'] as $request) {
						/**
						 * @var RequestManager $request
						 */
						$tmp = $request->getExtraFieldManagement();
						foreach ($tmp as $element_type => $element_type_list) {
							if ($element_table != $element_type) continue;

							foreach ($element_type_list as $extra_field_key => $extra_field_info) {
								if (!isset($extra_field_management[$extra_field_key])) {
									$extra_field_management[$extra_field_key] = array(
										'show' => false,
										'mandatory' => false,
										'readonly' => false,
									);
								}
								$extra_field_management[$extra_field_key]['show'] |= !empty($extra_field_info['show']);
								$extra_field_management[$extra_field_key]['mandatory'] |= !empty($extra_field_info['mandatory']);
								$extra_field_management[$extra_field_key]['readonly'] |= !empty($extra_field_info['readonly']);
							}
						}
					}
				}

				if (!empty($extra_field_management)) {
					$error = 0;
					$restore = false;
					foreach ($ef->attributes[$element_table]['label'] as $k => $l) {
						// Separator
						if ($ef->attributes[$object->table_element]['type'][$k] == 'separate') continue;

						$perms = isset($ef->attributes[$element_table]['editperms'][$k]) ? dol_eval($ef->attributes[$element_table]['editperms'][$k], 1) : 1;
						$perms = $perms && empty($extra_field_management[$k]['readonly']) ? 1 : 0;
						$edition = !empty($extra_field_management[$k]['show']) && empty($extra_field_management[$k]['readonly']) ? 1 : 0;
						$mandatory = !empty($extra_field_management[$k]['mandatory']);

						// Restore old value
						if (!$perms || !$edition) {
							$old_value = isset($object->oldcopy->array_options['options_' . $k]) ? $object->oldcopy->array_options['options_' . $k] : (preg_match('/_CREATE$/', $action) ? '' : $object->array_options['options_' . $k]);
							if ($object->array_options['options_' . $k] != $old_value &&
								(!isset($object->context['rm_force_extra_fields_update']) || !is_array($object->context['rm_force_extra_fields_update']) || !in_array($k, $object->context['rm_force_extra_fields_update']))
							) {
								$object->array_options['options_' . $k] = $old_value;
								$restore = true;
							}
						}
						// Check mandatory
						elseif ($mandatory) {
							$fail_check = false;
							if ($ef->attributes[$object->table_element]['type'][$k] == 'link' && $object->array_options['options_' . $k] == '-1') $fail_check = true;
							if ($object->array_options['options_' . $k] === '' || !isset($object->array_options['options_' . $k])) $fail_check = true;
							if ($fail_check) {
								if (!empty($ef->attributes[$element_table]['langfile'][$k])) {
									$langs->load($ef->attributes[$element_table]['langfile'][$k]);
								}
								$this->errors[] = $langs->trans('ErrorFieldRequired', $langs->transnoentitiesnoconv($l));
								$error++;
							}
						}
					}
					if ($error) {
						return -1;
					} elseif ($restore) {
						$object->insertExtraFields();
					}
				}
            }
        }

        return 0;
    }
}