<?php
/* Copyright (C) 2005-2013  Laurent Destailleur     <eldy@users.sourceforge.net>
 * Copyright (C) 2017       Open-DSI                <support@open-dsi.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *	\file       htdocs/module/lib/opendsi_common.lib.php
 * 	\ingroup	module
 *	\brief      Common functions opendsi for the module
 */

if (!function_exists('opendsi_common_getChangeLog')) {
	/**
	 * Gives the changelog. First check ChangeLog-la_LA.md then ChangeLog.md
	 *
	 * @param string $moduleName Name of module
	 *
	 * @return  string                              Content of ChangeLog
	 */
	function opendsi_common_getChangeLog($moduleName)
	{
		global $langs;
		$langs->load("admin");

		include_once DOL_DOCUMENT_ROOT . '/core/lib/files.lib.php';
		include_once DOL_DOCUMENT_ROOT . '/core/lib/geturl.lib.php';

		$filefound = false;

		$modulePath = dol_buildpath('/' . strtolower($moduleName), 0);

		// Define path to file README.md.
		// First check README-la_LA.md then README.md
		$pathoffile = $modulePath . '/ChangeLog-' . $langs->defaultlang . '.md';
		if (dol_is_file($pathoffile)) {
			$filefound = true;
		}
		if (!$filefound) {
			$pathoffile = $modulePath . '/ChangeLog.md';
			if (dol_is_file($pathoffile)) {
				$filefound = true;
			}
		}

		$content = '';

		if ($filefound) {    // Mostly for external modules
			$moduleUrlPath = dol_buildpath('/' . strtolower($moduleName), 1);
			$content = file_get_contents($pathoffile);

			if ((float) DOL_VERSION >= 6.0) {
				@include_once DOL_DOCUMENT_ROOT . '/core/lib/parsemd.lib.php';
				$content = dolMd2Html($content, 'parsedown', array('doc/' => $moduleUrlPath . '/doc/'));
			} else {
				$content = opendsi_common_dolMd2Html('codenaf', $content, 'parsedown', array('doc/' => $moduleUrlPath . '/doc/'));
			}
		}

		return $content;
	}
}

if (!function_exists('opendsi_common_dolMd2Html')) {
	/**
	 * Function to parse MD content into HTML
	 *
	 * @param string $moduleName Name of module
	 * @param string $content MD content
	 * @param string $parser 'parsedown' or 'nl2br'
	 * @param string $replaceimagepath Replace path to image with another path. Exemple: ('doc/'=>'xxx/aaa/')
	 *
	 * @return    string                          Parsed content
	 */
	function opendsi_common_dolMd2Html($moduleName, $content, $parser = 'parsedown', $replaceimagepath = null)
	{
		if (is_array($replaceimagepath)) {
			foreach ($replaceimagepath as $key => $val) {
				$keytoreplace = '](' . $key;
				$valafter = '](' . $val;
				$content = preg_replace('/' . preg_quote($keytoreplace, '/') . '/m', $valafter, $content);
			}
		}

		if ($parser == 'parsedown') {
			dol_include_once('/' . strtolower($moduleName) . '/includes/parsedown/Parsedown.php');
			$Parsedown = new Parsedown();
			$content = $Parsedown->text($content);
		} else {
			$content = nl2br($content);
		}

		return $content;
	}
}

if (!function_exists('opendsi_execute_sql_file')) {
	/**
	 *    Launch a sql file.
	 *
	 * @param string $sqlfile Full path to sql file
	 * @param int $entity Entity targeted for multicompany module
	 * @return  int                             <0 if KO, >0 if OK
	 */
	function opendsi_execute_sql_file($sqlfile, $entity = null)
	{
		global $conf, $db;

		$sql = '';

		$fp = fopen($sqlfile, "r");
		if ($fp) {
			while (!feof($fp)) {
				$buf = fgets($fp, 4096);

				// Add line buf to buffer if not a comment
				if (!preg_match('/^--/', $buf) && trim($buf) != "") {
					$buf = preg_replace('/--.*$/', '', $buf); //remove comment from a line that not start with -- before add it to the buffer
					$sql .= trim($buf) . " ";
				}
			}

			fclose($fp);
		} else {
			dol_syslog("opendsi_common.lib.php::opendsi_execute_sql_file failed to open file " . $sqlfile, LOG_ERR);
		}

		// Replace the prefix tables
		if (MAIN_DB_PREFIX != 'llx_') {
			$sql = preg_replace('/llx_/i', MAIN_DB_PREFIX, $sql);
		}
		// Replace the entity
		$sql = preg_replace('/__ENTITY__/i', (!empty($entity) ? $entity : $conf->entity), $sql);

		$resql = $db->query($sql);
		if ($resql)
			return 1;
		else
			return -1;
	}
}
